/***************************************************************************
                            PackageSelect.cpp
                             -------------------                                         
    begin                : Fri Oct 29 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/



#include <alloca.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <newt.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qfont.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qtooltip.h>


#include "log.h"
#include "windows.h"
#include "smp.h"

#include "language.h"
#include "PackageSelect.h"
#include "mountpoint.h"
#include "packagelistitem.h"
#include "installselect.h"

#include <fstream.h>

extern QString GBLI_DIR;

PackageSelect::PackageSelect(	QWidget* parent, InstallSelect *a_install, const char* name ):
	PackageSelectData( parent, name ), install( a_install ),
	initialized( false ), blManualSourceMount( false )
{
	QFont font( QFont::defaultFont() );
//	font.setFamily( "times" );
	font.setPointSize( Language::iSmallFontSize );
	viewPackages->setFont( font );
}

PackageSelect::~PackageSelect()
{
	int res = -1;
//	if( blManualSourceMount )
		res = system( ("umount "+ INSTALL_SOURCE +" >/dev/null 2>&1").latin1() );
#ifndef GBLI_DEVEL
	if( !res && INSTALL_SOURCE.right( 5 ) != "/data" )
		system( ("rm -rf "+ INSTALL_SOURCE).latin1() );
#endif
}

static int pkgCompare( const void * first, const void * second )
{
    packageInfo ** a = (packageInfo **)first;
    packageInfo ** b = (packageInfo **)second;

    // put packages w/o names at the end
    if (!(*a)->name) return 1;
    if (!(*b)->name) return -1;

    return strcasecmp( (*a)->name, (*b)->name );
}

void PackageSelect::showEvent( QShowEvent * )
{
	bool changedCursorHere = false;
	if( !qApp->overrideCursor() ){
	 	qApp->setOverrideCursor( Qt::waitCursor );
		changedCursorHere = true;
	}

	if(!InstallSelect::blUpdate)	
		displayFreeSpace();
	else
	{
		lcdFreeSpace->hide();
		qtarch_Label_6FreeSpace->hide();
	}

	if( !initialized){
  	install->loadData();

    QFile FGroups( INSTALL_SOURCE + "/software/base/groups_convert" );
    if( FGroups.open( IO_ReadOnly ) ){
      QTextStream TSGroups( &FGroups );
  		QString temp, key, *value;
  		int pos, posPrev;
  		while( (temp = TSGroups.readLine()).length() ){
        if( temp.simplifyWhiteSpace().left( 1 ) == "#" ) continue;
        pos = posPrev = 0;
        pos = temp.find( ',', pos );
        key = temp.mid( posPrev, pos-posPrev ).simplifyWhiteSpace();
				key.replace( QRegExp( "\\\"" ), "" );
        posPrev = ++pos;
        value = new QString( temp.mid( posPrev, temp.length()-posPrev ).simplifyWhiteSpace() );
				value->replace( QRegExp( "\\\"" ), "" );
				if( value->at( value->length()-1 ) == ',' )
					value->replace( value->length()-1, 1, "" );
//printf( "%s - %s\n", key.ascii(), value->ascii() );

  			DGroups.insert( key, value );
      }
      FGroups.close();
    }

#ifndef GBLI_STANDALONE
		install->selectPackages(); // select set packages
#else
		if( install->cs.numComponents ) // packages from CD
			mirrorSystemPackagesSelectionToList();
		else{
			for( int i = 0; i < install->ps.numPackages; i++ )
				install->ps.packages[i]->selected = true;
		}

// TODO: should we make normal "real" free space indication?
// But how to calculate it? Not easy task...
  	qtarch_Label_6FreeSpace->hide();
		qtarch_Label_5NeededSpace->hide();
		lcdNeededSpace->hide();
		lcdFreeSpace->hide();
#endif

  	insertItems();
#ifndef GBLI_STANDALONE
  	initialized = true;
#else
		if(!InstallSelect::blUpdateNP)
	  	initialized = true;
#endif
//printf( "my test\n" );
//		QLayoutIterator qli( layout()->iterator() );
//		QLayoutItem *item;
//		while( (item = qli.current() ) ){
//			item->setEnabled( false );
//			++qli;
//		}
//printf( "my test1\n" );
	}else{
#ifndef GBLI_STANDALONE
		install->selectPackages(); // select set packages
		initializeSelection();
#else
		initializeSelection( false );
#endif
	}

	if( !install->ps.numPackages ){
		viewPackages->setEnabled( false );
		bttnCheckDeps->setEnabled( false );
		bttnStartUpdate->setEnabled( false );
	}

	if( changedCursorHere )
	 	qApp->restoreOverrideCursor();
}

int PackageSelect::mirrorSystemPackagesSelectionToList(){
  rpmdb db = NULL;

  if( rpmdbOpen( NEWROOT.ascii(), &db, O_RDONLY, 0644 ) ){
		Log::logMain(32, "Failed to open rpmdb to get the list of packages!\n" );
		return INST_ERROR;
	}

	bool changedCursorHere = false;
	if( !qApp->overrideCursor() ){
	 	qApp->setOverrideCursor( Qt::waitCursor );
		changedCursorHere = true;
	}

	char *name;
	struct packageInfo **pack;
	struct packageInfo key;
	struct packageInfo *keyaddr = &key;
	Header h = NULL;
	rpmdbMatchIterator mi;
	int count = 0;    	

	mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, NULL, 0);
	while( (h = rpmdbNextIterator(mi)) != NULL ){
		count++;
		headerGetEntry( h, RPMTAG_NAME, NULL, (void **) &name, NULL );
   	key.name = name;
   	pack = (struct packageInfo **) bsearch( &keyaddr, install->ps.packages, install->ps.numPackages,
              						    	sizeof(*(install->ps.packages)), pkgCompare );
		if( pack ){
			(*pack)->selected = true;
		}else{
//			printf( "name not found: %s\n", name );
		}
  }
	rpmdbFreeIterator(mi);

	rpmdbClose( db );

	if( changedCursorHere )
	 	qApp->restoreOverrideCursor();

	return INST_OK;
}

int PackageSelect::findInstallSource()
{
	if( install && install->ps.numPackages ) return INST_OK; // already loaded smth!
	
	if( !QDir( INSTALL_SOURCE ).exists() )
		::mkdir( INSTALL_SOURCE.latin1(), 0755 );

	QFile checkFile( INSTALL_SOURCE +"/SOT-PGP-KEY" );
	bool found = checkFile.exists();
	int	res = 0;

	if( !found ){
		res = system( "grep iso9660 /etc/mtab >/dev/null 2>&1" );
		if( res ){// not found in mtab, but mtab could be old
			res = system( "grep iso9660 /proc/mounts >/dev/null 2>&1" );
			if( !res ) system( "cat /proc/mounts > /etc/mtab" );// let's update too old file
		}
		if( !res ){// some mount found
      FILE *mtab = fopen( "/etc/mtab", "r" );
      if( mtab ){
        struct mntent *mnt = getmntent( mtab );
        while( mnt ){
      	  if( !strcmp( mnt->mnt_type, "iso9660" ) ){
						found = QFile( QString::fromLatin1(mnt->mnt_dir) +"/SOT-PGP-KEY" ).exists();
						if( found ){// correct CD, but wrong mount point
							char buf[64];
							sprintf( buf, "umount %s", mnt->mnt_dir );
							system( buf );
							sprintf( buf, "mount %s %s -tiso9660 -oro", mnt->mnt_fsname, INSTALL_SOURCE.latin1() );
							system( buf );
							break;
						}
					}
      	  mnt = getmntent(mtab);
      	}
        fclose( mtab );
      }
			found = checkFile.exists();
		}

  	if( !found ){
  		res = QMessageBox::information( this, tr("Insert SOT Linux Installation CD "),
  	 		tr("Please, insert \"SOT Linux Installation CD\" into\n"
  				 "your CD-ROM drive and click Ok in order to proceed\n"
  				 "with update!" ),
   			QMessageBox::Ok, QMessageBox::Cancel );
  		if( res != QMessageBox::Ok )
			{
				InstallSelect::blUpdateNP = true;
				return INST_CANCEL;
			}
			found = checkFile.exists();
  		if( !found ){ // user could insert CD and mount it somethere - stupid ;(
        FILE *mtab = fopen( "/etc/mtab", "r" );
        if( mtab ){
          struct mntent *mnt = getmntent( mtab );
          while( mnt ){
        	  if( !strcmp( mnt->mnt_type, "iso9660" ) ){
  						found = QFile( QString::fromLatin1(mnt->mnt_dir) +"/SOT-PGP-KEY" ).exists();
  						if( found ){// correct CD, but wrong mount point
  							char buf[64];
  							sprintf( buf, "umount %s", mnt->mnt_dir );
  							system( buf );
  							sprintf( buf, "mount %s %s -tiso9660 -oro", mnt->mnt_fsname, INSTALL_SOURCE.latin1() );
  							system( buf );
  							break;
  						}
  					}
        	  mnt = getmntent(mtab);
        	}
          fclose( mtab );
        }
  			found = checkFile.exists();
			}
  	}

		if( !found ){
			if( QFile( "/dev/cdrom" ).exists() ){
				res = system( ("mount /dev/cdrom "+ INSTALL_SOURCE +" -tiso9660 -oro").latin1() );
				if( !res ){ // succeeded
					found = checkFile.exists();
					if( !found ) system( ("umount "+ INSTALL_SOURCE).latin1() );
				}else{ // no CD at least in default drive
					system( ("umount "+ INSTALL_SOURCE).latin1() );
				}
			}

			if( !found ){ // let's search for device
  			QDir d( "/proc/ide/" );
  			if( d.exists() && d.isReadable() ){
    			d.setFilter( QDir::Dirs );
    			d.setNameFilter( "hd*" );
    			const QFileInfoList *list = d.entryInfoList();
    			QFileInfoListIterator it( *list ); // create list iterator
    			QFileInfo *fi;
    			while( (fi = it.current()) ){
  //  printf( "Drive: %s\n", QString( fi->absFilePath() + "/media" ).ascii() );
    				QFile fileMedia( fi->absFilePath() + "/media" );
    				if( fileMedia.exists() && fileMedia.open( IO_ReadOnly ) ){
    					QString str;
    					if( fileMedia.readLine( str, 6 ) != -1 && str == "cdrom" ){
								res = system( ("mount /dev/" +fi->fileName() +" "+ INSTALL_SOURCE +" -tiso9660 -oro").latin1() );
         				if( !res ){ // succeeded
         					found = checkFile.exists();
         					if( !found ) system( ("umount "+ INSTALL_SOURCE).latin1() );
         				}else{ // no CD
         					system( ("umount "+ INSTALL_SOURCE).latin1() );
								}
							}
	  					fileMedia.close();
							if( found ) break;
						}
	  				++it;
					}
				}
			}
		}
	}

	if( !found ){ // nothing found ;(((
   	res = QMessageBox::information( this, tr("SOT Linux CD not found"),
	 		tr("Failed to find SOT Linux CD!\n"
			"Update is not possible!" ),
 			QMessageBox::Ok );
		InstallSelect::blUpdateNP = true;
		return INST_ERROR;
	}

	InstallSelect::blUpdateNP = false;
	return INST_OK;
}

void PackageSelect::displayFreeSpace()
{
	MountPointList mps = MountPoints::allDirectories();

	MountPoint *mp;
	int iFreeSpace = 0;
	for( mp = mps.first(); mp != NULL; mp = mps.next() ){
    if( mp->name().ascii()[0] == '/' )// some good one
			iFreeSpace += (int)(mp->free() / 0x100000);
	}
  lcdFreeSpace->display( iFreeSpace );
}

void PackageSelect::slotSelectionChanged(class QListViewItem *a_item )
{
	int type, count;
	char *buf;
	descriptionTextArea->clear();
	lboxRequires->clear();
	if( ((PackageListItem*)a_item)->package ){
		QString str;
		if( headerGetEntry( ((PackageListItem*)a_item)->package->h, RPMTAG_VERSION, &type,
			( void **)&buf, &count ) )
			str = tr("Version: ") + QString::fromLatin1( buf );
		if(	headerGetEntry( ((PackageListItem*)a_item)->package->h, RPMTAG_DESCRIPTION, &type,
			( void **)&buf, &count ) )
			str = str +"\n"+ QString::fromLatin1( buf );
		descriptionTextArea->setText( str );

		dependenciesInfo *dep = NULL;
    if(	!install->deps.isEmpty() )
			dep = install->deps[ ((PackageListItem*)a_item)->package->pos ];
		QToolTip::remove( viewPackages );
		if( dep && dep->dependOn.count() ){
      QStringList::Iterator s;
			QString strRequires = tr( "Requires:" )+ "\n";
			int i = 0;
      for( s = dep->dependOn.begin(); s != dep->dependOn.end(); ++s ){
					if( i++ ) strRequires += ",\n";
					strRequires += *s;
//      	lboxRequires->insertItem( *s );
			}
			QToolTip::add( viewPackages, strRequires );
			descriptionTextArea->setFocus();
			viewPackages->setFocus();
		}else{
  		char **dataBufferName;
  		int rpmItemTypeName, itemCountName, tmp;
    	int statusBadName = headerGetEntry(((PackageListItem*)a_item)->package->h,RPMTAG_REQUIRENAME, &rpmItemTypeName,
    		( void **) &dataBufferName,  &itemCountName);
    	if( statusBadName && statusBadName != 1 )	return;
   		char **dataBufferVersion;
  		int rpmItemTypeVersion;
    	int statusBadVersion = headerGetEntry(((PackageListItem*)a_item)->package->h,RPMTAG_REQUIREVERSION, &rpmItemTypeVersion,
    		( void **) &dataBufferVersion, &tmp );
    	if( statusBadVersion && statusBadVersion != 1 )	return;   	
     	for( int ipt = 0; ipt < itemCountName ; ipt ++ ){
  			if( rpmItemTypeName == RPM_STRING_ARRAY_TYPE )
  			  if (dataBufferName)
  			    if (dataBufferName[ipt]){
  			      str = QString::fromLatin1( dataBufferName[ipt] );
  					}else
    					continue;
  			if( rpmItemTypeVersion == RPM_STRING_ARRAY_TYPE )
     		  if (dataBufferVersion)
     		    if (dataBufferVersion[ipt])
  					  str += " "+ QString::fromLatin1( dataBufferVersion[ipt] );
  			lboxRequires->insertItem( str );
  		}
  	}
	}
}

void PackageSelect::slotCurrentChanged(class QListViewItem *)
{
}

void PackageSelect::slotTotalChanged( int a_size )
{
	int newsize = a_size;
//printf( "size: %d\n", newsize );fflush( stdout );
	int i = ((unsigned int)newsize) >> 20;
	if( i < 0 ) i = 0;
	lcdNeededSpace->display( i );
	if(!InstallSelect::blUpdate)
		emit nextEnabled( this, i > 0 && i+3 < lcdFreeSpace->intValue() );
	else
	  emit nextEnabled( this, true );
}

void PackageSelect::slotToggleSelectionOfPackage(class QListViewItem * a_item )
{
	if( a_item->isSelectable() )
		((PackageListItem *)a_item)->setPackageSelected();
}

void PackageSelect::updateDependencies()
{
	if( !initialized ) return;
 	qApp->setOverrideCursor( Qt::waitCursor );

  // Lets always select "Base"
//#ifndef GBLI_STANDALONE
	if( !InstallSelect::blUpdate )
//#endif
	  for ( int i = 0; i < install->cs.base->ps.numPackages; i++)
  		install->cs.base->ps.packages[i]->selected = true;

	initializeSelection();

/*
	for(int i = 0; i < install->ps.numPackages; i++)
	{
		if(!strcmp(install->ps.packages[i]->name, "XFree86-xf86cfg"))
   	{
			install->verifyDependencies_(install->ps.packages[i], 0, 0);
//			install->verifyDependencies_();
			break;
		}
	}
*/
 	qApp->restoreOverrideCursor();
}

void PackageSelect::startUpdate()
{
//printf( "start update\n" );
 	int res = QMessageBox::Yes;
	if(install->isFinished()){
  	res = QMessageBox::warning( this, tr("Update?"),
   		tr("You have updated your system recently.\n"
  		"Do you want to repeat the operation?" ),
   		QMessageBox::Yes, QMessageBox::No );
	}else{
  	if( !QFile( NEWROOT +"/etc/sot-release" ).exists() && !QFile( NEWROOT +"/etc/lsb-release").exists() )
    	res = QMessageBox::warning( this, tr("No Best/SOT Linux found"),
     		tr("Failed to determine the Best/SOT Linux version of your system.\n"
    		"If your system is not SOT Linux then update can destroy it!\n"
				"Do you want to continue with the update?" ),
     		QMessageBox::Yes, QMessageBox::No );

/*		if( res == QMessageBox::Yes )
  	res = QMessageBox::warning( this, tr("Update?"),
   		tr("Do you really want to update your system packages?\n"
  		"Updating of the system is a long process which afterall\n"
  		"could require additional reconfiguration of the system!" ),
   		QMessageBox::Yes, QMessageBox::No );*/
	}
 	if( res != QMessageBox::Yes )
 		return;

	viewPackages->setEnabled( false );
	bttnCheckDeps->setEnabled( false );
	bttnStartUpdate->setEnabled( false );

	//Added by dk to avoid the duplicating info in modules.conf
	//Moved here from previous place to comment the old contents
	//only if updating system is launched. (22.03.2002)dk.
	if (QFile::exists(QString(NEWROOT + "/etc/modules.conf")))
	{
		::system("cat "+ NEWROOT + "/etc/modules.conf| while read LINE; do echo \"#$LINE\" >> testtt.tmp; done; mv testtt.tmp "+ NEWROOT + "/etc/modules.conf");
		#ifdef TESTCOMMENT
		::system("echo \"#\" >>"+ NEWROOT+ "/etc/modules.conf");
		#endif
	}

	emit updateStarted();
}

void PackageSelect::updateFinished()
{
	viewPackages->setEnabled( true );
	bttnCheckDeps->setEnabled( true );
	bttnStartUpdate->setEnabled( true );
	mirrorSystemPackagesSelectionToList();
	initializeSelection( false );
}

void PackageSelect::insertItems()
{
	viewPackages->clear();
	if( !install->ps.numPackages ) return; // absolutely nothing to insert

	viewPackages->setSorting( -1 );
	root = new PackageListItem( viewPackages, &(install->ps) );
	connect( root, SIGNAL( totalChanged(int) ), this, SLOT( slotTotalChanged(int) ) );

	QListViewItem* tmpItem;

	PackageListItem *gr;

	if( install->cs.base ){
		component *comps = install->cs.base;
	 	QString group( QString::fromLatin1( "System/Base" ) );
		gr = findAppropriateGroup( group );
  	if( gr ){
     	for( int k = 0; k < comps->ps.numPackages; k++ ){
     		packageInfo *package = comps->ps.packages[k];
  			comps->ps.packages[k]->iBase = 1;
     		tmpItem = new PackageListItem(
     			gr,	QString::fromLatin1( package->name ),	package	);
  			if( !comps->ps.packages[k]->mustBe && !InstallSelect::blUpdate )
  				Log::logMain(32, "Add to comps: %s\n", comps->ps.packages[k]->name );
     	}
  	}
	}
	int count = 0;
	for( int i = 0; i < install->ps.numPackages; i++ ){
//printf( "%d", install->ps.packages[i]->iBase );
			if( install->ps.packages[i]->iBase ) continue; // Base is already added
//		if( !install->ps.packages[i]->inmenu ){
//printf( "group: %s\n", install->ps.packages[i]->group );
    	QString group( QString::fromLatin1( install->ps.packages[i]->group ) );
			QString *newgroup = DGroups.find( group );
			if( newgroup ){
				QString str = *newgroup;
				gr = findAppropriateGroup( str );
//				free( install->ps.packages[i]->group );
//				install->ps.packages[i]->group = (char*)malloc( newgroup->length() + 1 );
//				if( install->ps.packages[i]->group )
//					delete install->ps.packages[i]->group;
				install->ps.packages[i]->group = new char[ newgroup->length() + 1 ];
				strcpy( install->ps.packages[i]->group, newgroup->ascii() );
			}else{
//printf( "Using old group: \"%s\" for package \"%s\"\n", group.ascii(), install->ps.packages[i]->name );
				gr = findAppropriateGroup( group );
				count++;
			}
			if( !gr ) gr = root;

  		tmpItem = new PackageListItem(
  			gr,	QString::fromLatin1( install->ps.packages[i]->name ),	install->ps.packages[i]	);
//		}
	}
Log::logMain(32, "Total not converted or already new groups: %d\n", count );
	viewPackages->setSorting( 0 );
	initializeSelection();
	root->setOpen( true );
}

PackageListItem * PackageSelect::findAppropriateGroup( QString& a_group )
{
	PackageListItem *tmpItem = root;
	if( tmpItem->text(0) == a_group )
		return root;
	int pos = 0;
	int prevpos = 0;
	PackageListItem *prevItem;
	do{
		pos = a_group.find( "/", pos );
		QString cur = a_group.mid( prevpos, (pos!=-1?pos:a_group.length()) - prevpos );
		prevpos = ++pos;
//printf( "cur: %s\n", cur.ascii() );
		prevItem = tmpItem;
		tmpItem = (PackageListItem*)tmpItem->firstChild();
		while( tmpItem ){
//printf( "tmpItem: %s\n", tmpItem->text(0).ascii() );
			if( !tmpItem->text(0).compare( cur ) ){
				prevItem = tmpItem;
				break;
			}
			tmpItem = (PackageListItem*)tmpItem->nextSibling();
		}
		if( !tmpItem )
			prevItem = tmpItem = new PackageListItem( prevItem, cur );
	}while( pos > 0 );
	return prevItem;
}

void PackageSelect::initializeSelection( bool blDepsCheck )
{
//printf( "init selection\n" );
	bool changedCursorHere = false;
	if( !qApp->overrideCursor() ){
	 	qApp->setOverrideCursor( Qt::waitCursor );
		changedCursorHere = true;
	}

	if( blDepsCheck ){
#ifdef GBLI_STANDALONE
		if( initialized )
#endif
			install->verifyDependencies_();
	}
	for( int i = 0; i < install->ps.numPackages; i++ ){ // reflect selection
		if( install->ps.packages[i]->item )
		{
//			Log::logMain(32, "name: %s; selected: %d\n", install->ps.packages[i]->name, install->ps.packages[i]->selected);
			install->ps.packages[i]->item->setPackageSelected(
				install->ps.packages[i]->selected, false );
		}
	}
	int i = 0;
	root->initSize( &i, &i, &i );

	if( changedCursorHere )
	 	qApp->restoreOverrideCursor();
}

