/***************************************************************************
                             diskpartitioner.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "diskpartitioner.h"

#include <qlayout.h>
#include <qpushbutton.h>

/*A Disk Partitioner provides a user interface around a partition
  view. It allows the user to partition the disk with a bunch of push
  buttons. Furthermore it provides some general information about the
  disk in an information label. */
DiskPartitioner::DiskPartitioner( QWidget *parent, const char *name )
    : GBLIPage( parent, name)
{
    QVBoxLayout* topVbox = new QVBoxLayout( this, 6);

    view = new PartitionView(this);
    topVbox->addWidget( view, 10 );

    QHBoxLayout* hbox2 = new QHBoxLayout( 6 );
    topVbox->addLayout( hbox2 );

    QPushButton* button;

    QVBoxLayout* vbox1 = new QVBoxLayout( 6 );
    hbox2->addLayout( vbox1 );

    button = new QPushButton( this );
    button->setText( tr("&Edit") );
    connect(view, SIGNAL( editAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( editCurrent() ) );
    button->setEnabled( FALSE );
    vbox1->addWidget( button );

    button = new QPushButton( this );
    button->setText( tr("&Add logical") );
    connect( button, SIGNAL( clicked() ), view, SLOT( addLogicalPartition() ) );
    connect(view, SIGNAL( diskAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    vbox1->addWidget( button );

    QVBoxLayout* vbox2 = new QVBoxLayout( 6 );
    hbox2->addLayout( vbox2 );

    button = new QPushButton( this );
    button->setText( tr("&Delete") );
    connect(view, SIGNAL( deleteAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    connect( button, SIGNAL( clicked() ), view, SLOT( removeOrClearCurrent() ) );
    button->setEnabled( FALSE );
    vbox2->addWidget( button );

    button = new QPushButton( this );
    button->setText( tr("Re&size") );
    connect( button, SIGNAL( clicked() ), view, SLOT( Resize() ) );
    connect(view, SIGNAL( resizeAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    vbox2->addWidget( button );

//button->hide();
vbox2->addStretch( 1 );

    hbox2->addStretch( 10 );

    QVBoxLayout* vbox3 = new QVBoxLayout( 6 );
    hbox2->addLayout( vbox3 );

    writeButton = new QPushButton( this );
    writeButton->setText( tr("&Write") );
    connect(view, SIGNAL( changed(bool) ), writeButton, SLOT( setEnabled(bool) ) );
    connect(view, SIGNAL( changed(bool) ), this, SLOT( writeEnabled(bool) ) );
    connect( writeButton, SIGNAL( clicked() ), view, SLOT( writePartitionTable() ) );
    connect( writeButton, SIGNAL( clicked() ), this, SIGNAL( changePartitionTable() ) );
    writeButton->setEnabled( FALSE );
    vbox3->addWidget( writeButton );

    btnReset = new QPushButton( this );
    btnReset->setText( tr("&Reset") );
    connect( view, SIGNAL( notOriginal(bool) ), btnReset, SLOT( setEnabled(bool) ) );
    connect( btnReset, SIGNAL( clicked() ), view, SLOT( defaultPartitionTable() ) );
    btnReset->setEnabled( FALSE );
    vbox3->addWidget( btnReset );

    view->setFocus();
}


DiskPartitioner::~DiskPartitioner()
{
}


PartitionView* DiskPartitioner::getView()
{
    return view;
}


void DiskPartitioner::writeEnabled(bool b)
{
		/*Added by dk, 25.02.2002, to avoid crashing on partition overflow*/
		/*dk*/
    if (view->partitionOverflow)
			return;
		/*dk*/
    emit nextEnabled( this, !b );
		btnReset->clearFocus();
}


void DiskPartitioner::showEvent( QShowEvent * )
{
    view->checkState();
}
