/***************************************************************************
				                         grub.cpp
                             -------------------
    begin                : Tue Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Anisimov
    author email         : aa@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "grub.h"
#include "linlinehelp.h"
#include "partitionview.h"
#include "partitionitem.h"
#include "mountpoint.h"
#include "gblidebug.h"
#include "smp.h"
#include "log.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h> 

#include <alloca.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <stream.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <sys/utsname.h>
#include <stdio.h>
#include <qdir.h>
#include <qregexp.h>
#include <qlistview.h>

//---------------------------Initrd STUFF Here-------------------------
int GRUB::mkinitrd( const char *a_szPrefix, const char * a_szKernelVersion, const char * a_szInitrdImage ) {
int iRez = 0;
char szInitrdName[ 80 ];
char szCmd[ 512 ];

#ifndef USE_NEW_KERNEL_DIVISION
    if ( can_do_smp ) sprintf( szInitrdName, "%ssmp", a_szKernelVersion );
    else
#endif	
		sprintf( szInitrdName, "%s", a_szKernelVersion );

//    winStatus(32, 3, _( "Creating initial ramdisk..." ) );

   sync();
//#ifdef UNMOUNT_PROC
//   system( "umount /proc" );
//#endif
//   char cmd[128];
//   sprintf( cmd, "mount -t proc proc %s/proc", a_szPrefix );
//   system( cmd );

   system( "insmod -qs loop >/dev/null 2>&1" );
   sprintf( szCmd, "chroot %s /bin/sh -c \"/sbin/mkinitrd -f %s --ifneeded %s\"", a_szPrefix, a_szInitrdImage, szInitrdName );
	 Log::logMain( yp_loglevel, "%s", szCmd);
   iRez = system( szCmd );
	 if (iRez<0) Log::logMain( yp_loglevel, "MKINITRD:res=%d",iRez);
   sync();
//   sprintf( cmd, "umount %s/proc", a_szPrefix );
//   system( cmd );
//#ifdef UNMOUNT_PROC
//   system( "mount -t proc proc /proc" );
//#endif

    if ( iRez ) unlink( ( NEWROOT +"/boot/initrd" ).latin1() );
	
		chdir( "/" ); //Added for unmounting.
	
    return iRez;
}

int GRUB::CreateInitrdImage( const char *a_szRootPath, const char *a_szKernelVersion ) {
char szAbsinitrdImage[ 80 ];
char szTmpInitrdImgName[ 80 ];
struct stat sb;
FILE *fp=NULL;
useInitrd = FALSE;
#ifndef USE_NEW_KERNEL_DIVISION
	if ( can_do_smp ) {
		sprintf( szIinitrdImage, "vmlinuz-%ssmp.img", a_szKernelVersion );
		sprintf( szTmpInitrdImgName, "/boot/vmlinuz-%ssmp.img", a_szKernelVersion );	

		if ( mkinitrd( a_szRootPath, a_szKernelVersion, szTmpInitrdImgName ) )
	    		can_do_smp = FALSE;
		else {
			sprintf( szAbsinitrdImage, "%s%s", a_szRootPath, szTmpInitrdImgName );
	
			if ( !stat ( szAbsinitrdImage, &sb ) )
				useInitrd = TRUE;
		}
	} else {
#endif
		sprintf( szIinitrdImage, "vmlinuz-%s.img", a_szKernelVersion );
		sprintf( szTmpInitrdImgName, "/boot/vmlinuz-%s.img", a_szKernelVersion );
		if ( mkinitrd( a_szRootPath, a_szKernelVersion, szTmpInitrdImgName ) )
			return -1;
		sprintf( szAbsinitrdImage, "%s%s", a_szRootPath, szTmpInitrdImgName );
		Log::logMain( yp_loglevel,"szAbsinitrdImage<- %s%s", a_szRootPath, szTmpInitrdImgName );
#ifndef USE_NEW_KERNEL_DIVISION
		if ( !stat( szAbsinitrdImage, &sb ) )
				useInitrd = TRUE;
#else
		if ((fp=fopen(szAbsinitrdImage,"r"))!=NULL)
		{
				fclose(fp);				
				useInitrd = TRUE;
		}
#endif
#ifndef USE_NEW_KERNEL_DIVISION
	}
#endif
	return 0;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~`~~~~~~

GRUBInfoItem::GRUBInfoItem(QListView *parent, bool active, const QString &partition,
			     const QString &type, const QString &label)
  : QCheckListItem(parent, "", QCheckListItem::CheckBox)
{
  setText(1,partition);
  setText(2,type);
  setText(3,label);
  setOn(active);
}


GRUBInfoItem::~GRUBInfoItem()
{
}

int GRUB::link_file( int a_iSMP, const char *a_szKernelPrefix, const char *a_szKernelVersion, const char *a_szFileName, const char *a_szMapAs ) {
char szPathName[ 1024 ];
int iPathLen;
char szLinkPath[ 1024 ];
//char szLinkedAs[ 1024 ];

	if  ( chdir( a_szKernelPrefix ) ) return -1;

	iPathLen = readlink(  a_szMapAs, szPathName, 1024 );
	 //If iPathLen == -1 file not found or it`s not readable use default file name
	if ( iPathLen != -1 ) {
		//Check if link contain perfect SMP support & look`s like current version
		szPathName[ iPathLen+1 ] = 0;
		//Check first version number
		if ( strstr( szPathName, a_szKernelVersion ) ) {
			if ( strstr( szPathName, "smp" ) ) {
				if ( a_iSMP ) return 0;
			}
		}
		if ( unlink( a_szMapAs ) ) return errno* -1;
	}
	sprintf( szLinkPath, "%s-%s", a_szFileName, a_szKernelVersion );
	if ( a_iSMP ) strcat( szLinkPath, "smp" );
	
	if ( symlink( szLinkPath, a_szMapAs ) ) return errno * -1;

	return 0;
}

int GRUB::link_boot_files( int a_iSMP, const char *a_szKernelPrefix, const char *a_szKernelVersion ) {
int iRetStat = 0;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "System.map", "System.map" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "module-info", "module-info" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "vmlinuz", "vmlinuz" );
	if ( iRetStat < 0 ) return iRetStat;

	return 0;
}

QString GRUB::lastVersionName( QString a_QFindPath, QString a_QVersionNumber ) {
QDir d( a_QFindPath );
QString MaxVer = "", MaxVerName = "";
d.setNameFilter( "vmlinuz-" + a_QVersionNumber + ".*" );
uint i;
const QFileInfoList *list = d.entryInfoList();
QFileInfoListIterator it( *list );      // create list iterator
QFileInfo *fi;                          // pointer for traversing
while ( (fi=it.current()) ) {           // for each file...
	QString QFinder = fi->fileName().mid( 8, fi->fileName().length() - 8 );
	int iSeekPos = QFinder.find( '-', 0 );
	if ( iSeekPos != -1 ) {
		i = QFinder.find( '-', 0 ) + 1;
		while ( ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) || QFinder[ i ] == '.' ) i++;
		QFinder = QFinder.mid( 0, i );
	} else {
		i = QFinder.length();
		if ( !i ) continue;	i--;
		while( !( ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) || QFinder[ i ] == '.' ) ) {
			i--; if ( !i ) break;			
		}		
		QFinder = QFinder.left( i + 1 );
	}

	if ( strcmp( QFinder, MaxVer ) > 0 ) {
		MaxVer = QFinder; MaxVerName = fi->fileName();
	}
++it;                               // goto next list element
}
	return MaxVerName;
}	

//Only version without filename
QString GRUB::lastVersion( QString a_QFindPath, QString a_QVersionNumber ) {
QDir d( a_QFindPath );
QString MaxVer = "", MaxVerName = "";
d.setNameFilter( "vmlinuz-" + a_QVersionNumber + ".*" );
uint i;
const QFileInfoList *list = d.entryInfoList();
QFileInfoListIterator it( *list );      // create list iterator
QFileInfo *fi;                          // pointer for traversing
while ( (fi=it.current()) ) {           // for each file...
	QString QFinder = fi->fileName().mid( 8, fi->fileName().length() - 8 );
	int iSeekPos = QFinder.find( '-', 0 );
	if ( iSeekPos != -1 ) {
		i = QFinder.find( '-', 0 ) + 1;
		while ( ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) || QFinder[ i ] == '.' ) i++;
		QFinder = QFinder.mid( 0, i );
	} else {
		i = QFinder.length();
		if ( !i )
			continue;	
		i--;
		while( !( ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) || QFinder[ i ] == '.' ) ) {
			i--; if ( !i ) break;			
		}		
		QFinder = QFinder.left( i + 1 );
	}

	if ( strcmp( QFinder, MaxVer ) > 0 ) {
		MaxVer = QFinder; MaxVerName = fi->fileName();
	}
++it;                               // goto next list element
}
	Log::logMain( yp_loglevel, "RETURN  %s",MaxVer.latin1());
	return MaxVer;
}	

/* New,normal working version (should be :)
modified detection of device name and their presentation in grub
using device-map of grub
*/
#define TRUE true
#define FALSE false

char *GRUB::ConvLinuxDev2GRUB( char *a_szConvDevName, const char *a_szDeviceName,char *a_szDevName2DevNameDisk=NULL,int *ipDiskNum=NULL) {
#define dfMaxLen 400
char *cpPointer, *cpDeviceNamePtr,*cpDevName;//cpDevName should always keep /dev/..
FILE *fp;
char szTmpDeviceName[dfMaxLen]; //hope it will not bigger
char szBuf[dfMaxLen];
char cpCmd[300];
char cpDeviceMapName[]="/tmp/device.map";

int iPartNum=-1; //partition number
int iCount;
bool bDataReady = FALSE; //historical name that all is already done
bool bProc=FALSE,bDev=FALSE;

//check for exceptions
if (a_szDeviceName==NULL)
{
Log::logMain(2,"ConvLinuxDev2GRUB:parameter(a_szDeviceName) is NULL,return NULL");
return NULL;
}
if ((a_szConvDevName==NULL) && (a_szDevName2DevNameDisk==NULL) && (ipDiskNum==NULL))
{
Log::logMain(2,"ConvLinuxDev2GRUB:There is no variable,which should used to return result");
return NULL;
}
if (a_szDevName2DevNameDisk!=NULL) a_szDevName2DevNameDisk[0] = 0x00; //can be used for result
if (ipDiskNum!=NULL)	*ipDiskNum = -1;
//end of checks

	cpDevName = cpPointer = strstr( a_szDeviceName, "/dev/" );
	if ( cpPointer ) cpDeviceNamePtr = cpPointer + 5;
	else return NULL;
	//cpDeviceNamePtr = (char *)a_szDeviceName;

	/*We should divide device and partition parts*/	
  /*devfs - check,is it require additional checkings */
	/*do not forget add devfs floppy support*/
	//add more checks for future stability
#ifdef DEVFS
	//I think this DEVFS block is more than not perfect
	if ((cpPointer = strstr( cpDeviceNamePtr, "part" ))!=NULL)
	{//important: this code should be first,because of some confusion
			if (sscanf(&cpPointer[4],"%d",&iPartNum)==1)
				{
					iPartNum--; //don't forget about it
					iCount = strlen(cpDevName)-strlen(cpPointer);
					if (iCount<(dfMaxLen-5))
					{
						memcpy(szTmpDeviceName,cpDevName,iCount);
						szTmpDeviceName[iCount]=0;
					}//if
					//make name conversion to get hd[x] later
					strcat(szTmpDeviceName,"disk");
					bDataReady = true;
				}//sscanf disk //else error
	}	
	else
	if ((cpPointer = strstr( cpDeviceNamePtr, "disk" ))!=NULL)
	{
	   /*
		THAT IS OK!
	   */	
	} //end of nfs //but does anybody knows how it work?
	else
#endif
	if ((cpPointer = strstr( cpDeviceNamePtr, "ida/"))!=NULL)
	{     //cut information about partition
				if((cpPointer =strchr(cpPointer,'p'))!=NULL)
				if (sscanf(cpPointer,"p%d",&iPartNum)==1)
				{
					iPartNum--; //don't forget about it
					iCount = strlen(cpDevName)-strlen(cpPointer);
					if (iCount<(dfMaxLen-5)) //do it more beautifull
					{
						memcpy(szTmpDeviceName,cpDevName,iCount);
						szTmpDeviceName[iCount]=0;
					}//if
					bDataReady = true;
				}//sscanf disk //else error	
	}
	else
	if ((cpPointer = strstr( cpDeviceNamePtr, "cciss/"))!=NULL)
	{     //cut information about partition
				if((cpPointer =strchr(cpPointer,'p'))!=NULL)
				if (sscanf(cpPointer,"p%d",&iPartNum)==1)
				{
					iPartNum--; //don't forget about it
					iCount = strlen(cpDevName)-strlen(cpPointer);
					if (iCount<(dfMaxLen-5))
					{
						memcpy(szTmpDeviceName,cpDevName,iCount);
						szTmpDeviceName[iCount]=0;
					}//if
					bDataReady = true;
				}//sscanf disk //else error	
	}
	else
	if ((cpPointer = strstr( cpDeviceNamePtr, "rd/"))!=NULL)
  		{     //cut information about partition
           if((cpPointer =strchr(cpPointer,'p'))!=NULL)
           if (sscanf(cpPointer,"p%d",&iPartNum)==1)
           {
              iPartNum--; //don't forget about it
              iCount = strlen(cpDevName)-strlen(cpPointer);
       		       if (iCount<(dfMaxLen-5))
                    {
                        memcpy(szTmpDeviceName,cpDevName,iCount);
                        szTmpDeviceName[iCount]=0;
          	        }//if
                        bDataReady = true;
           }//sscanf disk //else error
        }
  else
	if ((cpPointer = strstr( cpDeviceNamePtr, "sd"))!=NULL)
	{     //cut information about partition
			if ((( cpPointer - cpDeviceNamePtr )==0) && (strlen(cpPointer)>3))
			{
			if ((cpPointer[2]>='a') && (cpPointer[2]<='z')){
				cpPointer+=3;								
				if (sscanf(cpPointer,"%d",&iPartNum)==1)
				{
					iPartNum--; //don't forget about it
					iCount = strlen(cpDevName)-strlen(cpPointer);
					if (iCount<(dfMaxLen-3))
					{
						memcpy(szTmpDeviceName,cpDevName,iCount);
						szTmpDeviceName[iCount]=0; //idea change it to strncpy :)
					bDataReady = TRUE;	
					}//if
				}//sscanf disk //else error
			}	
			}//if >0
	}
	else
	if ((cpPointer = strstr( cpDeviceNamePtr, "hd"))!=NULL)
	{     //cut information about partition
			if ((( cpPointer - cpDeviceNamePtr )==0) && (strlen(cpPointer)>3))
			{
				if ((cpPointer[2]>='a') && (cpPointer[2]<='z')){
				cpPointer+=3;								
				if (sscanf(cpPointer,"%d",&iPartNum)==1)
				{
					iPartNum--; //don't forget about it
					iCount = strlen(cpDevName)-strlen(cpPointer);
					if (iCount<(dfMaxLen-3))
					{
						memcpy(szTmpDeviceName,cpDevName,iCount);
						szTmpDeviceName[iCount]=0;
					}//if
					bDataReady = true;
				}//sscanf disk //else error
				}	
			}//if >0
	}
	//it seems old floppies should go as usual device ,not as exceptions
	
	if (!bDataReady)
	{
		iCount = strlen(cpDevName);
	//for test	printf("\n!BDataReady");
		if (iCount<dfMaxLen)
		{
			memcpy(szTmpDeviceName,cpDevName,iCount);
			szTmpDeviceName[iCount]=0;
		}	
	}
bDataReady = FALSE; 		//reset flag,now it is used as flag for found device

/************CHECK IF IT IS LINK****************/
/////////////////BE CAREFULLY//////////////////
iCount = readlink(szTmpDeviceName, szBuf, dfMaxLen-1);
if (iCount>0)
{
	szBuf[iCount]=0;
	if (strstr(szBuf,"/dev/")) sprintf(szTmpDeviceName,"%s",szBuf);
	else sprintf(szTmpDeviceName,"/dev/%s",szBuf);		
	Log::logMain( yp_loglevel, "READLINK: %s", szTmpDeviceName );
}

/////////////////IMPORTANT PART////////////////
//if it is required,return name of disk,without partition
if (a_szDevName2DevNameDisk!=NULL) memcpy(a_szDevName2DevNameDisk,szTmpDeviceName,strlen(szTmpDeviceName)+1);
if ((a_szConvDevName==NULL) && (ipDiskNum==NULL))
{
	Log::logMain( yp_loglevel, "ConvLinuxDev2GRUB::a_szDevName2DevNameDisk %s",a_szDevName2DevNameDisk);
	return NULL;
}

///*******FOUND IN GRUB DEVICE-MAP*******///
system(("rm -f "+NEWROOT+cpDeviceMapName).latin1());
//mount if it is not mounted
//required by GRUB (extremally)


if (system(("grep "+ NEWROOT +"/proc /proc/mounts").latin1()))
{
#ifdef UNMOUNT_PROC
		system( "killall -15 klogd" ); // it holds proc
		system( "umount /proc" );
#endif
		Log::logMain( yp_loglevel, "Conv:mounting proc" );
		system( ("mount -t proc proc "+ NEWROOT +"/proc").latin1() );
		bProc = TRUE;
}
#ifdef DEVFS
if (system(("grep "+ NEWROOT +"/dev /proc/mounts").latin1()))
{		Log::logMain( yp_loglevel, "Conv:mounting dev" );
		system( ("mount -t devfs devfs "+ NEWROOT +"/dev").latin1() );
		bDev = TRUE;
}
#else
if (system(("grep "+ NEWROOT +"/dev /proc/mounts").latin1()))
{		Log::logMain( yp_loglevel, "Conv:mounting dev" );
		system( ("mount --bind /dev "+ NEWROOT +"/dev").latin1() );
		bDev = TRUE;
}
#endif

//added for more insurance |^
sprintf(cpCmd,"chroot %s /bin/sh -c \"/sbin/grub --no-curses --batch --device-map=%s <<EOT\n\nquit\nEOT\n\"",NEWROOT.latin1(),cpDeviceMapName);

if (system(cpCmd)<0)	
{
	Log::logMain( 2, "\nError:ConvLinuxDev2Grub:Grub %s\n",cpCmd);
}
else{ 							//sometimes it works even with error
        if ((fp = fopen( ( NEWROOT +cpDeviceMapName).latin1(),"r"))!=NULL) //fix file name
        {
				#define s2 szBuf
				#define s1 cpCmd
                while (fscanf(fp,"%s%s",s1,s2)!=EOF)
                {
                Log::logMain( yp_loglevel,"fscanf(2) %s %s",s1,s2);
								//use after found device
								if (strstr(szTmpDeviceName,s2)!=NULL) //s2 can be destructed now //we found we wanted
								if (sscanf(s1,"%*[(]%[^)]",s2)==1)
								{
									Log::logMain( yp_loglevel,"\nAfter sscanf %s\n",s2);
									bDataReady=TRUE;

									if (ipDiskNum!=NULL)
									if (strlen(s2)>2){										
											*ipDiskNum = atoi(&s2[2]);
											if (strstr(s2,"hd")) *ipDiskNum+=0x80;//BIOS id for start HDD
											Log::logMain( yp_loglevel,"\"hdX(fdX) to int: %d\"",*ipDiskNum);									
								  }else
											Log::logMain(yp_loglevel,"ConvLinuxDev2GRUB::ipDiskNum==NULL or syntax error");
																		
	                if (a_szConvDevName!=NULL){
										if (iPartNum>=0) sprintf(a_szConvDevName,"%s,%d",s2,iPartNum);
										else sprintf(a_szConvDevName,"%s",s2);
									}else Log::logMain(yp_loglevel,"Do not return hdX(fdX),..,because called parameter does not required it");

									break;			
								}else		Log::logMain( yp_loglevel,"Cannot cut clear disk name from device.map");
								}
                fclose(fp);
        }else		Log::logMain( 2, "\nConvLinuxDev2GRUB:Cannot open map-file\n");
     }//else
//unmount if it is mounted by me

if (bProc)
{
		system( ("umount "+ NEWROOT +"/proc").latin1() );
#ifdef UNMOUNT_PROC
  	system( "mount -t proc proc /proc" );
#endif
}
if (bDev) system( ("umount "+ NEWROOT +"/dev").latin1() );

  	
Log::logMain( yp_loglevel, "\nDevice name to convert: %s\n",a_szDeviceName);
if (bDataReady)
{
	if (a_szConvDevName	!=NULL)
	Log::logMain( yp_loglevel, "\nConverted device: %s Part:%d\n",a_szConvDevName,iPartNum);
	else Log::logMain(yp_loglevel, "Cannot return result into NULL pointer parameter");
	return a_szConvDevName;
}
else {
	Log::logMain( 2, "\nDevice not found\n");
	//can be called for old compatibility: I not recommend it
	//return OldConvLinuxDev2GRUB( a_szConvDevName,a_szDeviceName );
	if (ipDiskNum!=NULL) *ipDiskNum = -1;
	return NULL; //call OldConv.. from gbli	
	}
#undef dfMaxLen
#undef s2
#undef s1
}

char *GRUB::OldConvLinuxDev2GRUB( char *a_szConvDevName, const char *a_szDeviceName ) {
char *cpPointer, *cpDeviceNamePtr;
char szTmpDriveNum[ 2 ];
char szTmpPartitionNum[ 255 ];
int iMaxDevNum;
int iCtrlNum; //number of controller
int iDiskNum; //disk number
int iPartNum; //partition number
bool bCnDn = FALSE;

	iMaxDevNum = 0;
	szTmpDriveNum[ 0 ] = 0; szTmpDriveNum[ 1 ] = 0;
  /*!!!!!!!!!!!!!!!*/
 	QStringList ds( Fdisk::availableDisks() );
 	QStringList::Iterator a;
	/*!!!!!!!!!!!!!*/
	cpPointer = strstr( a_szDeviceName, "/dev/" );
	if ( cpPointer ) cpDeviceNamePtr = cpPointer + 5;
	else cpDeviceNamePtr = (char *)a_szDeviceName;
	
	////////////support of scsi raid///////
	////        /dev/ida/* and dev/cciss/*
	cpPointer = strstr( cpDeviceNamePtr, "ida/" );
	if (cpPointer){
			cpDeviceNamePtr = cpPointer + 4;
			bCnDn = TRUE;
		}
	else{
				cpPointer = strstr( cpDeviceNamePtr, "cciss/" );
				if (cpPointer){
						cpDeviceNamePtr = cpPointer + 6;
						bCnDn = TRUE;
					}
			}
	
		if (bCnDn){
				 switch(sscanf(cpDeviceNamePtr,"c%dd%dp%d",&iCtrlNum,&iDiskNum,&iPartNum))
				 {
 				///make some calculations according to controlled number and etc.
				////not done,required more information
				 		case 2:
				 					sprintf(a_szConvDevName,"hd%d",iDiskNum);
									return a_szConvDevName;
				 		case 3:
				 					sprintf(a_szConvDevName,"hd%d,%d",iDiskNum,iPartNum-1);
				 					return a_szConvDevName;
				 		default:
				 					fprintf(stderr,"\n/dev/ida/: non-recognizible device name\n");
				 }
						
		} // bCnDn
			
	
				
	
	///////////////////////////////////////
	cpPointer = strstr( cpDeviceNamePtr, "fd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device which contain string fd somewhere in the text
		strncpy( a_szConvDevName, cpPointer, 3 ); //Copy also one digit number like fd0
		a_szConvDevName[ 3 ] = 0;		
	}
	cpPointer = strstr( cpDeviceNamePtr, "sd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );

		//FIXME: SCSI disks in BIOSes is last disks
		char cDriveStart = 0;
		for (a=ds.begin(); a != ds.end(); ++a) {
			if ( (*a).mid( 5, 1 ).ascii()[0] == 'h' ) {
				if( (*a).mid( 7, 1 ).ascii()[0] > cDriveStart ) {
					cDriveStart = (*a).mid( 7, 1 ).ascii()[0];
					iMaxDevNum ++;
				}
			}
		}
		//****************************************

		/*
		if ( *( cpPointer + 2 ) >= 'a' && *( cpPointer + 2 ) <= 'z' )
			szTmpDriveNum[ 0 ] = ( *( cpPointer + 2 ) - 'a' ) + 0x30;
		*/
		for (a=ds.begin(); a != ds.end(); ++a) {
	   		 if (*( cpPointer + 2 ) > (*a).mid( 7, 1 ).ascii()[0]  )  iMaxDevNum ++;
		}
		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;

		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	cpPointer = strstr( cpDeviceNamePtr, "hd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );
		/*
		if ( *( cpPointer + 2 ) >= 'a' && *( cpPointer + 2 ) <= 'z' )
			szTmpDriveNum[ 0 ] = ( *( cpPointer + 2 ) - 'a' ) + 0x30;
		*/
		for (a=ds.begin(); a != ds.end(); ++a) {
	   		 if (*( cpPointer + 2 ) > (*a).mid( 7, 1 ).ascii()[0]  ) iMaxDevNum ++;
		}
		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;

		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	return a_szConvDevName;
}

GRUB::GRUB( PartitionView *pv, QWidget *parent, const char *name )
  : GBLIPage(parent, name),
    QSBootP( NULL ), QSRootP( NULL ), QSEtcP( NULL ), pview(pv), firstTimeMBRWarning( true )
{
	// Now construct the page
  QHBoxLayout *hbox = new QHBoxLayout(this, 0, 6);

  QVBoxLayout *vbox = new QVBoxLayout( 6 );
	hbox->addStretch( 1 );
	hbox->addLayout( vbox, 8 );
	hbox->addStretch( 1 );

	vbox->addStretch( 1 );

  QLabel *label = new QLabel(tr("Operating systems to load:"), this);
	label->setAutoMask( true );
  vbox->addWidget(label);

  list = new QListView(this);
  list->setSorting( -1 );
  list->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  list->setAllColumnsShowFocus(TRUE);
  vbox->addWidget(list);
// server default boot loader is lilo, workstation grub
  int col;
#ifdef GBLI_SERVER
	col = list->addColumn("LILO"/*tr("bootable")*/);
#else
	col = list->addColumn("GRUB"/*tr("bootable")*/);
#endif

  list->setColumnWidthMode( col, QListView::Maximum );
  col = list->addColumn(tr("Partition"));
  list->setColumnWidthMode( col, QListView::Maximum );
  col = list->addColumn(tr("Type"));
  list->setColumnWidthMode( col, QListView::Maximum );
  col = list->addColumn(tr("Label"));
  list->setColumnWidthMode( col, QListView::Maximum );

  vbox->addStretch( 1 );


  QHBoxLayout *hbLay = new QHBoxLayout(vbox);
  label = new QLabel(tr("Where to install"),this);
	label->setAutoMask( true );
  hbLay->addWidget(label);

	ptqcomboSelect = new QComboBox( FALSE, this, "read-only combo" );
	ptqcomboSelect->insertItem ("GRUB");
	ptqcomboSelect->insertItem ("LILO");

#ifdef GBLI_SERVER	
	ptqcomboSelect->setCurrentItem(1); // LILO
#else 		
	ptqcomboSelect->setCurrentItem(0); //GRUB
#endif
	
	hbLay->addWidget (ptqcomboSelect);
	hbLay->addStretch ();
	
  connect ( ptqcomboSelect, SIGNAL (activated(const QString &)),this,SLOT(LoaderChanged(const QString &)));

  QButtonGroup *bg = new QButtonGroup(0);
  connect(bg, SIGNAL(clicked(int)), this, SLOT(targetChanged(int)));

  QHBoxLayout *hb = new QHBoxLayout(vbox);

  mbr = new QRadioButton(tr("&Master boot record (MBR)"), this);
	mbr->setAutoMask( true );
  mbr->setFocusPolicy(QWidget::StrongFocus);
  hb->addWidget(mbr);
  bg->insert(mbr,0);

  ofLabel = new QLabel( tr("of"), this);
	ofLabel->setAutoMask( true );
  hb->addWidget( ofLabel );

  disks = new QComboBox( this );

  connect(disks, SIGNAL(activated(const QString &)), this, SLOT(diskChanged(const QString &)));
	disks->setAutoMask( true );
  hb->addWidget(disks);
  diskinfo = new QLabel( this );
	diskinfo->setMinimumWidth( 90 );
	diskinfo->setAutoMask( true );
  if( disks->count() )
    diskChanged( disks->text(0) ); //
  else
    diskChanged( "" );
  hb->addWidget(diskinfo);
  hb->addStretch();

  target = new QRadioButton(tr("&Target partition"), this);
	target->setAutoMask( true );
  target->setFocusPolicy(QWidget::StrongFocus);
  vbox->addWidget(target);
  bg->insert(target,1);

  QHBoxLayout *hh2 = new QHBoxLayout( vbox );

  other = new QRadioButton(tr("&Other:"), this);
	other->setAutoMask( true );
  other->setFocusPolicy(QWidget::StrongFocus);
  hh2->addWidget(other);
  bg->insert(other,2);


  otherDest = new QLineEdit(this);
  hh2->addWidget(otherDest);
	hh2->addStretch( 2 );
  connect( otherDest, SIGNAL(textChanged ( const QString & )),
    this, SLOT(otherChanged( const QString & )) );

	vbox->addSpacing( 5 );
  no_grub = new QRadioButton(tr("Skip"), this);
	no_grub->setAutoMask( true );
  no_grub->setFocusPolicy(QWidget::StrongFocus);
  vbox->addWidget(no_grub);
  bg->insert(no_grub,3);

	vbox->addStretch( 1 );

  WriteConfigFileFlag = 0;
/*  LInlineHelp *inlineHelp = new LInlineHelp(this);
  inlineHelp->setDocument( "lilo_overview.qml" );
  hbox->addWidget( inlineHelp );*/

  targetChanged( 0/*1:Maddy*/ );
  useInitrd = FALSE;
}



void GRUB :: LoaderChanged(const QString & strLoaderName)
{
// QMessageBox::warning((QWidget *) this,"NB!" ,"loader changed!",
//			QMessageBox::Yes, QMessageBox::No );
			
	list->setColumnText(0, strLoaderName);

}

GRUB::~GRUB()
{
}

int GRUB::GoodOldLILOConfWrite(bool a_addRunlevel5 ) {
char strTmp[400];//reserve some mem for extra long device-name(for ex.devfs)
int iMap1=-1,iMap2=-1;
	/* Small backup why not? :)*/
	rename( (NEWROOT +"/etc/lilo.conf").latin1(), (NEWROOT +"/etc/lilo.conf.rpmsave").latin1() );
	rename( (NEWROOT +"/etc/silo.conf").latin1(), (NEWROOT +"/etc/silo.conf.rpmsave").latin1() );

	QDir dir;
	dir.mkdir( NEWROOT +"/etc" );
 	QFile f( NEWROOT +"/etc/lilo.conf" );
    if ( !f.open( IO_WriteOnly ) ) return -1;

	QTextStream conf(&f);
	
	conf << "#" << endl;
    conf << "# /etc/lilo.conf - generated by GBLI" << endl;
    conf << "#" << endl << endl;
	//make lilo look like GRUB :)
		conf <<	"menu-title=\"SOT Linux boot menu\"" << endl;
		conf << "menu-scheme=wb:kw:wb:Wb" << endl;

	conf << "boot=";
  	if (mbr->isChecked())
		conf << disks->currentText() << endl;
	if (target->isChecked())
    	conf << MountPoints::mountPartition("/") << endl;
    if (other->isChecked())
    	conf << otherDest->text() << endl;

    conf << "map=/boot/map" << endl;
		conf << "install=/boot/boot.b" << endl;
   	conf << "prompt" << endl;
    conf << "timeout=50" << endl;
		
		for (int i=0;i<3;i++){
			char *paramS=NULL,*paramLabel=NULL;
			if ((!a_addRunlevel5) && (i==0)) continue;
			switch (i)
				{
					case 0:	paramS="5";paramLabel="graphical"; break;
					case 1:	paramS="3";paramLabel="text"; 	break;
					case 2:	paramS="1";paramLabel="failsafe"; break;
				}
			conf << "image=/boot/vmlinuz" << endl;
			conf << "\tlabel=" << "\"SL " << paramLabel<<"\"" << endl;
			conf << "\tappend="<< paramS << endl;
			conf << "\troot=" << MountPoints::mountPartition("/") << endl;
			if ( useInitrd )
	    conf << "\tinitrd=/boot/" << szIinitrdImage << endl;
			conf << "\tread-only" << endl;
		}


    QListViewItem *item = list->firstChild();
    item = item->nextSibling();

	while (item) {

      QCheckListItem *ci = static_cast<QCheckListItem*>(item);
	
      if (ci && ci->isOn()) {
		if ( ( item->text(3) ).contains( "Linux" ) ) {
		    conf << "image=/boot/vmlinuz" << endl;
				conf << "\tlabel=" << item->text(3) << endl;
				conf << "\troot=" << item->text(1) << endl;
		} else {
			conf << "other=" << item->text(1) << endl;
			conf << "\tlabel=" << item->text(3) << endl;
			//get disk name
			if (item->text(1).latin1()==NULL)
			{
					Log::logMain(2,"LiloConf:disk name(item->text(1)==NULL)");
					strTmp[0] = 0x00;
			}else
			ConvLinuxDev2GRUB(NULL,item->text(1).latin1(),strTmp,&iMap1);
			if (iMap1!=-1) //if we got converted disk-name
			{
					conf << "#\ttable=" << strTmp << endl;	
					ConvLinuxDev2GRUB(NULL,MountPoints::mountPartition("/"),NULL,&iMap2);
					Log::logMain(yp_loglevel,"Liloconf:table = %s",strTmp);
			}else
			    conf << "#\ttable=" << item->text(1) << endl;

       	//if ( strcmp( item->text(1), MountPoints::mountPartition("/") ) ) {
				//in most standart case it should work			
				if ((iMap1!=-1) && (iMap2!=1)){
				  	if (iMap1!=iMap2){ //test it seriously!!!!!!!!!!!!!
						//do megamapping
      			sprintf(strTmp,"0x%.2x",iMap1);
					  conf << "\tmap-drive=0x80" << endl;
				    conf << "\t   to = " << strTmp << endl;
		  	  	conf << "\tmap-drive=" << strTmp << endl;
			  	  conf << "\t   to = 0x80" << endl;
						}
				}else
					if ( strcmp( item->text(1), MountPoints::mountPartition("/") ) ){
				    /* boot off the second drive, so reverse the BIOS maps */
				    conf << "\tmap-drive=0x80" << endl;
				    conf << "\t   to = 0x81" << endl;
		  	  	conf << "\tmap-drive=0x81" << endl;
			  	  conf << "\t   to = 0x80" << endl;
					}
						
		}
      }
      item = item->nextSibling();
    }
	conf << flush;
	f.close();
	return 0;
}

void GRUB::init()
{
  // do we have only ide or only scsi disks?
  int ide=0;
  int scsi=0;
  int special=0;
  QStringList ds( Fdisk::availableDisks() );
  QStringList::Iterator aa;
  for (aa=ds.begin(); aa != ds.end(); ++aa) {
    if ((*aa).left(8) == "/dev/hd")
      ide++;
    else if ((*aa).left(8) == "/dev/sd")
      scsi++;
    else
      special++;
  }

  // if yes, MBR will be on the first disk
  if (special==0 && (ide == 0 || scsi == 0)) {
    ofLabel->hide();
    disks->hide();
  }else{
    ofLabel->show();
    disks->insertStringList(ds);
    disks->show();
	}

  // clear list
  list->clear();

  // add other partitions
  QString system, label;
  int nr=0, linuxnr=0;
  QString root = MountPoints::mountPartition( "/" );

  QStringList all = pview->allPartitions();
  QStringList::Iterator a;
  for( a = all.begin(); a != all.end(); ++a ) {

    int id = pview->idOfPartition(*a);

    if (id == 0x01 ||	id == 0x04 ||
  			id == 0x11 ||	id == 0x14 ||
	  		id == 0x16 ||	id == 0x24 ||
		  	id == 0xc1 ||	id == 0xc4 ||
			  id == 0xc6 ||	id == 0xc4 ||
  			id == 0x06 ||	id == 0x0b ||
	  		id == 0x0c || 	id == 0x0e ){
    	if (nr == 0)
    	  label = "\"Windows/DOS\"";
    	else
    	  label = QString("\"Windows/DOS %0\"").arg(nr);
    	nr++;
	
    	new GRUBInfoItem(list, true /*FALSE - by Maddy*/, *a, Fdisk::sysname(id), label);
    }else /*if ( id == 0x83 && (MountPoints::mountPartition("/") != *a) ) {*/
    if ( (id == 0x83 || id == 0x19) && !pview->partition( *a )->willBeFormated &&
         *a != root && pview->partition( *a )->bootable /*by Maddy*/ ){

    	if ( linuxnr == 0 )
	      label = "\"Linux\"";
    	else
	      label = QString("\"Linux %0\"").arg( linuxnr );

        new GRUBInfoItem(list, true /*FALSE*/, *a, Fdisk::sysname(id), label);	
    }/* else if (id != 0x82 && (MountPoints::mountPartition("/") != *a))
      {
	// check for an installed lilo

	int dev = open(*a, O_RDONLY);
	if (dev >= 0)
	  {
	    char buffer[12];
	    if (read(dev, buffer, 12) == 12)
	      {
		if ((strncmp(&buffer[2],"LILO",4) == 0)
		    || (strncmp(&buffer[6],"LILO",4) == 0))
		  {
		    QString label = *a;
		    int pos = label.findRev('/');
		    if (pos>0)
		      label.remove(0,pos+1);
		    label = "lilo-"+label;

		    new GRUBInfoItem(list, FALSE, *a, Fdisk::sysname(id), label);
		  }
	      }
	  }
      }
*/
  }

  // add ourselves
  QListViewItem *l = new QListViewItem(list);
  l->setText(1,root);
  l->setText(2,"Linux");
  l->setText(3,"SOT Linux");

  // if we do not install onto the first disk, propose to write the MBR
// by Maddy
/*  if (root.left(8) == "/dev/hda" || root.left(8) == "/dev/sda")
    target->setChecked(TRUE);
  else*/
    mbr->setChecked(TRUE);
		targetChanged( 0 );// quick hack

  target->setText(tr("&Target partition (")+root+")");

  // lay out the width of the columns
/*  list->setColumnWidth(0,20);
  list->setColumnWidth(1,32+list->columnWidth(1));
  list->setColumnWidth(2,32+list->columnWidth(2));
  list->setColumnWidth(3,32+list->columnWidth(3));*/

	emit statusChanged( GBLIPage::Good );
}

void GRUB::targetChanged(int index)
{
	if( firstTimeMBRWarning && index ){
		index = askToSwitchToMBR( index );
		if( index ){
	  	disks->setEnabled( index == 0 );
		  otherDest->setEnabled( index == 2 );
		}
	}else{	
		if( index == 3 )
			index = askToSwitchToMBR( index );
  	disks->setEnabled( index == 0 );
	  otherDest->setEnabled( index == 2 );
	}
  emit nextEnabled( this, index==2?(otherDest->text().find( "/" )!=-1):true );
}

int GRUB::askToSwitchToMBR( int new_index )
{
		int res = QMessageBox::warning( this, tr("Master Boot Record"),
			tr("NB! If you choose not to install GRUB bootloader to\n"
      "the MBR (Master Boot Record) then it is likely that\n"
      "your computer will not boot Linux after restarting\n"
      "unless you manually configure some other boot loader\n"
      "to support SOT Linux.\n"
      "So, it is assumed that you really know what you are doing.\n"
      "Install GRUB bootloader to MBR?"),
			QMessageBox::Yes, QMessageBox::No );
		if( res == QMessageBox::Yes ){
	    mbr->setChecked( true );
			return 0;
		}else{
			return new_index;
		}
}

void GRUB::diskChanged( const QString &disk )
{
  QString device( disk );
  QString desc;

  if( device.left(8) == "/dev/rd/" )
    desc = tr("Mylex disk");

  if (device.left(9) == "/dev/ida/")
    desc = tr("Compaq Smart Array");

  if (device.left(11) == "/dev/cciss/")
    desc = tr("Compaq Next Generation Array");

  if (device.left(7) == "/dev/hd") {
    device.remove(0,5);
    QFile file( NEWROOT +"/proc/ide/"+ device +"/model" );
    int res = file.open(IO_ReadOnly);
    if( !res ){
      file.setName( "/proc/ide/"+device+"/model" );
      res = file.open(IO_ReadOnly);
    }
    if( res ){
      QTextStream ts(&file);
      ts >> desc;
      file.close();
    }
  }

  // TODO: Identify the SCSI disks

  if( !desc.isEmpty() )
    diskinfo->setText( "("+ desc +")" );
  else
    diskinfo->setText( "" );
}

void GRUB::otherChanged( const QString &device )
{
  emit nextEnabled( this, device.find( "/" ) != -1 );
}

void GRUB::checkPartitionDevNames( void ) {

 QSBootP = MountPoints::mountPartition("/boot");
 QSRootP = MountPoints::mountPartition("/");
 if ( QSBootP.isEmpty() ) {
	QSKernelPrefix = "/boot/";
	QSBootP = QSRootP;
 } else {
	QSKernelPrefix = "/";
 }
 QSEtcP = MountPoints::mountPartition("/etc");
 if ( QSEtcP.isEmpty() ) {
	QSEtcPrefix = "/etc/";
	QSEtcP = QSRootP;
 } else {
	QSEtcPrefix = "/";
 }
}

int GRUB::writeConfFile( QString a_QSConfFilePath, bool a_addRunlevel5 ) { //Must be as NEWROOT +"/etc/grub.conf"
char szConvDevName[ 255 ];

QFile f( a_QSConfFilePath );

 if ( QSRootP.isEmpty() ) checkPartitionDevNames( );

// QString QSBootP = MountPoints::mountPartition("/boot");
// QString QSRootP = MountPoints::mountPartition("/");


  if (f.open(IO_WriteOnly)) {

//Set write Config Write flag here
  WriteConfigFileFlag = 1;
//~~~~~~~~~~~~~~~~~~~~~~~~
    QTextStream conf(&f);

    conf << "# This file has been generated by Graphical SOT Linux Installer\n";
    conf << "# Options\n";
    conf << "color= light-gray/blue black/light-gray\n";

    //Here get current language and setup specifical language file
    char *lang = getenv("LANG");
    if ( lang && strcmp( lang, "en" ) ){
    	if( QFile( QString(NEWROOT +"/boot/grub/")+ lang +".grub" ).exists() )
      	conf << "messages=" << QSKernelPrefix << "grub/" << lang << ".grub" << endl;
      else
      	conf << "messages=" << QSKernelPrefix << "grub/en.grub" << endl;
    } else {
    	conf << "messages=" << QSKernelPrefix << "grub/en.grub" << endl;
    }

    conf << "default=0\n";
    conf << "timeout=30\n";

    QString QSKernelName;
//Old    QString QSBLPartition = ConvLinuxDev2GRUB( szConvDevName, ( MountPoints::mountPartition("/") ).ascii() );
//New check /boot partitions position.
    QString QSBLPartition = ConvLinuxDev2GRUB( szConvDevName, QSBootP.ascii() );

    if( a_addRunlevel5 ) {
    	conf << "\n\ntitle=" << tr("SOT Linux graphical mode") << endl;
     	conf << "root=(" <<  QSBLPartition << ")" << endl;
    	conf << "kernel --no-mem-option "<< QSKernelPrefix << "vmlinuz 5 root=" << MountPoints::mountPartition("/") << endl;
    	if ( useInitrd ) conf << "initrd=" << QSKernelPrefix <<  szIinitrdImage << endl;
   }

   conf << "\ntitle=" << tr("SOT Linux text mode") << endl;
   conf << "root=(" << QSBLPartition << ")" << endl;
   conf << "kernel --no-mem-option "<< QSKernelPrefix << "vmlinuz 3 root=" << MountPoints::mountPartition("/") << endl;
   if ( useInitrd ) conf << "initrd=" <<  QSKernelPrefix << szIinitrdImage << endl;

   conf << "\ntitle=" << tr("SOT Linux failsafe mode") << endl;
   conf << "root=(" << QSBLPartition << ")" << endl;
   conf << "kernel --no-mem-option " <<  QSKernelPrefix << "vmlinuz 1 root=" << MountPoints::mountPartition("/") << endl;
   if ( useInitrd ) conf << "initrd=" <<  QSKernelPrefix << szIinitrdImage << endl;

    QListViewItem *item = list->firstChild();
    item = item->nextSibling();

    while (item) {

      QCheckListItem *ci = static_cast<QCheckListItem*>(item);

      if (ci && ci->isOn()) {
	if ( ( item->text(3) ).contains( "Windows/DOS" ) ) {
			
		QString QSInstPartition = ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() );

 		conf << "\n\ntitle=" << item->text(3) << endl;
		conf << "root=(" << QSInstPartition << ")" << endl;
		conf << "makeactive" << endl;
    	conf << "chainloader +1" << endl;
		if ( strncmp( QSInstPartition, QSBLPartition, 3 ) ) {
        /* boot off the second drive, so reverse the BIOS maps */
//as:   	conf << "map (hd0) (" << QSBLPartition.left( 3 ) <<  ")" << endl;
//as:			conf << "map (" << QSBLPartition.left( 3 ) << ") (hd0)" <<  endl;

// New code by Maddy:
			conf << "map (hd0) (" << QSInstPartition.left( 3 ) <<  ")" << endl;
			conf << "map (" << QSInstPartition.left( 3 ) << ") (hd0)" <<  endl;
		}
	} else
		if ( ( item->text(3) ).contains( "Linux" ) ){
	     		conf << "\n\ntitle=" << item->text(3) << endl;
			conf << "root=(" << ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() ) << ")" << endl;
			conf << "kernel=/boot/" << "vmlinuz" << " root=" << item->text(1) << endl;

        	} else {
	     	conf << "\n\ntitle=" << item->text(3) << endl;
			conf << "root=(" << ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() ) << ")" << endl;
	}
      }

      item = item->nextSibling();
    }
    conf << flush;
    f.close();
  }
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
return 0;
}

void GRUB::finish( bool a_addRunlevel5 )
{
//GbliDebug( "GRUB::finish" );
//  QFile f( NEWROOT +"/boot/grub/lilo.conf" );//???
  QString GRUBInstallPath;
//  QFile f( NEWROOT +"/etc/grub.conf" );
  QFile cpuinfo("/proc/cpuinfo");
  char szConvDevName[ 255 ], szBootDevName[ 50 ], szEtcDevName[ 50 ];
  char **argv;
  int iTmp;
	char *cpConvDevName=NULL;


   argv = (char **)alloca(sizeof(*argv) * 4);
   for ( iTmp = 0; iTmp < 4; iTmp++ ) argv[ iTmp ] = (char *)alloca(1000);

  if (mbr->isChecked())
   GRUBInstallPath = disks->currentText();
  if (target->isChecked())
   GRUBInstallPath = MountPoints::mountPartition("/");
  if (other->isChecked())
   GRUBInstallPath = otherDest->text();

  can_do_smp = FALSE;
  if (cpuinfo.open(IO_ReadOnly)) {
     QTextStream cinfo(&cpuinfo);
     QString line;
     QRegExp r = QRegExp("flags.* apic ");

     line = cinfo.readLine();
     while ( !line.isEmpty() ) {
      	if (line.contains(r)) {
      	    can_do_smp = TRUE;
      	    break;
      	}
      	line = cinfo.readLine();
     }
     cpuinfo.close();
  }
//	can_do_smp = detectSMP();


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 QString  QSKVer = lastVersion( NEWROOT +"/boot", "2.4" );

#ifndef USE_NEW_KERNEL_DIVISION
// fast hack (by Maddy)
Log::logMain( 2, "Grub smp check: %s -- %d, %d\n", (NEWROOT +"/boot/vmlinuz-"+ QSKVer +"smp").ascii(),can_do_smp, QFile( NEWROOT +"/boot/vmlinuz-"+ QSKVer +"smp" ).exists() );
  if( can_do_smp && !QFile( NEWROOT +"/boot/vmlinuz-"+ QSKVer +"smp" ).exists() ){
    can_do_smp = false;
	}
Log::logMain( 2, "Can do smp: %d", can_do_smp );
//I don`t now is it a good place for that but why not?
    if ( !QSKVer.isEmpty() )
			link_boot_files( can_do_smp ?1:0, NEWROOT +"/boot", QSKVer.ascii() );
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	CreateInitrdImage( (NEWROOT +"/").ascii(),  QSKVer.ascii() );
#else
int iRes=0;

	#ifdef GBLI_SERVER
//			#warning FOR DEMO PURPOSE #ifdef GBLI_SERVER is put to #ifndef GBLI_SERVER
//			#warning Should be restored if compiled for normal production
//		  #ifndef GBLI_SERVER
		Log::logMain( yp_loglevel, "Creating image for divided kernel-server");
		Log::logMain( yp_loglevel, "PAR1:%s, PAR2:%s",(NEWROOT +"/").latin1(),(QSKVer+"server").latin1());
		iRes = CreateInitrdImage( (NEWROOT +"/").latin1(),(QSKVer+"server").latin1());
		YPKernelVer = (QSKVer+"server").latin1();
	#else
	 	Log::logMain( yp_loglevel, "Creating image for divided kernel-desktop");
		iRes = CreateInitrdImage( (NEWROOT +"/").latin1(),(QSKVer+"desktop").ascii());
		YPKernelVer = (QSKVer+"desktop").latin1();
	#endif
		if (iRes<0)Log::logMain( 2, "initrd image not created");
#endif

 //Now save old LILO settings to config file
 GoodOldLILOConfWrite( a_addRunlevel5);	

 // Check where /boot and / is mounted and set such variables
 checkPartitionDevNames(  );

 if ( !WriteConfigFileFlag ) writeConfFile( NEWROOT +"/etc/grub.conf", a_addRunlevel5 );

//Old  ConvLinuxDev2GRUB( szConvDevName, ( ( MountPoints::mountPartition("/") ).ascii() ) );
// And now check where /boot partition is placed.

  ConvLinuxDev2GRUB( szConvDevName, QSBootP.ascii() );
  strcpy( szBootDevName, szConvDevName );

  if ( !strstr( szConvDevName, "," ) )
  	strcat( szBootDevName, ",0" );
 //required additional checks
 ConvLinuxDev2GRUB( szEtcDevName, QSEtcP.ascii() );

  if( !no_grub->isChecked() ){

//#ifdef UNMOUNT_PROC
//		system( "killall -15 klogd" ); // it holds proc
//		system( "umount /proc" );
//#endif
//  	system( ("mount -t proc proc "+ NEWROOT +"/proc").latin1() );
//		system( ("mount -t devfs devfs "+ NEWROOT +"/dev").latin1() );

		//required by ConvLinuxDec2GRUB                                                                 	
  	char cmd[1024]; //old 512
  	
 Log::logMain( yp_loglevel, "GRUBInstallPath latin1=%s",GRUBInstallPath.latin1());
 Log::logMain( yp_loglevel, "ConvLinuxDev2GRUB( szConvDevName, GRUBInstallPath )=%s",ConvLinuxDev2GRUB( szConvDevName, GRUBInstallPath ));

		cpConvDevName = ConvLinuxDev2GRUB( szConvDevName, GRUBInstallPath.latin1());

		if((ptqcomboSelect->currentItem() == 0) && (cpConvDevName!=NULL))/* grub */
    	sprintf( cmd, "chroot %s /bin/sh -c \"/sbin/grub --no-floppy --no-curses --batch <<EOT\nroot=(%s)\ninstall=(%s)%sgrub/stage1 d (%s) (%s)%sgrub/stage2 0x8000 p (%s)%sgrub.conf\nquit\nEOT\n\"",
  			NEWROOT.latin1(),			
  			szBootDevName, szBootDevName, QSKernelPrefix.ascii(), cpConvDevName,
  			szBootDevName, QSKernelPrefix.ascii(), szEtcDevName, QSEtcPrefix.ascii());
		else{  /* LILO */
    	sprintf( cmd, "chroot %s /bin/sh -c \"/sbin/lilo\"",NEWROOT.latin1() );

			if((ptqcomboSelect->currentItem() == 0) && (cpConvDevName==NULL))
			QMessageBox::information( this, tr("Grub bootloader"),
			tr("Grub cannot properly recognize your disks. \n"
			"Lilo bootloader will be used instead.") );

		}

	fflush( stdout );
	Log::logMain( yp_loglevel, "%s\n", cmd );
	fflush( stdout );

	char redirect[1024];
	// put loader output into result file
	sprintf ( redirect,"%s 2> %s", cmd, (NEWROOT + "/tmp/result").latin1() );

	// setup loader.
	int iRet = system( redirect );

  if (iRet<0){
  	Log::logMain( 2, "system: %d ", iRet);
		Log::logMain( 2, "redirect: %s",redirect);
  }

	char databuff[1024];
	
	if (iRet != 0)
  {
		FILE* ptFile = fopen (NEWROOT + "/tmp/result" ,"r");

		if (ptFile != NULL)
		{
		  fgets (databuff, sizeof (databuff) ,ptFile);
	
			size_t size = strlen (databuff);

			if (size >0)
			{
				QMessageBox::warning((QWidget *) this,tr("Loader failed!"),databuff);
	    	sprintf( cmd, "chroot %s /bin/sh -c \"/sbin/lilo -L\"", NEWROOT.latin1() );
				//needed additional checks for CHS modes(make in future)
				if (system(cmd)<0) Log::logMain( 2, "LILO with -L: %s ", cmd);

			}

			fclose (ptFile);
		}
	}
//  	system( ("umount "+ NEWROOT +"/proc").latin1() );
//  	system( ("umount "+ NEWROOT +"/dev").latin1() );
  	//required by ConvLinuxDec2GRUB
//#ifdef UNMOUNT_PROC
//  	system( "mount -t proc proc /proc" );
//#endif

	}
	chdir( "/" ); //Added for unmounting.
	Log::logMain( yp_loglevel, "ConvLinuxDev2GRUB 10");
}

//#ifdef DEVEL_createBootDisk
//It`s no GRUB page work, it must be moved on different class name, but now why not? :)
int GRUB::makeBootdisk(char * prefix, char * kernelVersion) {

//    if (loadModule("loop", DRIVER_OTHER, DRIVER_MINOR_NONE, NULL)) return INST_ERROR;
	
//    winStatus(32, 3, _("Bootdisk"), _("Creating bootdisk..."));
   char cmd[512];
    int iRes;
    bool bOk = true;
		
	  system("modprobe -qs loop >/dev/null 2>/dev/null");		
//#ifdef UNMOUNT_PROC	
//   system( "killall -15 klogd" ); // it holds proc
//   system( "umount /proc" );
//#endif
//   system( ("mount -t proc proc "+ NEWROOT +"/proc").latin1() );

   do{
	   qApp->setOverrideCursor( Qt::waitCursor );
	
	   sprintf( cmd, "chroot %s /bin/sh -c \" /sbin/mkbootdisk --noprompt --grub --device /dev/fd0 %s\"",  prefix, kernelVersion );
		 Log::logMain(32, "Running mkbootdisk: %s\n", cmd);

	   iRes = system( cmd );
//printf( "System exec mkbootdisk = %d\n", iRes );
 		 qApp->restoreOverrideCursor();	
	   if ( iRes ) {
  		bOk = QMessageBox::critical( this, tr("Error!"), tr("Could not mount floppy disc or disc is wrong!"), tr("Retry"), tr("Cancel") );
	   }
    }while( iRes && !bOk );

//   system( ("umount "+ NEWROOT +"/proc").latin1() );
//#ifdef UNMOUNT_PROC
//   system( "mount -t proc proc /proc" );
//#endif

//    removeModule("loop");

	chdir( "/" ); //Added for unmounting.
		
    return 0;
}
#define NOSTUFFHERE -1
#define NOBOOTDISKS 1
#define LINUXNOTFOUND -2
int GRUB::createBootdisk( bool a_addRunlevel5 ) {
int stage;
stage = 1;

//Check if start stuff in right place closed now
if (access( (NEWROOT +"/sbin/mkbootdisk").latin1(), X_OK) ||
		access( (NEWROOT +"/sbin/mkinitrd").latin1(), X_OK) )
  return NOSTUFFHERE;

if ( !WriteConfigFileFlag ) {
	checkPartitionDevNames();
	writeConfFile( (NEWROOT +"/etc/grub.conf").latin1(), a_addRunlevel5 );
}

/**///printf( "Checked mkbootdisk & mkinitrd\n" );
 while (stage < 4) {
	switch (stage) {
		case 1:
			switch( QMessageBox::information( /*( QWidget * )  */this, tr( "Bootdisk" ),
          tr( "A custom bootdisk provides a way of booting into your \n"
          "Linux system without relying on the normal bootloader. \n"
          "This is useful if you don't want to install GRUB on your \n"
          "system, another operating system removes GRUB, or GRUB \n"
          "doesn't work with your hardware configuration. A custom \n"
          "bootdisk can also be used with the SOT Linux rescue image, \n"
          "making it much easier to recover from severe system failures.\n"
          "\n"
          "Would you like to create a bootdisk for your system?")+ "\n",
					QMessageBox::Yes, QMessageBox::No, 0 ) ) {
				case QMessageBox::Yes: break;
				default: return NOBOOTDISKS; break;
			}
			stage = 2;
			break;
		case 2:
			switch( QMessageBox::information(  /*( QWidget * )*/this, tr("Bootdisk"),/*warning replaced with information (QMessageBox)*/
					tr("Insert a blank floppy into the first floppy drive."),
					 QMessageBox::Ok, QMessageBox::Cancel, 0 ) ) {
		  		case QMessageBox::Cancel: stage = 1; break;
				case QMessageBox::Ok: stage = 3; break;
				default: stage = 1;
			}
	    	break;
		case 3:
			qApp->setOverrideCursor( Qt::waitCursor );
			QString QSKernelName;
			QSKernelName = lastVersion( NEWROOT +"/boot", "2.4" );
			if ( QSKernelName.isEmpty() )
				QSKernelName = lastVersion( NEWROOT +"/boot", "2.2" );
			if ( QSKernelName.isEmpty() ) {
				QSKernelName = lastVersion( NEWROOT +"/boot", "2.0" );
				if ( QSKernelName.isEmpty() ){
					qApp->restoreOverrideCursor();
					return LINUXNOTFOUND;
				}
			}
			qApp->restoreOverrideCursor();
	    QApplication::syncX();
      QApplication::flushX();
//We must overwrite function lastVersion to meet current requirements or remove it at all
//it`s just dublicate function lastVersionName and returns not full names, like
//2.4.12-16 instead of 2.4.12-16server
#ifndef USE_NEW_KERNEL_DIVISION			
   		makeBootdisk( (char*)NEWROOT.latin1(), (char*)QSKernelName.latin1() );
#else
#ifdef GBLI_SERVER			
			makeBootdisk( (char*)NEWROOT.latin1(), (char*)(QSKernelName + "server").latin1() );
#else
			makeBootdisk( (char*)NEWROOT.latin1(), (char*)(QSKernelName + "desktop").latin1() );
#endif
#endif
//makeBootdisk( (char*)NEWROOT.latin1(), (char*)QSKernelName.latin1() );
   		stage++;
   		break;			
	}
 }

	chdir( "/" ); //Added for unmounting.
	
return 0;
}
//#endif












































