/***************************************************************************
                               installselect.h
                             -------------------
    begin                : Sat Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INSTALLSELECT_H
#define INSTALLSELECT_H

class LInlineHelp;

#include "config.h"
#include "gblipage.h"
#include "upgradepage.h"
#include "liconloader.h"

#include <alloca.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <popt.h>

//#include <depends.h>

#include <rpmlib.h>
#include <rpmmacro.h>

#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qwidget.h>
#include <qsocketnotifier.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>

#define MAX_INSTALL_TYPE 4

#define INST_CANCEL -1
#define INST_OK     0
#define INST_ERROR  1
#define INST_NOP    2

class PackageListItem;

// a logical entity which represents rpm in the system
class packageInfo {
public:
    int pos; // index in dependencies table
    Header h; // rpm header
    char * name, * group; // name and group name of the package
    bool selected; // is it selected or not; reflects to the list
    bool updatable; // true if package is newer than that in system or it's totally new
    int mustBe; // package which MUST BE installed or updated (Base and dependencies of base)
    bool inmenu; // show it in the menu or not; doens't have real effect anymore
    int size; // size of installed rpm; needed for hard drive space checking
    void * data;	/* this points to different things for different */
									/* install methods, e.g. filename */
		PackageListItem *item; // the item in the custom selection tree
		int iBase; // for tracking special groups 'System/Base'
};

// represents dependencies of the package, loaded from deps file
class dependenciesInfo{
public:
    QStringList dependOn; // a list of package names which some package depends on
};

// a collection of packages
class pkgSet {
public:
    packageInfo ** packages;// array of pointers to package information
					// note that package info is unique in the system,
					// so if you changed it in one place, it will change everywhere
    int numPackages; // number of packages in this set
};

// logical installation group (or type), e.g. Base, Typical
class component {
public:
    char * name; // group name
    bool selected; // is it selected
    bool inmenu; // is it showable in the menu
    pkgSet ps; // set of packages in this component
};

// collection of installation types
class componentSet {
public:
    component * comps;  // full components array, aka "Full", contains ALL packages
    component * base;   // pointer to component with name "Base"
#ifndef GBLI_SERVER
    component * typical;// pointer to component with name "Typical"
#else
		component * develop;// development platform
		component * server; // server
#endif
    int numComponents;
};

// Such a class returned by rpm installation function when it wants to
// install a package.
// It is passed to installCallback() function as a "data" variable,
// and that function should open real rpm file and pass back file handler
// See doInstall(), installCallback(), rpmRunTransactions()
class callbackInfo {
public:
    packageInfo * pkg; // package information
    int upgrade; // is it upgrade or install; seems to be not used
    FD_t fd; // file handler of open rpm
    char *lastFile; // real rpm name; together with version and ".rpm" at the end
    int totalnum, totalsize; // total number and size of packages to be installed
};

// provides package installation routines
// Main functions:
// loadData() - loads package descriptions from "hdlist" and "comps"
// verifyDependencies() - checks if selected package set contains all needed packages
// runInstall() - starts background installation of packages
// doInstall() - real package installation
class InstallSelect: public GBLIPage
{
    Q_OBJECT
public:
    InstallSelect( QWidget * parent, const char * name = 0 );
    ~InstallSelect() {}

		enum Type { undefined=-1, full=0,
#ifndef GBLI_SERVER
			typical,
#else
			develop, server,
#endif
			base, update=16 };
		
		enum Stage { notdefined=-1, initialized=0, install,	finished };

		int getInstallType() const { return selected; }
		static bool blUpdate;
		static bool blUpdateNP; // update not possible. rebuild of old rpm database failed or there are no partitions with Linux OS
		bool isCustom(){ return chkCustom->isChecked(); }

		::pkgSet *getPackages(){	return &ps;	};
		bool withX;
		int verifyDependencies();
		int verifyDependencies_(struct packageInfo *newPackage = 0, int mustBe = 0, int check = 0);
		int verifyDependencies__(struct packageInfo *newPackage = 0, int mustBe = 0);

/*
		int checkPackageDeps_(rpmTransactionSet ts, struct problemsSet * psp,
													Header h, const char * keyName, int multiLib);
		int rpmdepCheck_(rpmTransactionSet ts, struct rpmDependencyConflict ** conflicts, int *numConflicts);
		int unsatisfiedDepend_(rpmTransactionSet ts,
			const char * keyType, const char * keyDepend,
			const char * keyName, const char * keyEVR, int keyFlags,
			struct availablePackage ** suggestion);
		struct availablePackage * InstallSelect::alSatisfiesDepend_(
			struct availableList * al,
			const char * keyType, const char * keyDepend,
			const char * keyName, const char * keyEVR, int keyFlags);

		char *printDepend(const char * depend, const char * key,
											const char * keyEVR, int keyFlags);
		void alMakeIndex(struct availableList * al);
*/
		int loadData();
		componentSet cs;
		pkgSet ps;
		QArray<dependenciesInfo*> deps;

		bool eventFilter( QObject *o, QEvent *e );

		static bool keyOK;

		void selectPackages();
		int countSelected();

		bool isFinished(){ return iStage == finished; }
		bool isRunning(){  return iStage == install;  }
		void changeStage(Stage val) { iStage = val; }		
signals:
    void prevEnabled( QWidget *, bool );
    void nextEnabled( QWidget *, bool );

    void enableKeyboardPass( bool );

    void showProgressBar();
    void hideProgressBar();
    void resetProgress();
    void setTotalSteps(int);
    void setProgress(int);
    void setProgressLabel(const QString&);

    void installationFinished();

public slots:
		void runInstall();
    void setSelected( int );
		void finish() {}

protected:
		void showEvent( QShowEvent * );
    void hideEvent( QHideEvent * );

private slots:
    void showInstallationProgress( int fd );
		void showComFields( bool a_bl );

private:
		int getPackageSet( QString &a_strFileName );
		int getComponentSet( QString &a_strFileName );
		int getPackageSetFromSystem();
    int getDepsList( QString &a_strFileName );
		bool skipPackage( char * name );
		void pkgSort();
		void freeComponentSet();
		int doInstall();
		void createDirs();
		bool checkForX();	
		void checkFreeSpace();
		void renameConfModules();
		void afterUpdateScripts();

    QSocketNotifier* notifier;
    pid_t pid;

		Stage iStage;
    int selected;

    int currentProgress;
    int maxProgress;

    QButtonGroup * gb;
		QRadioButton * rbAll[ MAX_INSTALL_TYPE ];

		QCheckBox *chkCustom;

		QCheckBox *chkCom;

		QLineEdit *leComUser;
		QLabel *lblComUser;
		QLineEdit *leComEmail;
		QLabel *lblComEmail;
		QLineEdit *leComKey;
		QLabel *lblComKey;

		QLabel *lblComStatus;

		QString labels[ MAX_INSTALL_TYPE ];
};

#ifdef GBLI_STANDALONE
class KeyForBLC: public QDialog
{
public:
	KeyForBLC(QWidget*, const char*);
	bool eventFilter(QObject *o, QEvent *e);
private:
	QLineEdit *leComKey;
	QLabel *lblComKey;
	QPushButton *OKBut;
	QPushButton *CancelBut;
};
#endif

#endif

