/***************************************************************************
                               lmonitor.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "lmonitor.h"
#include "liconloader.h"
#include "language.h"

#include <qlayout.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qdict.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcolor.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include <ctype.h>

#include "ini_file.h"

LMonitor::LMonitor( QWidget * parent, const char * name )
    : GBLIPage( parent, name )
{
    vendors = 0;
    f = 0;
    ts = 0;
    recursionHack = TRUE;

		bgMode = new QButtonGroup();
		rbPredefined = new QRadioButton( tr( "Use typical monitor type" ), this );
		rbPredefined->setAutoMask( true );

		//Adding default selection
		rbPredefined->setFocus();

		bgMode->insert( rbPredefined, 0 );
		rbInf = new QRadioButton( tr( "Use windows 9X drivers" ), this );
		rbInf->setAutoMask( true );
		bgMode->insert( rbInf, 1 );
		rbCustom = new QRadioButton( tr( "Enter custom monitor details" ), this );
		rbCustom->setAutoMask( true );
		bgMode->insert( rbCustom, 2 );

    connect( bgMode, SIGNAL(clicked(int)), this, SLOT(setMode(int)) );

    monitors = new QListView( this );
  	QFont font( QFont::defaultFont() );
//		font.setFamily( "times" );
		font.setPointSize( Language::iSmallFontSize );	
    monitors->setFont( font );
    monitors->addColumn( tr( "Name" ), 100 );
    QString s( tr( "Size" ) );
    monitors->addColumn( s, monitors->fontMetrics().width( s ) + 16 );
    s = tr( "Horizontal Sync Range" );
    monitors->addColumn( s, monitors->fontMetrics().width( s ) + 16 );
    s = tr( "Vertical Sync Range" );
    monitors->addColumn( s, monitors->fontMetrics().width( s ) + 16 );
    monitors->setColumnWidthMode( 0, QListView::Maximum );
    monitors->setColumnWidthMode( 1, QListView::Maximum );
    monitors->setColumnAlignment( 1, AlignRight );
    monitors->setColumnWidthMode( 2, QListView::Maximum );
    monitors->setColumnWidthMode( 3, QListView::Maximum );
    monitors->setRootIsDecorated( TRUE );
    monitors->setAllColumnsShowFocus( TRUE );
    monitors->setFrameStyle( QFrame::WinPanel + QFrame::Sunken );

    connect( monitors, SIGNAL(selectionChanged()),
	     this, SLOT(monitorSelected()) );

		customPanel = new QWidget( this );

    monitorNameEdit = new QLineEdit( customPanel );
//		customPanel->setAutoMask( true );
		customPanel->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );
    QVBoxLayout * lvC = new QVBoxLayout( customPanel, 6 );

    QGridLayout * g2 = new QGridLayout( 4, 2 );
		lvC->addStretch( 1 );
    lvC->addLayout( g2 );
		lvC->addStretch( 4 );

    g2->setRowStretch( 6, 42 );
		QLabel *lab = new QLabel( tr( "Monitor name (optional):" ), customPanel );
		lab->setAutoMask( true );
    g2->addWidget( lab, 0,0 );
		lab = new QLabel( tr( "Horizontal sync range in kHz:" ), customPanel );
		lab->setAutoMask( true );
    g2->addWidget( lab, 1,0);
		lab = new QLabel( tr( "Vertical sync range in Hz:" ), customPanel );
		lab->setAutoMask( true );
		g2->addWidget( lab, 2,0);

    g2->addMultiCellWidget( monitorNameEdit, 0, 0, 1, 2 );

    hMinBox = new QSpinBox( 0, 200, 1, customPanel );
    hMinBox->setValue( 31 );
    hMaxBox = new QSpinBox( 0, 200, 1, customPanel );
    hMaxBox->setValue( 64 );
    vMinBox = new QSpinBox( 18, 300, 1, customPanel );
    vMinBox->setValue( 50 );
    vMaxBox = new QSpinBox( 18, 300, 1, customPanel );
    vMaxBox->setValue( 90 );
    g2->addWidget( hMinBox, 1, 1 );
    g2->addWidget( hMaxBox, 1, 2 );
    g2->addWidget( vMinBox, 2, 1 );
    g2->addWidget( vMaxBox, 2, 2 );
		
		customPanel->hide();

    connect( hMinBox, SIGNAL(valueChanged(int)),
	     this, SLOT(spinBoxChanged(int)) );
    connect( hMaxBox, SIGNAL(valueChanged(int)),
	     this, SLOT(spinBoxChanged(int)) );
    connect( vMinBox, SIGNAL(valueChanged(int)),
	     this, SLOT(spinBoxChanged(int)) );
    connect( vMaxBox, SIGNAL(valueChanged(int)),
	     this, SLOT(spinBoxChanged(int)) );

    connect( monitorNameEdit, SIGNAL(textChanged(const QString &)),
	     this, SLOT(lineEditChanged(const QString &)) );

		QBoxLayout * loMode = new QBoxLayout( QBoxLayout::Up );
		loMode->addWidget( monitors );
		loMode->addWidget( customPanel );
    QHBoxLayout * lh = new QHBoxLayout( this, 5, 6 );
    l = new QVBoxLayout( 6 );

		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		l->addSpacing( 10 );
		l->addWidget( rbPredefined );
		l->addWidget( rbInf );
		l->addWidget( rbCustom );
		l->addSpacing( 5 );
    l->addLayout( loMode, 100 );
		l->addSpacing( 10 );
		lh->addStretch( 1 );

/*    LInlineHelp* inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "monitor_overview.qml" );
    lh->addWidget( inlineHelp );*/

    monitors->setFocus();
    initialized = FALSE;
    recursionHack = FALSE;
		iCurMode = 0;
    rbPredefined->setChecked( true );
}

LMonitor::~LMonitor()
{
    delete vendors;
    vendors = 0;
    delete f;
    f = 0;
    delete ts;
    ts = 0;
}


void LMonitor::showEvent( QShowEvent*)
{
    if (!initialized) {
			readMonitors();
			initialized = TRUE;
    }
}

int LMonitor::getMountAnswer( char *data, int len )
{
  char dst[len+1];
  memmove(dst,data,len);
  dst[len]=0;
  if ( strstr( dst, "not a valid block device" ) ) return -1;
  if ( strstr( dst, "wrong fs type" ) ) return -2;
	if ( strstr( dst, "Device not configured" ) ) return -3;
  return 0;
}

void LMonitor::setMode( int a_iMode )
{
	bool ok = true;
	int res = -1;
	switch( a_iMode ){
		case 0:
			if ( !initialized || iCurMode == 2 ) {
				readMonitors();
				initialized = TRUE;
			}
			monitors->show();
			customPanel->hide();
			if( !rbPredefined->isChecked() )
		    rbPredefined->setChecked( true );
				//Adding selection redraw
				rbPredefined->setFocus();
			break;
		case 1:
			initialized = false;
			do{
				res = system( "mount /dev/fd0 /mnt/floppy -t msdos" );
				if( res )
					ok = QMessageBox::critical( this, tr("Error!"), tr("Could not mount floppy disc or disc is wrong!"), tr("Retry"), tr("Cancel") );
			}while( res && !ok );
			if( !res ) {
				QString filename = QFileDialog::getOpenFileName( "/mnt/floppy", "*.inf" );
				ReadMonitorsTypeFromINF( filename );	
				system("umount /dev/fd0" );
				monitors->show();
				customPanel->hide();
				//Adding selection redraw
				rbInf->setChecked(true);
				rbInf->setFocus();
			}else{
				if( iCurMode < a_iMode ){
			    rbCustom->setChecked( true );
					//Adding redraw selection
					rbCustom->setFocus();
					setMode( 2 );
				}else{
			    rbPredefined->setChecked( true );
          //Adding selection redraw
  				rbPredefined->setFocus();
					setMode( 0 );
				}
				return;
			}
			break;
		default:
//			initialized = false;
			monitors->hide();
			customPanel->show();
			if( !rbCustom->isChecked() )
		    rbCustom->setChecked( true );
        //Adding redraw selection
				rbCustom->setFocus();
			break;
	}
	iCurMode = a_iMode;
}

LMonitorType * LMonitorType::standardMonitors = 0;


LMonitorType::LMonitorType( QListView * parent, const QString & text )
    : QListViewItem( parent )
{
    t = text;
    setSelectable( FALSE );
}


QString LMonitorType::text( int column ) const
{
    return column ? QString::null : QString( t );
}


QString LMonitorType::key( int, bool ascending ) const
{
    if ( this == standardMonitors )
			return ascending ? QString( " " ) : QString( "\377" );
    return text( 0 ).lower();
}

void LMonitorType::activate()
{
    setOpen( TRUE );
    QListViewItem * i = firstChild();
    QListView * lv = listView();
    if ( i && lv )
	lv->setSelected( i, TRUE );
}


// multisync monitors

class LMonitorModel: public QListViewItem
{
public:
    LMonitorModel( LMonitorType *, const QString &,
		   float, float, float, float, int );

    QString text( int ) const;
    QString key( int, bool ) const;
    virtual bool supportsHorizontalFrequency( float );
    virtual bool supportsVerticalFrequency( float );

    int size() const { return s; }

private:
    QString m;
    float horizMin, horizMax, vertMin, vertMax;
    int s;

    friend class LMonitor;
};


LMonitorModel::LMonitorModel( LMonitorType * parent,
			      const QString & modelName,
			      float hmin, float hmax, float vmin, float vmax,
			      int size )
    : QListViewItem( parent )
{
    m = modelName;
    horizMin = hmin;
    horizMax = hmax;
    vertMin = vmin;
    vertMax = vmax;
    s = size;
    if ( s < 10 || s > 25 )
	s = 0;
}


bool LMonitorModel::supportsHorizontalFrequency( float f )
{
    return f >= horizMin && f <= horizMax;
}


bool LMonitorModel::supportsVerticalFrequency( float f )
{
    return QABS(f) >= vertMin && QABS(f) <= vertMax;
}


QString LMonitorModel::text( int column ) const
{
    if ( column == 0 )
	return m;

    QString tmp;
    if ( column == 1 && s == 0 )
	tmp = "? ";
    else if ( column == 1 )
	tmp.sprintf( "%d ", s );
    else if ( column == 2 && horizMin == horizMax )
	tmp.sprintf( "%g", horizMin );
    else if ( column == 2 )
	tmp.sprintf( "%g-%g", horizMin, horizMax );
    else if ( column == 3 && vertMin == vertMax )
	tmp.sprintf( "%g", vertMin );
    else if ( column == 3 )
	tmp.sprintf( "%g-%g", vertMin, vertMax );
    return tmp;
}


QString LMonitorModel::key( int column, bool ) const
{
    QString result = text( column );
    if ( result.length() > 0 && result[0].isDigit()) {
	uint i = 0;
	while( i < result.length() && result[i].isDigit() )
	    i++;
	while ( i++ < 5 )
	    result.prepend( "0" );
    }
    return result.lower();
}


void LMonitor::readMonitors()
{
		monitors->clear();

    // this comes from XFree86
    LMonitorType * misc	= new LMonitorType( monitors, tr( "Typical monitors" ) );
    misc->setOpen( TRUE );
    (void)new LMonitorModel( misc, "640x480, 60Hz",
			     31.5, 31.5, 60, 60, 14 );
    (void)new LMonitorModel( misc, "800x600, 56Hz",
			     31.5, 35.1, 55, 60, 14 );
    (void)new LMonitorModel( misc, "800x600, 60Hz", 31.5, 37.9, 55, 90, 14 );
    (void)new LMonitorModel( misc, "1024x768, 60Hz", 31.5, 48.5, 55, 90, 15 );
    monitors->setSelected( new LMonitorModel( misc, "1024x768, 70Hz", 31.5, 57, 50, 90, 15 ), TRUE );
    (void)new LMonitorModel( misc, "1280x1024, 60Hz", 31.5, 64.3, 50, 90, 16 );
    (void)new LMonitorModel( misc, "1280x1024, 74Hz", 31.5, 79, 50, 100, 17 );
    (void)new LMonitorModel( misc, "1280x1024, 76Hz", 31.5, 82, 40, 100, 17 );

    LMonitorType::standardMonitors = misc;

    vendors = new QDict<LMonitorType>( 61 );
    vendors->setAutoDelete( FALSE );
    f = new QFile( QString( GBLI_DATA_PATH ) +"/data/Monitors.jumbled" );
    if ( !f->open( IO_ReadOnly ) ) {
			f->setName( QString( GBLI_DATA_PATH ) +"/data/Monitors" );
			if ( !f->open( IO_ReadOnly ) ) {
	  	  return;
			}
    }
    ts = new QTextStream( f );
//    QTimer::singleShot( 10, this, SLOT(readSomeMonitors()) );
		readSomeMonitors();
};


void LMonitor::readSomeMonitors() {
    QString vendor;
    QString model;
    QString hf;
    QString vf;
    QString size;
    int count = 0;
    bool norepeated = TRUE;
    while( !ts->eof() && ( count < 50 || norepeated ) ) {
			QString line = ts->readLine();
			if ( line != "" &&
	  	  vendor.length() && model.length() &&
	     hf.length() && vf.length() ) {
	    float hmin, hmax, vmin, vmax;
	    int tmp = sscanf( hf, "%f-%f", &hmin, &hmax );
	    if ( tmp < 2 ) {
		tmp = sscanf( hf, "%f", &vmin );
		if ( tmp == 1 ) {
		    hmax = hmin;
		    tmp = 2;
		}
	    }
	    if ( tmp == 2 ) {
		vmin = 42;
		tmp = sscanf( vf, "%f-%f", &vmin, &vmax );
		if ( tmp < 2 ) {
		    tmp = sscanf( vf, "%f", &vmin );
		    if ( tmp == 1 ) {
			vmax = vmin;
			tmp = 2;
		    }
		}
		if ( tmp == 2 ) {
		    LMonitorType * vendorItem = vendors->find( vendor );
		    if ( vendorItem ) {
			// found first "second monitor by one vendor"
			norepeated = FALSE;
		    } else {
			vendorItem = new LMonitorType( monitors, vendor );
			vendors->insert( vendor, vendorItem );
		    }
		    new LMonitorModel( vendorItem, model,
				       hmin, hmax, vmin, vmax, size.toInt() );
		    count++;
		}
	    }
	    hf = vf = model = vendor = size = "";
	}
	int i = line.find( '#' );
	if ( i > -1 )
	    line.truncate( i );
	line = line.simplifyWhiteSpace();
	i = line.find( " " );
	if ( i >= 0 ) {
	    QString firstWord, arg;
	    firstWord = line.left( i ).lower();
	    arg = line.mid( i+1, line.length() );
	    if ( firstWord == "vendor" )
		vendor = arg;
	    else if ( firstWord == "model" )
		model = arg;
	    else if ( firstWord == "size" )
		size = arg;
	    else if ( firstWord == "horizsync" )
		hf = arg;
	    else if ( firstWord == "vertrefresh" || firstWord == "verrefresh" )
		vf = arg;
	}
    }
    if ( count >= 50 ) {
//	QTimer::singleShot( 10, this, SLOT(readSomeMonitors()) );
			readSomeMonitors();
    } /*else {
	delete vendors;
	vendors = 0;
	delete f;
	f = 0;
	delete ts;
	ts = 0;
    }*/
}


void LMonitor::spinBoxChanged( int )
{
    if ( recursionHack )
	return;
    hMaxBox->setRange( hMinBox->value(), hMaxBox->maxValue() );
    hMinBox->setRange( hMinBox->minValue(), hMaxBox->value() );
    vMaxBox->setRange( vMinBox->value(), vMaxBox->maxValue() );
    vMinBox->setRange( vMinBox->minValue(), vMaxBox->value() );
    if ( !monitors->selectedItem() )
	return;
    LMonitorModel * m = ((LMonitorModel*)monitors->selectedItem());

    if ( (int)m->horizMin != hMinBox->value() ||
	 (int)m->horizMax != hMaxBox->value() ||
	 (int)m->vertMin != vMinBox->value() ||
	 (int)m->vertMax != vMaxBox->value() )
	monitors->clearSelection();
}


QString bestPrefixHack( QListViewItem * i, const QString & s)
{
    QString r = QString::null;
    QString l = s.lower();
    while ( i ) {
	QString tmp = i->text( 0 ).simplifyWhiteSpace();
	if ( tmp.length() > r.length() &&
	     ( tmp[r.length()] == ' ' || r.length() == 0 ) &&
	     tmp.lower() == l.mid( 0, tmp.length() ) )
	    r = tmp;
	i = i->nextSibling();
    }
    return r;
}


QString LMonitor::vendorName()
{
    QString tmp = ::bestPrefixHack( monitors->firstChild(),
				    monitorNameEdit->text() );
    if ( tmp != QString::null )
			return tmp;

    return QString( "(Entered by hand)" );
}


QString LMonitor::modelName()
{
    QString tmp = ::bestPrefixHack( monitors->firstChild(),
				    monitorNameEdit->text() );
    if ( tmp == QString::null )
	tmp = monitorNameEdit->text();
    else
	tmp = monitorNameEdit->text().mid( tmp.length(), 32767 );
    return tmp.simplifyWhiteSpace();
}


QString LMonitor::horizSync()
{
    QString h;

    if ( monitors->selectedItem() ) {
	LMonitorModel * m = ((LMonitorModel*)monitors->selectedItem());
	if ( m->horizMin == m->horizMax )
	    h.sprintf( "%g", m->horizMin );
	else
	    h.sprintf( "%g-%g", m->horizMin, m->horizMax );
    } else {
	if ( hMinBox->value() == hMaxBox->value() )
	    h.sprintf( "%d", hMinBox->value() );
	else
	    h.sprintf( "%d-%d", hMinBox->value(), hMaxBox->value() );
    }
    return h;
}


QString LMonitor::vertRefresh()
{
    QString v;
    if ( monitors->selectedItem() ) {
	LMonitorModel * m = ((LMonitorModel*)monitors->selectedItem());
	if ( m->vertMin == m->vertMax )
	    v.sprintf( "%g", m->vertMin );
	else
	    v.sprintf( "%g-%g", m->vertMin, m->vertMax );
    } else {
	if ( vMinBox->value() == vMaxBox->value() )
	    v.sprintf( "%d", vMinBox->value() );
	else
	    v.sprintf( "%d-%d", vMinBox->value(), vMaxBox->value() );
    }
    return v;
}


bool LMonitor::supportsHorizontalFrequency( float f )
{
    LMonitorModel * model = ((LMonitorModel*)monitors->selectedItem());
    if ( model )
	return model->supportsHorizontalFrequency( f );
    return f >= hMinBox->value() && f <= hMaxBox->value();
}


bool LMonitor::supportsVerticalFrequency( float f )
{
    LMonitorModel * model = ((LMonitorModel*)monitors->selectedItem());
    if ( model )
	return model->supportsVerticalFrequency( f );
    return f >= vMinBox->value() && f <= vMaxBox->value();
}


void LMonitor::lineEditChanged( const QString & newString )
{
    monitor = newString;
    if ( monitorNameEdit->edited() )
	monitors->clearSelection();
}


int LMonitor::size()
{
    if ( monitors && monitors->selectedItem() )
	return ((LMonitorModel *)monitors->selectedItem())->size();
    return 15;
}


void LMonitor::monitorSelected()
{
    if ( !monitors || !monitors->selectedItem() ||
	 !monitors->selectedItem()->parent() )
	return;

    LMonitorModel * m = (LMonitorModel*)(monitors->selectedItem());

    monitorNameEdit->setEdited( FALSE );
    monitorNameEdit->setText( m->parent()->text( 0 ) + " " + m->text( 0 ) );

    recursionHack = TRUE;
    hMinBox->setValue( (int) m->horizMin );
    hMaxBox->setValue( (int) m->horizMax );
    vMinBox->setValue( (int) m->vertMin );
    vMaxBox->setValue( (int) m->vertMax );
    recursionHack = FALSE;
    spinBoxChanged( 42 );
}

//void LMonitor::setMonitor( QString name, int a_iHorzMin, int a_iHorzMax, int a_iVertMin, int a_iVertMax ){
//    monitorNameEdit->setText( name );
//
//    recursionHack = TRUE;
//    hMinBox->setValue( (int) a_iHorzMin );
//    hMaxBox->setValue( (int) a_iHorzMax );
//    vMinBox->setValue( (int) a_iVertMin );
//    vMaxBox->setValue( (int) a_iVertMax );
//    recursionHack = FALSE;
//		iCurMode = 2;
//    rbCustom->setChecked( true );
////    spinBoxChanged( 42 );  ????
//}

void LMonitor::setMonitor( QString sMonitorVendor, QString sMonitorModel, int a_iHorzMin, int a_iHorzMax, int a_iVertMin, int a_iVertMax )
{
	LMonitorType*		MonitorType;
	if (vendors==0) return;
	MonitorType = vendors->find((const char*)sMonitorVendor);
	
	if (MonitorType!=NULL)
	{
		QListViewItem*		Item;
	
		MonitorType->activate();
		Item = MonitorType->firstChild();
		while (Item)
		{
			QString		sModelName = Item->text(0);
			
			if (sModelName.lower()==sMonitorModel.lower())
			{
				setMode(0);
				monitors->setSelected(Item, TRUE);
				monitors->setCurrentItem(Item);
				monitors->ensureItemVisible(Item);
		    monitorSelected();
		    return;
			}
			Item = Item->nextSibling();
		};
	}

    recursionHack = TRUE;
    monitorNameEdit->setText( sMonitorVendor + " " + sMonitorModel );
    hMinBox->setValue( (int) a_iHorzMin );
    hMaxBox->setValue( (int) a_iHorzMax );
    vMinBox->setValue( (int) a_iVertMin );
    vMaxBox->setValue( (int) a_iVertMax );
    recursionHack = FALSE;
    spinBoxChanged( 42 );
		setMode( 2 );
}


int LMonitor::RetMinMaxSyncRange( QString a_QSFileName, QString a_QSModelKey, float *a_pfHorMin, float *a_pfHorMax, float *a_pfVertMin, float *a_pfVertMax )
{
	IniFile inifile;
	char szData[ 256 ];
	unsigned char bModeFound = 0;
	float fHorizontalMin = 999, fHorizontalMax = 0, fVerticalMin = 999, fVerticalMax = 0;

	QString QSModelKeyReal = a_QSModelKey + ".AddReg";	
	while ( inifile.ProfileStrEnum( a_QSFileName.ascii(), QSModelKeyReal.ascii(), szData ) > 0 ) {
		QString QSData = szData;
		if ( QSData.contains( "Mode", TRUE ) ) {
			bModeFound = 1;
		        QString QSSyncRange = QSData.mid( QSData.find( ",,", TRUE ) + 3, QSData.length() - QSData.find( ",,", TRUE ) - 4 );
			int iFirstBreak = QSSyncRange.find( ',', TRUE );
			QString QSHorizSyncRange = QSSyncRange.left( iFirstBreak );
			QString QSVertSyncRange = QSSyncRange.mid( iFirstBreak + 1, QSSyncRange.find( ',', iFirstBreak + 1, TRUE ) - iFirstBreak - 1 );

			if ( QSHorizSyncRange.contains( "-", TRUE ) && QSVertSyncRange.contains( "-", TRUE ) ) {
				int iDelimPos = QSHorizSyncRange.find( '-', TRUE );

	                        float fTmpHorMinVal = ( unsigned short )( QSHorizSyncRange.left( iDelimPos ) ).toFloat();
				if ( fTmpHorMinVal < fHorizontalMin ) fHorizontalMin = fTmpHorMinVal;
				float fTmpHorMaxVal = ( unsigned short )( QSHorizSyncRange.right( QSHorizSyncRange.length() - 1 - iDelimPos ) ).toFloat();
      				if ( fTmpHorMaxVal > fHorizontalMax ) fHorizontalMax = fTmpHorMaxVal;

				iDelimPos = QSVertSyncRange.find( '-', TRUE );
				float fTmpVertMinVal = ( unsigned short )( QSVertSyncRange.left( iDelimPos ) ).toFloat();
				if ( fTmpVertMinVal < fVerticalMin ) fVerticalMin = fTmpVertMinVal;
				float fTmpVertMaxVal = ( unsigned short )( QSVertSyncRange.right( QSVertSyncRange.length() - 1 - iDelimPos ) ).toFloat();
				if ( fTmpVertMaxVal > fVerticalMax ) fVerticalMax = fTmpVertMaxVal;

			} else {
				float fTmpHorVal = ( unsigned char )QSHorizSyncRange.toFloat();
				if ( fTmpHorVal < fHorizontalMin ) fHorizontalMin = fTmpHorVal;
				if ( fTmpHorVal > fHorizontalMax ) fHorizontalMax = fTmpHorVal;				

				float fTmpVertVal = ( unsigned char )QSVertSyncRange.toFloat();
				if ( fTmpVertVal < fVerticalMin ) fVerticalMin = fTmpVertVal;
				if ( fTmpVertVal > fVerticalMax ) fVerticalMax = fTmpVertVal;
			}
		}
	}
	if ( bModeFound ) {
		*a_pfHorMin = fHorizontalMin;
		*a_pfHorMax = fHorizontalMax;
		
		*a_pfVertMin = fVerticalMin;
		*a_pfVertMax = fVerticalMax;
		return 0;
	}
	return -1;
}

int LMonitor::ReadMonitorsTypeFromINF( QString a_QSFileName )
{
	IniFile inifile;
	char szKey[ 256 ], szValue[ 256 ];
	char szRetManufName[ 256 ];

	//Search if user insert drivers disk with monitor inf file, else return with error.
	inifile.ProfileStr( a_QSFileName, "Version", "Class", "None", szValue );
	if ( strcmp( szValue, "Monitor" ) ) return -1;


	//Read manufacturer name and return it
	if ( inifile.ProfileEnum( a_QSFileName, "Manufacturer", szKey, szRetManufName ) <= 0 )
		inifile.ProfileStr( a_QSFileName, "Version", "NameProvider", "Unknown", szRetManufName );

	vendors = new QDict<LMonitorType>( 61 );
  vendors->setAutoDelete( FALSE );

	monitors->clear();
	LMonitorType *vendorItem = new LMonitorType( monitors, szRetManufName );
	vendorItem->setOpen( TRUE );
	vendors->insert( szRetManufName, vendorItem );

	//Read model name and horiz/vert sync ranges
	int iStat;
	while ( ( iStat = inifile.ProfileEnum( a_QSFileName, "Strings", szKey, szValue ) ) != -1 ){
		if ( strcmp( szKey, "MonitorClassName" ) && strcmp( szKey, szRetManufName ) && iStat ) {

			float fHorMin = 0, fHorMax = 0, fVertMin = 0, fVertMax = 0;
			QString QSModelName = szValue;
			QSModelName = QSModelName.simplifyWhiteSpace();
			QSModelName = QSModelName.mid( 1, QSModelName.find( '\"', 2 ) - 1 );

			// If model name contain Manufacturer name remove it from model name
			if ( QSModelName.find( szRetManufName ) == 0 ) {
			//I.e. first position
				QSModelName = QSModelName.remove( 0, strlen( szRetManufName ) );
				QSModelName = QSModelName.simplifyWhiteSpace();
			}

      RetMinMaxSyncRange( a_QSFileName, szKey, &fHorMin, &fHorMax, &fVertMin, &fVertMax );

			new LMonitorModel( vendorItem, QSModelName, fHorMin, fHorMax, fVertMin, fVertMax, 0 );
		}
	}
	delete vendors;
	//delete vendorItem;
	return 0;
}
