/***************************************************************************
                               lsound.cpp
                             -------------------
    begin                : Thu Aug 01 2000
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Natalja Drozzina
    author email         : nd@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"

#include <stdio.h>
#include <stdlib.h> 
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <fcntl.h> 
#include <linux/soundcard.h>
#include <unistd.h>
#include <string.h>

#include <qstring.h>
#include <qstrlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qaccel.h>
#include <qmessagebox.h>


#include "linlinehelp.h"
#include "lsound.h"
#include "liconloader.h"
#include "log.h"

#define SOUND_BY_SNDCONFIG

/* buses */
#define isa             1
#define pci             (1<<2)
#define sbus            (1<<3)

/* Card flags... */
#define HAS_OPL3        1
#define HAS_MPU401      (1<<1)
#define SB_AS_LIB       (1<<2)
#define LOAD_CS4232     (1<<3)
#define SM_GAMES        (1<<4)
#define ACER            (1<<5)
#define JOYSTICK        (1<<6)
#define NO_INPUT_REQ    (1<<7)
#define COMPAQ          (1<<8)
#define ACI             (1<<9)
#define NEEDS_SOX       (1<<10)
#define HAS_AWE         (1<<11)
#define SOUNDPRO        (1<<12)

#define _(String) gettext((String))
#define N_(String) (String)
/* Typing all this out is a pain... */
#define STANDARD_IRQ    {5,7,9,10,-1}
#define LOW_DMA         {0,1,3,-1}
#define SB16_IO         {0x220,0x240,0x260,0x280,-1}
#define WSS_IO          {0x530,0x534,0x604,0xE80,0xF40,-1}
#define MPU401_IO       {0x330,0x300,-1}

#ifdef OLD_SNDCONFIG_LOWER_THAN_0_57
enum cards{
    ACERNB, AD1816, ADLIB, ALIM15XX,ALS007, ALS100, SGALAXY, CMI8330, CMI8X38, DESKPROXL,
    CS4232, CS46XX, CS461X, ES1370,ES1371, SOUNDSCAPE, SSVIVO, ESS688, ESS1688, ESS1868,
    ESSMAESTRO, ESSSOLO1, GUS, GUSMAX, GUSPNP, I810, SMGAMES, AUDIOTRIX, JAZZ16,
    MOZART, MAD16PRO, MIROSOUND, NEOMAGIC, OPL3_SA, OPL3_SAX,
    PAS16, PSS, SONICVIBES, SBORIG, SBPRO, SB16, SB32, TRIDENT,
    MSND_CLASSIC, MSND_PINNACLE, /* WAVEFRONT, */ VIA82CXXX, WSS, YMFPCI,
    AMD7930, CS4231, DBRI, UNSPEC
    };
#else
enum cards {
    ACERNB, AD1816, ADLIB, ALIM15XX, ALS007, ALS100, SGALAXY, CMI8330,
    CMI8X38, DESKPROXL, CS4232, CS46XX, CS461X, CS4281, ES1370, ES1371, SOUNDSCAPE,
    SSVIVO, ESS688, ESS1688, ESS1868, ESSMAESTRO, ESSMAESTRO3, ESSSOLO1, GUS,
    GUSMAX, GUSPNP, I810, SMGAMES, AUDIOTRIX, JAZZ16, MOZART, MAD16PRO,
    MIROSOUND, NEOMAGIC, OPL3_SA, OPL3_SAX, PAS16, PSS, SONICVIBES,
    SBORIG, SBPRO, SB16, SB32, SBLIVE, TRIDENT,
    MSND_CLASSIC, MSND_PINNACLE, WAVEFRONT, VIA82CXXX, WSS,
    YMFPCI, AMD7930, CS4231, DBRI, UNSPEC
};
#endif
//sondcard's base from sndconfig

struct SoundCardsTable {
    enum cards type;
    int irq[10];
    int io[10];
    int dma[10];
    int dma2[10];
    int mpu_io[10];
    int mpu_irq[10];
    int mem[10];
    int  flags;
    char modulename[32];
    char descr[256];
    int messageno;
    int bus;
    int avail;
  };

#ifdef OLD_SNDCONFIG_LOWER_THAN_0_57
struct SoundCardsTable cardsArray[] = {

    {ACERNB, {5, 7, 9, 10, -1}, SB16_IO, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3 | ACER, "sb", N_("ACER Notebook sound"), 0, isa, 0},
    {AD1816, {5, 7, 9, 10, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3, "ad1816", N_("AD1816 (Acer FX-3D, HP Kayak)"), 0, isa, 0},
    {ADLIB, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "adlib_card", N_("AdLib"), 0, isa, 0},

// Added by bp from sndconfig 0.57
    {ALIM15XX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "ali_m1553",
     N_("ALI M15xx Chipset Audio"), 0, pci, 0},
// Added by bp from sndconfig 0.57

    {ALS007, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "sb", N_("Advance Logic ALS-007"), 0, isa, 0},
    {ALS100, STANDARD_IRQ, SB16_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "sb", N_("Advance Logic ALS-100/110"), 0, isa, 0},
    {SGALAXY, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, SB16_IO,
     HAS_OPL3, "sgalaxy",
     N_("Aztech Sound Galaxy Waverider Pro 32-3D, Washington 16"), 0, isa,
     0},
    {CMI8330, {5, 7, 9, 10, 11, -1}, WSS_IO, LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_MPU401 | HAS_OPL3 | SOUNDPRO, "ad1848",
     N_("CMI8330 sound chip (many motherboards)"), 0, isa, 0},
    {CMI8X38, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "cmpci", N_("C-Media PCI Audio (8338, 8738)"), 4, pci,
     0},
    {DESKPROXL, {7, 9, 10, 11, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3 | COMPAQ, "ad1848", N_("Compaq Deskpro XL Sound"), 0, isa,
     0},
    {CS4232, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "cs4232", N_("Crystal CS423x sound chip"), 0, isa, 0},

// Added by bp from sndconfig 0.57
    {CS46XX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "cs46xx",
     N_("Crystal SoundFusion CS46xx"), 0, pci, 0},
// Added by bp from sndconfig 0.57


// Added by bp from 0.65
    {CS4281, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "cs4281",
     N_("Crystal CS4281"), 0, pci, 0},
// Added by bp from 0.65
// for IBM laptop
    {CS461X, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "cs46xx",
     N_("Crystal 4280/461x + AC97 Audio"), 0, pci, 0},
//

    {ES1370, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "es1370",
     N_("Ensoniq AudioPCI 1370 (SoundBlaster 64/128 PCI)"), 0, pci, 0},
    {ES1371, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "es1371",
     N_("Creative/Ensoniq AudioPCI 1371"), 0, pci, 0},
    {SOUNDSCAPE, {5, 7, 9, 10, 15, -1}, WSS_IO, LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     0, "sscape", N_("Ensoniq SoundScape"), 0, isa, 0},
    {SSVIVO, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_MPU401, "ad1848", N_("Ensoniq SoundScape VIVO"), 0, isa, 0},
    {ESS688, {5, 7, 9, 10, -1}, SB16_IO, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS688 AudioDrive"), 0, isa, 0},
    {ESS1688, {5, 7, 9, 10, -1}, {0x220, 0x230, 0x240, 0x250, -1}, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS1688 AudioDrive"), 0, isa, 0},
    {ESS1868, {5, 7, 9, 10, -1}, {0x220, 0x230, 0x240, 0x250, -1}, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS1868 AudioDrive"), 0, isa, 0},

// Added by bp from sndconfig 0.57
    {ESSMAESTRO, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "maestro", N_("ESS Maestro 1/2/2E"), 0, pci, 0},

//    {ESSMAESTRO, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
//     NO_INPUT_REQ, "maestro", N_("ESS Maestro 1/2/2E"), 6, pci, 0},
// Added by bp from sndconfig 0.57

// by bp from 0.65
    {ESSMAESTRO3, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "maestro3", N_("ESS Maestro 3/Allegro"), 0, pci, 0},
// by bp from 0.65
    {ESSSOLO1, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "esssolo1", N_("ESS Solo 1"), 0, pci, 0},

    {GUS, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
      {-1}, {-1}, {-1}, {-1},
       0, "gus", N_("Gravis UltraSound"), 0, isa, 0},

	{GUSMAX, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
	 {1, 3, 5, 6, 7, -1}, {-1}, {-1}, {-1},
	 0, "gus", N_("Gravis UltraSound MAX"), 0, isa, 0},

	{GUSPNP, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
	 {1, 3, 5, 6, 7, -1}, {-1}, {-1}, {-1},
	 0, "gus", N_("Gravis UltraSound PnP"), 0, isa, 0},

// Added by bp from sndconfig 0.57
    {I810, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "i810_audio", N_("Intel i810 AC97 Audio"), 0, pci, 0},
// Added by bp from sndconfig 0.57

    {SMGAMES, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3 | SM_GAMES, "sb",
     N_("Logitech SoundMan Games (not SM16 or SM Wave!)"), 0, isa, 0},
    {AUDIOTRIX, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, {5, 7, 9, 10, 11, 12, 15, -1}, {-1},
     HAS_OPL3 | SB_AS_LIB, "trix", N_("MediaTrix AudioTrix Pro"), 0, isa,
     0},
    {JAZZ16, STANDARD_IRQ, {0x220, 0x240, 0x260, -1}, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, {0x310, 0x320, 0x330, -1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("MediaVision Jazz16 (ProSonic, SoundMan Wave)"), 0,
     isa, 0},
    {MOZART, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3 | SB_AS_LIB | JOYSTICK, "mad16",
     N_("Mozart/MAD16 (OPTi 82C928)"), 0, isa, 0},
    {MAD16PRO, STANDARD_IRQ, WSS_IO, LOW_DMA,
     {0, 1, 3, 5, 6}, {0x300, 0x310, 0x320, 0x330, -1}, STANDARD_IRQ, {-1},
     HAS_OPL3 | SB_AS_LIB | JOYSTICK, "mad16",
     N_("MAD16 Pro (OPTi 82C929/82C930)"), 0, isa, 0},
    {MIROSOUND, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     ACI, "mad16", N_("miroSOUND PCM12"), 0, isa, 0},
    {NEOMAGIC, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "nm256", N_("NeoMagic MagicMedia 256AV"), 7, pci, 0},
    {OPL3_SA, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "opl3sa", N_("OPL3-SA1 sound chip"), 0, isa, 0},
    {OPL3_SAX, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, {-1}, {0x370, 0x380, -1},
     HAS_OPL3, "opl3sa2", N_("OPL3-SA2/3/x sound chip"), 0, isa, 0},

	{PAS16, {3, 5, 7, 9, 10, 11, 12, -1}, {0x220, 0x230, 0x240, -1},
     {0, 1, 2, 3, 5, 6, 7, -1},
	 {-1}, {-1}, {-1}, {-1},
	 SB_AS_LIB | JOYSTICK, "pas2",
	 N_("Pro Audio Spectrum/Studio 16, Logitech SoundMan 16"), 0, isa,
	 0},
    {PSS, {3, 5, 7, 9, 10, 11, 12, -1}, {0x220, 0x230, 0x240, 0x250, -1},
     LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     0, "pss", N_("PSS (Orchid SW32, Cardinal DSP16)"), 0, isa, 0},
    {SONICVIBES, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "sonicvibes",
     N_("S3 SonicVibes"), 0, pci, 0},
    {SBORIG, {3, 5, 7, 9, -1}, {0x220, 0x240, -1}, {1, -1},
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster"), 0, isa, 0},
    {SBPRO, STANDARD_IRQ, {0x220, 0x240, -1}, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster Pro"), 0, isa, 0},
    {SB16, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster 16"), 0, isa, 0},
    {SB32, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3 | HAS_AWE, "sb", N_("Sound Blaster AWE32/64"), 0, isa, 0},

// Added by bp from sndconfig 0.57
    {TRIDENT, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "trident", N_("Trident 4D-Wave NX/DX"), 0, pci, 0},
// Added by bp from sndconfig 0.57
	
	{MSND_CLASSIC, {5, 7, 9, 10, 11, 12, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, 0x290, 0x3e0, -1}, {-1},
	 {-1}, {-1}, {-1}, {0xb0000, 0xc8000, 0xd0000, 0xd8000, 0xe0000,
			    0xe8000, -1},
	 0, "msnd_classic",
	 N_("Turtle Beach MultiSound Classic/Monterey/Tahiti"), 1, isa, 0},
    {MSND_PINNACLE, {5, 7, 9, 10, 11, 12, -1},
     {0x210, 0x220, 0x230, 0x240, 0x290, 0x3e0, -1}, {-1},
     {0x250, 0x260, 0x270, -1}, MPU401_IO, STANDARD_IRQ, {0xb0000, 0xc8000,
							  0xd0000, 0xd8000,
							  0xe0000, 0xe8000,
							  -1},
     HAS_MPU401, "msnd_pinnacle",
     N_("Turtle Beach MultiSound Pinnacle/Fiji"), 2, isa, 0},
/* WaveFront doesn't work (in sndconfig) */
/*#if 0
    {WAVEFRONT, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, {0x200, 0x210, -1}, STANDARD_IRQ, {-1},
     LOAD_CS4232, "wavefront", N_("Turtle Beach Maui/Tropez/Tropez+"), 3,
     isa, 0},
#endif */

// Added by bp from sndconfig 0.57
    {VIA82CXXX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ|SB_AS_LIB, "via82cxxx", N_("VIA 82CXXX PCI Audio"), 0, pci, 0},

//    {VIA82CXXX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
//     NO_INPUT_REQ, "via82cxxx", N_("VIA 82CXXX PCI Audio"), 5, pci, 0},
// Added by bp from sndconfig 0.57

    {WSS, {7, 9, 10, 11, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3, "ad1848", N_("Windows Sound System (AD1848/CS4248/CS4231)"),
     0, isa, 0},

// Added by bp from sndconfig 0.57
    {YMFPCI, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ|SB_AS_LIB, "ymfsb", N_("Yamaha YMF PCI Audio (Legacy SB Support *only*)"), 0, pci, 0},
// Added by bp from sndconfig 0.57

    {AMD7930, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "amd7930", N_("AMD7930 (sun4c)"), 0, sbus, 0},
    {CS4231, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "cs4231", N_("CS4231 (some sun4m, sun4u)"), 0, sbus, 0},
    {DBRI, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "dbri", N_("DBRI (SPARCstation 10, 20, LX, Voyager)"),
     5, sbus, 0},
    {UNSPEC, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     0, "", "", 0, isa | sbus | pci, 0}
};
#else
struct SoundCardsTable cardsArray[] = {
    {ACERNB, {5, 7, 9, 10, -1}, SB16_IO, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3 | ACER, "sb", N_("ACER Notebook sound"), 0, isa, 0},
    {AD1816, {5, 7, 9, 10, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3, "ad1816", N_("AD1816 (Acer FX-3D, HP Kayak)"), 0, isa, 0},
    {ADLIB, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "adlib_card", N_("AdLib"), 0, isa, 0},
    {ALIM15XX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "ali_m1553",
     N_("ALI M15xx Chipset Audio"), 0, pci, 0},
    {ALS007, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "sb", N_("Advance Logic ALS-007"), 0, isa, 0},
    {ALS100, STANDARD_IRQ, SB16_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "sb", N_("Advance Logic ALS-100/110"), 0, isa, 0},
    {SGALAXY, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, SB16_IO,
     HAS_OPL3, "sgalaxy",
     N_("Aztech Sound Galaxy Waverider Pro 32-3D, Washington 16"), 0, isa,
     0},
    {CMI8330, {5, 7, 9, 10, 11, -1}, WSS_IO, LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_MPU401 | HAS_OPL3 | SOUNDPRO, "ad1848",
     N_("CMI8330 sound chip (many motherboards)"), 0, isa, 0},
    {CMI8X38, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ | NEEDS_SOX, "cmpci", N_("C-Media PCI Audio (8338, 8738)"), 0, pci,
     0},
    {DESKPROXL, {7, 9, 10, 11, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3 | COMPAQ, "ad1848", N_("Compaq Deskpro XL Sound"), 0, isa,
     0},
    {CS4232, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "cs4232", N_("Crystal CS423x sound chip"), 0, isa, 0},
    {CS46XX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "cs46xx",
     N_("Crystal SoundFusion CS46xx"), 0, pci, 0},
    {CS4281, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "cs4281",
     N_("Crystal CS4281"), 0, pci, 0},
// for IBM laptop added by AA
    {CS461X, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "cs46xx",
     N_("Crystal 4280/461x + AC97 Audio"), 0, pci, 0},
//
    {ES1370, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "es1370",
     N_("Ensoniq AudioPCI 1370 (SoundBlaster 64/128 PCI)"), 0, pci, 0},
    {ES1371, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "es1371",
     N_("Creative/Ensoniq AudioPCI 1371"), 0, pci, 0},
    {SOUNDSCAPE, {5, 7, 9, 10, 15, -1}, WSS_IO, LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     0, "sscape", N_("Ensoniq SoundScape"), 0, isa, 0},
    {SSVIVO, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_MPU401, "ad1848", N_("Ensoniq SoundScape VIVO"), 0, isa, 0},
    {ESS688, {5, 7, 9, 10, -1}, SB16_IO, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS688 AudioDrive"), 0, isa, 0},
    {ESS1688, {5, 7, 9, 10, -1}, {0x220, 0x230, 0x240, 0x250, -1}, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS1688 AudioDrive"), 0, isa, 0},
    {ESS1868, {5, 7, 9, 10, -1}, {0x220, 0x230, 0x240, 0x250, -1}, LOW_DMA,
     {-1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("ESS1868 AudioDrive"), 0, isa, 0},
    {ESSMAESTRO, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "maestro", N_("ESS Maestro 1/2/2E"), 0, pci, 0},
    {ESSMAESTRO3, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "maestro3", N_("ESS Maestro 3/Allegro"), 0, pci, 0},
    {ESSSOLO1, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "esssolo1", N_("ESS Solo 1"), 0, pci, 0},
    {GUS, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
     {-1}, {-1}, {-1}, {-1},
     0, "gus", N_("Gravis UltraSound"), 0, isa, 0},
    {GUSMAX, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
     {1, 3, 5, 6, 7, -1}, {-1}, {-1}, {-1},
     0, "gus", N_("Gravis UltraSound MAX"), 0, isa, 0},
    {GUSPNP, {3, 5, 7, 9, 11, 12, 15, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1}, {1, 3, 5, 6, 7, -1},
      {1, 3, 5, 6, 7, -1}, {-1}, {-1}, {-1},
      0, "gus", N_("Gravis UltraSound PnP"), 0, isa, 0},
    {I810, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "i810_audio", N_("Intel i810 AC97 Audio"), 0, pci, 0},
    {SMGAMES, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3 | SM_GAMES, "sb",
     N_("Logitech SoundMan Games (not SM16 or SM Wave!)"), 0, isa, 0},
    {AUDIOTRIX, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, {5, 7, 9, 10, 11, 12, 15, -1}, {-1},
     HAS_OPL3 | SB_AS_LIB, "trix", N_("MediaTrix AudioTrix Pro"), 0, isa,
     0},
    {JAZZ16, STANDARD_IRQ, {0x220, 0x240, 0x260, -1}, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, {0x310, 0x320, 0x330, -1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("MediaVision Jazz16 (ProSonic, SoundMan Wave)"), 0,
     isa, 0},
    {MOZART, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3 | SB_AS_LIB | JOYSTICK, "mad16",
     N_("Mozart/MAD16 (OPTi 82C928)"), 0, isa, 0},
    {MAD16PRO, STANDARD_IRQ, WSS_IO, LOW_DMA,
     {0, 1, 3, 5, 6}, {0x300, 0x310, 0x320, 0x330, -1}, STANDARD_IRQ, {-1},
     HAS_OPL3 | SB_AS_LIB | JOYSTICK, "mad16",
     N_("MAD16 Pro (OPTi 82C929/82C930)"), 0, isa, 0},
    {MIROSOUND, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     ACI, "mad16", N_("miroSOUND PCM12"), 0, isa, 0},
    {NEOMAGIC, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ | NEEDS_SOX, "nm256_audio", N_("NeoMagic MagicMedia 256AV"), 0, pci, 0},
    {OPL3_SA, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, STANDARD_IRQ, {-1},
     HAS_OPL3, "opl3sa", N_("OPL3-SA1 sound chip"), 0, isa, 0},
    {OPL3_SAX, {5, 7, 9, 10, 11, 12, 15, -1}, WSS_IO, LOW_DMA,
     LOW_DMA, MPU401_IO, {-1}, {0x370, 0x380, -1},
     HAS_OPL3, "opl3sa2", N_("OPL3-SA2/3/x sound chip"), 0, isa, 0},
    {PAS16, {3, 5, 7, 9, 10, 11, 12, -1}, {0x220, 0x230, 0x240, 0x388, -1},
     {0, 1, 2, 3, 5, 6, 7, -1},
	 {-1}, {-1}, {-1}, {-1},
	 SB_AS_LIB | JOYSTICK, "pas2",
	 N_("Pro Audio Spectrum/Studio 16, Logitech SoundMan 16"), 0, isa,
	 0},
    {PSS, {3, 5, 7, 9, 10, 11, 12}, {0x220, 0x230, 0x240, 0x250, -1},
     LOW_DMA,
     {-1}, MPU401_IO, STANDARD_IRQ, {-1},
     0, "pss", N_("PSS (Orchid SW32, Cardinal DSP16)"), 0, isa, 0},
    {SONICVIBES, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | JOYSTICK | NO_INPUT_REQ, "sonicvibes",
     N_("S3 SonicVibes"), 0, pci, 0},
    {SBORIG, {3, 5, 7, 9, -1}, {0x220, 0x240, -1}, {1, -1},
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster"), 0, isa, 0},
    {SBPRO, STANDARD_IRQ, {0x220, 0x240, -1}, LOW_DMA,
     {-1}, {-1}, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster Pro"), 0, isa, 0},
    {SB16, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3, "sb", N_("Sound Blaster 16"), 0, isa, 0},
    {SB32, STANDARD_IRQ, SB16_IO, LOW_DMA,
     {0, 1, 3, 5, 6, 7, -1}, MPU401_IO, {-1}, {-1},
     HAS_OPL3 | HAS_AWE, "sb", N_("Sound Blaster AWE32/64"), 0, isa, 0},
    {SBLIVE, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NEEDS_SOX | NO_INPUT_REQ, "emu10k1", N_("Sound Blaster Live!"), 0, pci, 0},
    {TRIDENT, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ | NEEDS_SOX, "trident", N_("Trident 4D-Wave NX/DX"), 0, pci, 0},
     {MSND_CLASSIC, {5, 7, 9, 10, 11, 12, -1},
     {0x210, 0x220, 0x230, 0x240, 0x250, 0x260, 0x290, 0x3e0, -1}, {-1},
	 {-1}, {-1}, {-1}, {0xb0000, 0xc8000, 0xd0000, 0xd8000, 0xe0000,
			    0xe8000, -1},
	 0, "msnd_classic",
	 N_("Turtle Beach MultiSound Classic/Monterey/Tahiti"), 1, isa, 0},
    {MSND_PINNACLE, {5, 7, 9, 10, 11, 12, -1},
     {0x210, 0x220, 0x230, 0x240, 0x290, 0x3e0, -1}, {-1},
     {0x250, 0x260, 0x270, -1}, MPU401_IO, STANDARD_IRQ, {0xb0000, 0xc8000,
							  0xd0000, 0xd8000,
							  0xe0000, 0xe8000,
							  -1},
     HAS_MPU401, "msnd_pinnacle",
     N_("Turtle Beach MultiSound Pinnacle/Fiji"), 2, isa, 0},
/* WaveFront doesn't work (in sndconfig) */
    {WAVEFRONT, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, {0x200, 0x210, -1}, STANDARD_IRQ, {-1},
     LOAD_CS4232, "wavefront", N_("Turtle Beach Maui/Tropez/Tropez+"), 3,
     isa, 0},
    {VIA82CXXX, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ | NEEDS_SOX, "via82cxxx_audio", N_("VIA 82CXXX PCI Audio"), 0, pci, 0},
    {WSS, STANDARD_IRQ, WSS_IO, LOW_DMA,
     LOW_DMA, {-1}, {-1}, {-1},
     HAS_OPL3, "ad1848", N_("Windows Sound System (AD1848/CS4248/CS4231)"),
     0, isa, 0},
    {YMFPCI, {-1}, {-1}, {-1}, {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ | NEEDS_SOX|SB_AS_LIB, "ymfpci", N_("Yamaha YMF PCI Audio"),
     0, pci, 0},
    {AMD7930, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "amd7930", N_("AMD7930 (sun4c)"), 0, sbus, 0},
    {CS4231, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "cs4231", N_("CS4231 (some sun4m, sun4u)"), 0, sbus, 0},
    {DBRI, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     NO_INPUT_REQ, "dbri", N_("DBRI (SPARCstation 10, 20, LX, Voyager)"),
     0, sbus, 0},
    {UNSPEC, {-1}, {-1}, {-1},
     {-1}, {-1}, {-1}, {-1},
     0, "", "", 0, isa | sbus | pci, 0}
};
#endif

QArray < struct SoundCardsTable > cardsArrayShow ( sizeof( cardsArray )/sizeof( struct SoundCardsTable ) ) ;
int cardsArrayShow_size;


/********************************************************************************************/

bool LSound::sbAvailable = false;

/********************************************************************************************/

// constructor of "Advanced dialog"
AdvSoundDialog::AdvSoundDialog(QWidget* parent, LSound *Lsnd, const char *name )
   : QDialog( parent, name, TRUE ){

	this->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

	bFirstShow = true;
	sound = Lsnd;

  QHBoxLayout *h1 = new QHBoxLayout();
  QHBoxLayout *hbutton = new QHBoxLayout();
	QHBoxLayout *hmodule_name = new QHBoxLayout();

  QVBoxLayout *vMain = new QVBoxLayout( this, 20  );
  QVBoxLayout *vleft = new QVBoxLayout( 3 );
  QVBoxLayout *vright = new QVBoxLayout( 3 );


  QLabel *lblType = new QLabel( tr("Card type:"), this );
  lblType->setAutoMask( true );
  listType = new QListBox ( this );

  lblIO = new QLabel ( tr("I/O Port:"), this );
  lblIO->setAutoMask( true );
  comboIO = new QComboBox( this );
	comboIO->setMinimumWidth( comboIO->sizeHint().width() + 10 );

  lblIRQ = new QLabel ( tr("IRQ:"), this );
  lblIRQ->setAutoMask( true );
  comboIRQ = new QComboBox ( this );
	comboIRQ->setMinimumWidth( comboIRQ->sizeHint().width() + 10 );

  lblDMA = new QLabel ( tr("DMA:"), this );
  lblDMA->setAutoMask( true );
  comboDMA = new QComboBox ( this );
	comboDMA->setMinimumWidth( comboDMA->sizeHint().width() + 10 );

  lblDMA2 = new QLabel( tr("DMA 2:"), this );
  lblDMA2->setAutoMask( true );
  comboDMA2 = new QComboBox ( this );
	comboDMA2->setMinimumWidth( comboDMA2->sizeHint().width() + 10 );

 	lblMPU_IO = new QLabel( tr("MPU I/O:"), this );
  lblMPU_IO->setAutoMask( true ); 	
  comboMPU_IO = new QComboBox ( this );
	comboMPU_IO->setMinimumWidth( comboMPU_IO->sizeHint().width() + 10 );

	lblMPU_IRQ = new QLabel( tr("MPU IRQ:"), this );
  lblMPU_IRQ->setAutoMask( true );	
  comboMPU_IRQ = new QComboBox ( this );
	comboMPU_IRQ->setMinimumWidth( comboMPU_IRQ->sizeHint().width() + 10 );

  lblMEM = new QLabel ( tr("MEM:"), this );
  lblMEM->setAutoMask( true );
  comboMEM = new QComboBox( this );
	comboMEM->setMinimumWidth( comboMEM->sizeHint().width() + 10 );

	QLabel *lblHint = new QLabel ( tr("Module name:")+" ", this );
  lblHint->setAutoMask( true );	
	lblHint_text = new QLabel ( "", this );
  lblHint_text->setAutoMask( true );

  QPushButton *okbutton = new QPushButton ( tr("OK"), this );
  QObject::connect( okbutton, SIGNAL( clicked() ), this, SLOT( accept() ) );

  QPushButton *cancelbutton = new QPushButton ( tr("Cancel"), this );
  QObject::connect( cancelbutton, SIGNAL(clicked()), this, SLOT( reject() ) );

	hmodule_name->addWidget( lblHint );
	hmodule_name->addWidget( lblHint_text );	
	hmodule_name->addStretch();

  vleft->addWidget( lblType );
  vleft->addWidget( listType, 1 );

  vright->addWidget( lblIO );
  vright->addWidget( comboIO );
  vright->addWidget( lblIRQ );
  vright->addWidget( comboIRQ );
  vright->addWidget( lblDMA );
  vright->addWidget( comboDMA );
  vright->addWidget( lblDMA2 );
	vright->addWidget( comboDMA2 );
  vright->addWidget( lblMPU_IO );
  vright->addWidget( comboMPU_IO);
  vright->addWidget( lblMPU_IRQ );
  vright->addWidget( comboMPU_IRQ );
	vright->addWidget( lblMEM );
	vright->addWidget( comboMEM );
/*	vright->addWidget( lblHint );
	vright->addWidget( lblHint_text );	*/
  vright->addStretch();	

  h1->addLayout( vleft, 5 );
  h1->addLayout( vright, 1 );

	hbutton->addLayout( hmodule_name );
  hbutton->addStretch();
  hbutton->addWidget( okbutton );
  hbutton->addWidget( cancelbutton );

  vMain->addLayout( h1 );
  vMain->addLayout( hbutton );

  setCaption( tr("Manual setup of sound") );

  init();

  connect( listType, SIGNAL( highlighted ( int )), this, SLOT( ShowIO( int )) );
	connect( okbutton, SIGNAL( clicked () ), this, SLOT( NewCardSet() ) );
  connect( cancelbutton, SIGNAL( clicked() ), this, SLOT( CloseAdv() ) );

}
/********************************************************************************************/

void LSound :: Sort ( QArray <struct SoundCardsTable> sound_array, int size ){
  int a, b;
	QArray < struct SoundCardsTable > cardsArrayNew (1);
	
	for ( a=1; a < size; ++a ){
 		cardsArrayNew[0] = sound_array[a];
		for ( b=a-1; b>=0 && strcasecmp( cardsArrayNew[0].descr, sound_array[b].descr )<0 ; b-- ){
		 sound_array[b+1] = sound_array[b];
    }
  sound_array[b+1] = cardsArrayNew[0];
  }
}

/********************************************************************************************/
void AdvSoundDialog :: UnloadModules ( QString mod_name, int card_number ){

	//Temporarily
/*			printf("\n");
			system("lsmod");	
			printf("\n");*/
  //Temporarily

QString rmmod_line = "rmmod ";
QString rmmod_line_set = "";

#ifdef GBLI_DEVEL
	return;
#endif

if ( sound->GetMixerValue() >= 0 ){
    ::close( sound->GetMixerValue() );
		sound->InitMixer();
}

int result;

	if ( cardsArray[ card_number ].flags & HAS_OPL3 ){		
		rmmod_line_set = rmmod_line + "opl3";
		//printf( "RMMOD: %s \n", rmmod_line_set.ascii()   );
  	result = system( rmmod_line_set );
		//printf ( "RMMOD_opl3 result: %d \n", result );
  }

//	if ( cardsArray[ card_number ].flags & HAS_MPU401 ){
		rmmod_line_set = rmmod_line + "mpu401";
		//printf( "RMMOD: %s \n", rmmod_line_set.ascii()   );
  	result = system( rmmod_line_set );
		//printf ( "RMMOD_mpu401 result: %d \n", result );
//	}

  if ( cardsArray[ card_number ].flags & HAS_AWE ){
		rmmod_line_set = rmmod_line + "awe_wave";	
		result = system( rmmod_line_set );
		//printf( "RMMOD_awe: %d \n", result );
	}

	if ( cardsArray[ card_number ].flags & ACI ){
		rmmod_line_set = rmmod_line + "aci";	
		result = system( rmmod_line_set );		
		//printf("RMMOD_aci: %d \n", result);
	}

	rmmod_line_set = rmmod_line + mod_name;
	result = system ( rmmod_line_set );
//	printf( "Rmmod modulename: %s \n", rmmod_line_set.ascii() );
//  printf( "Result modulename: %d \n", result );


	if ( (QString ::fromLatin1( cardsArray[ card_number ].modulename ) == "cs4232") ||
     ( QString ::fromLatin1(cardsArray[ card_number ].modulename) == "gus" ) ){
		rmmod_line_set = rmmod_line + "ad1848";
		result = system( rmmod_line_set );
		//printf("RMMOD_ad1848: %d \n", result );
	}

	if ( cardsArray[ card_number ].flags & SB_AS_LIB ){
		rmmod_line_set = rmmod_line + "sb";
		result = system ( rmmod_line_set );
	  //printf("RMMOD_module: %d \n", result );	
	}
  result = system ( "rmmod uart401" );
	result = system ( "rmmod sound" );
	result = system ( "rmmod soundcore" );
	result = system ( "rmmod soundlow" );	

	//Temporarily
/*			printf("\n");
			system("lsmod");	
			printf("\n");*/
  //Temporarily


			
}

/****************************************************************************/
bool LSound :: LoadModule ( QString str_module, int itype,
                            QString str_mpu_io,
                            QString str_mpu_irq,
												    QString str_io,
														QString str_irq,
														QString str_dma,
														QString str_dma2,
														QString str_mem  ){

	
	int result;
	bool b_modprobe = true;

//	spath_tmp = NEWROOT + "/tmp/snd_err";
/*#ifdef GBLI_DEVEL
	spath_tmp = "/home/nd/temp/snd_err"; 	
#endif
	FILE *f_temp = fopen ( spath_tmp, "w" ) ;
	if ( f_temp )	fclose( f_temp );		*/

  if ( cardsArrayShow[itype].flags & HAS_MPU401 ){
    QString str_mpu_modprobe = "modprobe -qs mpu401 io=" + str_mpu_io + " irq=" + str_mpu_irq;// + " 1>>" + spath_tmp + " 2>&1";
#ifndef GBLI_DEVEL
  	result = system( str_mpu_modprobe );
#else
		result = 1;
#endif
    if ( result != 0 ) b_modprobe = false;

		//printf("Modprobe mpu401: %d \n", result );
		if ( cardsArrayShow[itype].type != MSND_PINNACLE ){
				str_mpu_io = "";
				str_mpu_irq = "";
		}                    			
  }

  isfx_result = 0;
	if ( cardsArrayShow[itype].flags & HAS_AWE ){
			QString sfxload = "sfxload";		
#ifdef GBLI_DEVEL
			system("modprobe -qs awe_wave");
		  sfxload = sfxload + " /etc/midi/GU11-ROM.SF2";
	//		printf( "AWE: %s\n", sfxload.ascii() );	
			isfx_result = system( sfxload );
		/*	printf( "sfxload running result: %d\n", isfx_result );
			if ( isfx_result != 0 ) {
					lblMidiSnd->hide();
					midi->hide();
			}*/
#endif
	}	

	QString str_modprobe = "modprobe -qs " + str_module;

	if ( !str_io.isEmpty() ) {
		if ( cardsArrayShow[itype].type == OPL3_SAX )
    		str_modprobe = str_modprobe + " mss_io=" + str_io;
		else {
				if ( str_module == "mad16" ){
           QString str_io_convert = str_io.right( str_io.length() - ( str_io.find("x",0) + 1 ) );
           bool ok;
           int i_num = str_io_convert.toInt( &ok, 16 );
    			 if ( i_num % 16 ) i_num -= 4;
           str_io = "";
           str_io = "0x" + str_io.setNum ( i_num, 16 );
           str_modprobe = str_modprobe + " io=" + str_io;						
				}
        else
						str_modprobe = str_modprobe + " io=" + str_io;
		}                                                      					
	}

	if ( !str_irq.isEmpty() )
		str_modprobe = str_modprobe + " irq=" + str_irq;

	if ( !str_dma.isEmpty() )
		str_modprobe = str_modprobe + " dma=" + str_dma;

	if ( !str_dma2.isEmpty() ){
		if ( ( str_module == "sb" ) ||
				( str_module == "pas2" ) ||
        ( str_module == "gus" ) ||
				( str_module == "mad16") ) {
				if  ( ( cardsArrayShow[ itype ].type != SB16 ) || ( str_dma2 >=5 ) )
						str_modprobe = str_modprobe + " dma16=" + str_dma2;        	
		}	
		else  if ( cardsArrayShow[itype].type == MSND_PINNACLE )
    			str_modprobe = str_modprobe + " cfg=" +  str_dma2;
			
		else str_modprobe = str_modprobe + " dma2=" + str_dma2;
	}

	if ( !str_mpu_io.isEmpty() && cardsArrayShow[itype].type != SGALAXY ){
			if ( cardsArrayShow[itype].flags & LOAD_CS4232 ){
					str_modprobe = str_modprobe + " synthio=" + str_mpu_io;
			}
			else{
					if ( str_module == "cs4232" )
							str_modprobe = str_modprobe + " mpuio=" + str_mpu_io;			
					else str_modprobe = str_modprobe + " mpu_io=" + str_mpu_io;			
			}
	}

	if ( !str_mpu_irq.isEmpty() && cardsArrayShow[itype].type != SGALAXY ){
			if ( cardsArrayShow[itype].flags & LOAD_CS4232 ){
					str_modprobe = str_modprobe + " synthirq=" + str_mpu_irq;
			}
  		else {
					if ( str_module == "cs4232" )
							str_modprobe = str_modprobe + " mpuirq=" + str_mpu_irq;
					else str_modprobe = str_modprobe + " mpu_irq=" + str_mpu_irq;
			}
	}

	if ( !str_mem.isEmpty() ){
		if ( cardsArrayShow[itype].type == OPL3_SAX )
				str_modprobe = str_modprobe + " io=" + str_mem;
		else if ( cardsArrayShow[itype].type == SGALAXY )
				str_modprobe = str_modprobe + " sgbase=" + str_mem;
		else str_modprobe = str_modprobe + " mem=" + str_mem;	
	}	

	if ( cardsArrayShow[itype].flags & JOYSTICK )
			str_modprobe = str_modprobe + " joystick=1";
	if ( cardsArrayShow[itype].flags & SM_GAMES )
			str_modprobe = str_modprobe + " sm_games=1";
	if ( cardsArrayShow[itype].flags & COMPAQ )
			str_modprobe = str_modprobe + " deskpro_xl=1";
	if ( cardsArrayShow[itype].flags & ACER )
			str_modprobe = str_modprobe + " acer=1";

	if ( cardsArrayShow[itype].flags & SOUNDPRO )
			str_modprobe = str_modprobe + " soundpro=1";


//printf ("MODPROBE_string: %s\n", str_modprobe.ascii() );

//	str_modprobe = str_modprobe + " 1>" + spath_tmp + " 2>&1";
#ifndef GBLI_DEVEL
	result = system ( str_modprobe );
#else
	result = 1;
#endif
	//printf( "Modprobe main module: %d \n", result  );
	if ( result!=0 ) b_modprobe = false;

	if ( cardsArrayShow[itype].flags & HAS_OPL3  ){
			QString str_modprobe_opl = "modprobe -qs opl3 io=0x388";// + " 1>>" + spath_tmp + " 2>&1";
#ifndef GBLI_DEVEL
			result = system ( str_modprobe_opl );
#else
			result = 1;
#endif
			if ( result!=0 ) b_modprobe = false ;					 	
	//printf( "Modprobe opl3: %d \n", result );
  }

//Is it reasonable??????????
/*  if ( cardsArrayShow[itype].flags & HAS_AWE ){
			QString str_modprobe_awe = "insmod -qs  awe_wave"; //+ " 1>>" + spath_tmp + " 2>&1";
  	  result = system ( str_modprobe_awe );
      if ( result!=0 ) b_modprobe = false;
  //printf( "Insmod awe_wave: %d \n", result );
	}*/

//Something strange
  if ( cardsArrayShow[itype].flags & SB_AS_LIB ){
		QString str_sb_as_lib = "insmod -qs  sb" + str_module + "=1"; //+" 1>>" + spath_tmp + " 2>&1";
#ifndef GBLI_DEVEL
  	result = system ( str_sb_as_lib );
#else
		result = 1;
#endif
		if (result!=0) b_modprobe = false;
		// printf( "Insmod sb: %d \n", result );
  }
/////////////

  if ( cardsArrayShow[itype].flags & ACI ){
		QString str_insmod_aci = "insmod -qs aci"; //+" 1>>" + spath_tmp + " 2>&1";
#ifndef GBLI_DEVEL
		result = system ( "insmod -qs aci" );
#else
		result = 0;
#endif
		if ( result!=0 ) b_modprobe = false;
	 	//printf("Insmod: %d \n", result );
  }
 	return b_modprobe;
}


/****************************************************************************/
void AdvSoundDialog :: CloseAdv(){
		bFirstShow = true;
}
/****************************************************************************/

void AdvSoundDialog :: NewCardSet(){	
				
		bFirstShow = true;
		
    if ( *( sound->ReturnCount() ) != -1  )		
			UnloadModules ( *( sound->ReturnStr(3) ), *( sound->ReturnType() )  );				

    sound->VarsPubNull(); // set all config. variables to 0

    int index = listType->currentItem();

		*( sound->ReturnStr(4) ) = cardsArrayShow[ index ].descr;
    *( sound->ReturnStr(5) ) = sound->CardTypeToString( cardsArrayShow [ index ].type );
    *( sound->ReturnType() ) = cardsArrayShow [ index ].type;
    *( sound->ReturnCount() ) = index;
    *( sound->ReturnStr(3) ) = cardsArrayShow[ index ].modulename;		
/*
0: sPubIRQ;
1: sPubIO;	
2: sPubDMA;
3: sPubModulename; 			
4: sPubDescr;
5: sPubType;
6: sPubDMA2;
7: sPubMPU_IO;
8: sPubMPU_IRQ;
9: sPubMEM;
*/
		*( sound->ReturnStr(0) ) = comboIRQ->text( comboIRQ->currentItem() );
		*( sound->ReturnStr(1) ) = comboIO->text( comboIO->currentItem() );
    if ( cardsArrayShow[index].dma[0] != -1 )
    	*( sound->ReturnStr(2) ) = comboDMA->text( comboDMA->currentItem() );
    if ( cardsArrayShow[index].dma2[0] != -1 )
    	*( sound->ReturnStr(6) ) = comboDMA2->text( comboDMA2->currentItem() ) ;
		if ( cardsArrayShow[index].mpu_io[0] != -1 )
    	*( sound->ReturnStr(7) ) = comboMPU_IO->text( comboMPU_IO->currentItem() );
    if ( cardsArrayShow[index].mpu_irq[0] !=-1 )
			*( sound->ReturnStr(8) ) = comboMPU_IRQ->text( comboMPU_IRQ->currentItem() );
    if ( cardsArrayShow[index].mem[0] !=-1 )
    	*( sound->ReturnStr(9) ) = comboMEM->text( comboMEM->currentItem() );

   //Temporarily
		QString tmpType = *( sound ->ReturnStr(5) );
    QString tmpIRQ = *( sound->ReturnStr(0) );
		QString tmpIO = *( sound->ReturnStr(1) );
		QString tmpDMA = *( sound->ReturnStr(2) );
		QString tmpModulename = *( sound->ReturnStr(3) );
		QString tmpDescr = *( sound->ReturnStr(4) );
 /*   printf( "Type: %s \n", tmpType.ascii() );
		printf( "Descr: %s \n", tmpDescr.ascii() );
		printf( "Module: %s \n", tmpModulename.ascii()  );	
    printf( "IO: %s \n", tmpIO.ascii() );	
    printf( "IRQ: %s \n", tmpIRQ.ascii() );	
    printf( "DMA: %s \n", tmpDMA.ascii() );*/

//		int i_type = *( sound->ReturnType() );
		int i_index = *( sound->ReturnCount() );
		QString tmp_mpu_io = *( sound->ReturnStr(7) );
		QString tmp_mpu_irq = *( sound->ReturnStr(8) );
		QString tmp_dma2 = *( sound->ReturnStr(6) );
		QString tmp_mem = *( sound->ReturnStr(9) );
		
		if ( sound -> LoadModule ( tmpModulename, i_index,
                            tmp_mpu_io,
                            tmp_mpu_irq,
												    tmpIO,
														tmpIRQ,
														tmpDMA,
														tmp_dma2,
														tmp_mem ) ){
  //  printf("LOAD MODULES IS TRUE\n");

/*if ( *( sound->ReturnType() ) != -1 ){
    sound->WriteConfModules(); // move to finish();
		sound->WriteCardSettings( tmpType );
}	*/

/*0: label;
  1: lblAudio;
	2: lblSynth;
  3: lblMidi;
  4: lblTimers;
  5: lblMixers;
	6: lblAudio_text;
	7: lblSynth_text;
	8: lblMidi_text;
	9: lblTimers_text;
	10: lblMixers_text;
  11: lblDigSnd;
	12: lblMidiSnd;
	13: lbl_volume;
	14: lbl_balance;
*/
    if (LSound::hasSound()){
				sound->GetLabel( 11 )->setEnabled( TRUE );
				sound->GetButton( 0 )->setEnabled( TRUE );
    }else{
				sound->GetLabel( 11 )->setEnabled( FALSE );
				sound->GetButton( 0 )->setEnabled( FALSE );
		}

		if (LSound::hasMidi() && !( *(sound->ReturnIsfxLoad()) ) ){
				if ( QString ::fromLatin1 ( cardsArrayShow[ i_index ].modulename ) != "gus"  ){
						sound->GetButton( 1 )->show();
						sound->GetLabel( 12 )->show();
						sound->GetButton( 1 )->setEnabled( TRUE );
						sound->GetLabel( 12 )->setEnabled( TRUE );
				}
				else{
						sound->GetButton( 1 )->hide();
						sound->GetLabel( 12 )->hide();
				}	
    }else{
				sound->GetButton( 1 )->hide();
				sound->GetLabel( 12 )->hide();
		}

	 	sound->GetLabel( 13 )->setEnabled( TRUE );
		sound->GetSlider( 0 )->setEnabled( TRUE );
    sound->GetLabel( 14 )->setEnabled( TRUE );
		sound->GetSlider( 1 )->setEnabled( TRUE );                         		
		
		sound->LoadMidi( *( sound->ReturnType() ) );

		bool b_proc = sound->PubReadInfo();				
				sound->GetLabel( 0 )->setText( tr("A sound card was detected:") );
				sound->GetLabel( 1 )->show();		
    if ( !b_proc ) {	 						 		
       	sound->GetLabel( 6 )->setText(  *( sound->ReturnStr(4) ) );
				sound->GetLabel( 2 )->hide();
				sound->GetLabel( 3 )->hide();
				sound->GetLabel( 4 )->hide();
				sound->GetLabel( 5 )->hide();
	 }else{
	 			sound->GetLabel( 2 )->setEnabled( TRUE );
				sound->GetLabel( 3 )->setEnabled( TRUE );
				sound->GetLabel( 4 )->setEnabled( TRUE );
				sound->GetLabel( 5 )->setEnabled( TRUE );	
	 }		                          	 						

	 } //if LoadModule()	
	 else {		

			if ( *( sound->ReturnCount() ) != -1  )	
				UnloadModules ( *( sound->ReturnStr(3) ), *( sound->ReturnType() )  );				

      sound->GetLabel( 0 )->setText( tr("Sound card is not detected") );
			sound->GetLabel( 1 )->hide();
			sound->GetLabel( 6 )->setText( "" );
			sound->GetLabel( 2 )->hide();
			sound->GetLabel( 7 )->setText( "" );
			sound->GetLabel( 3 )->hide();
			sound->GetLabel( 8 )->setText( "" );
			sound->GetLabel( 4 )->hide();
			sound->GetLabel( 9 )->setText( "" );
			sound->GetLabel( 5 )->hide();
			sound->GetLabel( 10 )->setText( "" );
			sound->GetLabel( 11 )->setEnabled( FALSE );			
			sound->GetLabel( 12 )->setEnabled( FALSE );			
			sound->GetLabel( 13 )->setEnabled( FALSE );
			sound->GetLabel( 14 )->setEnabled( FALSE );
			sound->GetButton( 0 )->setEnabled( FALSE );
			sound->GetButton( 1 )->setEnabled( FALSE );						
			sound->GetSlider( 0 )->setEnabled( FALSE );
			sound->GetSlider( 1 )->setEnabled( FALSE );
//disable data to the main page......

//printf("LOAD MODULES IS FALSE\n");
	
			QString sLineData = "";
/*			QFile file_err( sound->GetPath() );
		  if ( file_err.open(IO_ReadOnly) ){
			     QTextStream stream( &file_err );     								 	
			     while ( !stream.eof() ){
				       sLineData = sLineData + stream.readLiner();
				   }       			
  				 file_err.close();
					 file_err.remove();	
			}*/
//
					sLineData = tr("Module loading failed\n") + sLineData;
          switch (
							QMessageBox::critical (
          		this, tr("Warning"),
  	     			sLineData,
  	      		tr("OK") , 0, "", 1  )  ) {
					case 0: sound->showDialog(); break;
					}
      sound->VarsPubNull();
	 }		   	  	
}


/********************************************************************************************/
void AdvSoundDialog::init(){

  QStringList list;

  for( int i = 0; i < cardsArrayShow_size; i++ ){
    if (cardsArrayShow[i].type != UNSPEC) {
    	list.append( QString( cardsArrayShow[i].descr ) );
    }
  }

	for ( QStringList::Iterator it = list.begin(); it != list.end(); it++ )						
  listType->insertItem( *it  );
}

/********************************************************************************************/
int AdvSoundDialog::FindItem( QString str_var, QComboBox *combo ){

  int item_index = 0;
  QString str_combo;
	if ( !str_var.isEmpty() ){
		int items_num = combo->count();
		for ( int i=0; i<items_num; i++){
						str_combo = combo->text( i );							
						if ( str_combo == str_var ){								
					      item_index = i;
						}		
		}
	}
	return item_index;

}

/********************************************************************************************/
void AdvSoundDialog::ShowIO( int index ){

   index = listType->currentItem();

   QString str;
   comboIO->clear();
   comboIRQ->clear();
   comboDMA->clear();
   comboDMA2->clear();
   comboMPU_IO->clear();
	 comboMPU_IRQ->clear();
   comboMEM->clear();

	if ( cardsArrayShow[index].io[0] == -1 ){
		lblIO->hide();
		comboIO->hide();
	}else{
			lblIO->show();
			comboIO->show();
	}	 	

	if ( cardsArrayShow[index].irq[0] == -1 ){
			lblIRQ->hide();
			comboIRQ->hide();	
	}else{
			lblIRQ->show();
			comboIRQ->show();
	}                    	

	if ( cardsArrayShow[index].dma[0] == -1 ){
		lblDMA->hide();
		comboDMA->hide();
	}	 	
	else {
    if ( (cardsArrayShow[index].dma2[0] == 5) && (cardsArrayShow[index].dma[0] == 0) )
				lblDMA->setText( tr("8-bit DMA:") );
		else if ( cardsArrayShow[index].dma2[0] != -1 )
				lblDMA->setText( tr("DMA 1:") );
    else lblDMA->setText( tr("DMA:") );	
		lblDMA->show();
		comboDMA->show();

	}//else		


  if ( cardsArrayShow[index].dma2[0] == -1 ){
		lblDMA2->hide();
		comboDMA2->hide();
	}
  else {
		if ( cardsArrayShow[index].type == MSND_PINNACLE )
				lblDMA2->setText( tr("CONFIG I/O:") );
		if ( cardsArrayShow[index].dma2[0] == 5 )
				lblDMA2->setText( tr("16-bit DMA:") );
    else lblDMA2->setText( tr("DMA 2:") ) ;
		lblDMA2->show();
    comboDMA2->show();	
  }

  if ( cardsArrayShow[index].mpu_io[0] == -1 ){
		lblMPU_IO->hide();
		comboMPU_IO->hide();
  }
  else {
		lblMPU_IO->show();
    comboMPU_IO->show();
  }

  if ( cardsArrayShow[index].mpu_irq[0] == -1 ){
		lblMPU_IRQ->hide();
    comboMPU_IRQ->hide();
	}
  else {
		lblMPU_IRQ->show();
    comboMPU_IRQ->show(); 	
  }

  if ( cardsArrayShow[index].mem[0] == -1 ){
			lblMEM->hide();
			comboMEM->hide();		
	}
	else {
		if ( cardsArrayShow[index].type == OPL3_SAX ) lblMEM->setText( tr("CONTROL I/O:") );
    else
	    if ( cardsArrayShow[index].type == SGALAXY ) lblMEM->setText( tr("SB I/O:") );
    else lblMEM->setText( tr("BASE MEMORY:") );
		lblMEM->show();
		comboMEM->show();
	}

   int i;
	 QString str_temp;	

   for ( i=0; cardsArrayShow[index].io[i] != -1; i++ ){
     str = "";
     str = "0x" + str.setNum ( cardsArrayShow[index].io[i], 16 );
     comboIO->insertItem( str );
   }

   for ( i=0; cardsArrayShow[index].irq[i] != -1; i++ ){
     str = "";
     str = str.setNum ( cardsArrayShow[index].irq[i], 10 );
     comboIRQ->insertItem( str );
   }

   for ( i=0; cardsArrayShow[index].dma[i] != -1; i++  ){
     str = "";
     str = str.setNum ( cardsArrayShow[index].dma[i], 10 );
     comboDMA->insertItem( str );
   }

   for ( i=0; cardsArrayShow[index].dma2[i] != -1; i++  ){
		 if ( cardsArrayShow[index].type == MSND_PINNACLE ){
				str = "";
        str = "0x" + str.setNum( cardsArrayShow[index].io[i], 16 );
				comboDMA2->insertItem( str );
     }		
	   else {
     		str = "";
     		str = str.setNum ( cardsArrayShow[index].dma2[i], 10 );
     		comboDMA2->insertItem( str );
     }
   }


   for ( i=0; cardsArrayShow[index].mpu_io[i] != -1; i++ ){
 		 str = "";
     str = "0x" + str.setNum ( cardsArrayShow[index].mpu_io[i], 16 );
     comboMPU_IO->insertItem( str );
	 }	

   for ( i=0; cardsArrayShow[index].mpu_irq[i] != -1; i++ ){
  	 str = "";
     str = str.setNum ( cardsArrayShow[index].mpu_irq[i], 10 );
     comboMPU_IRQ->insertItem( str );			
   }

	 for ( i=0; cardsArrayShow[index].mem[i] != -1; i++ ){
	 	 str = "";
		 str = "0x" + str.setNum ( cardsArrayShow[index].mem[i], 16 );
		 comboMEM->insertItem( str );		
	 }	

	 lblHint_text->setText( cardsArrayShow[index].modulename );	

	if ( bFirstShow ){	

//Set previous values......			
			if  ( *(sound->ReturnType()) != -1 ){
				for ( int j=0; j<cardsArrayShow_size; j++ ){
						if ( cardsArrayShow[j].type == *( sound->ReturnType())  )
				 		*( sound->ReturnCount()) = j;		
				}
  			listType->setCurrentItem( *( sound->ReturnCount()) );
 			}
			listType->setFocus();

   		if ( cardsArrayShow[index].io[0] != -1 ){		
	 				str_temp =	*( sound->ReturnStr( 1 ) );	 	
   				comboIO->setCurrentItem( FindItem( str_temp, comboIO ) );
	 		}

			if ( cardsArrayShow[index].irq[0] != -1 ){	
	 				str_temp =	*( sound->ReturnStr( 0 ) );	 	
   				comboIRQ->setCurrentItem( FindItem( str_temp, comboIRQ ) );
	    }

		  if ( cardsArrayShow[index].dma[0] != -1 ){	
					str_temp = *( sound->ReturnStr( 2 ) );	 	
			 		comboDMA->setCurrentItem( FindItem( str_temp, comboDMA ) );
	 		}
	
			if ( cardsArrayShow[index].dma2[0] != -1 ){
					str_temp = *( sound->ReturnStr( 6 ) );
					comboDMA2->setCurrentItem( FindItem( str_temp, comboDMA2 ) );
	 		}
	
      if ( cardsArrayShow[index].mpu_io[0] != -1 ){
	 			str_temp = *( sound->ReturnStr( 7 ) );
				comboMPU_IO->setCurrentItem( FindItem( str_temp, comboMPU_IO ) );
	    }	
			
	 	  if ( cardsArrayShow[index].mpu_irq[0] != -1 ){
	 			str_temp = *( sound->ReturnStr( 8 ) ) ;	
				comboMPU_IRQ->setCurrentItem( FindItem( str_temp, comboMPU_IRQ ) );
	 		}	 	
			
			if ( cardsArrayShow[index].mem[0] != -1 ){
				str_temp = *( sound->ReturnStr( 9 ) ) ;
				comboMEM->setCurrentItem( FindItem( str_temp, comboMEM ) );
	 		}			

		  bFirstShow = false; 					
  }

}

/********************************************************************************************/
void LSound::VarsPubNull(){
	sPubType = "";
	sPubIRQ = "";
  sPubIO = "";
	sPubDMA = "";
	sPubModulename = "";
	sPubDescr = "";
  sPubDMA2 = "";
  sPubMPU_IO = "";
  sPubMPU_IRQ = "";
	sPubMEM = "";
  iPubType = -1;
	iPubCount = -1;
}

/***************************************************************************/
void LSound::InitializeValues(){

 VarsPubNull();
 int inum = 0;

 int cardsArray_size = sizeof( cardsArray )/sizeof( struct SoundCardsTable );

 for( int i = 0; i < cardsArray_size; i++ ){
		if (cardsArray[i].type != UNSPEC){
			cardsArrayShow[inum] = cardsArray[i];			
      inum++;
    }
  }
 cardsArrayShow_size = inum;
 Sort( cardsArrayShow, cardsArrayShow_size );	

}

/****************************************************************************************/
void LSound::LoadMidi( int icard_type ){

/*		if ( (cardsArray[icard_type].type == SOUNDSCAPE) ||
				 (cardsArray[icard_type]).type == ADLIB )
  		system ( "modprobe -qs sound" );
  		else{
					int result = system ( "modprobe -qs midi" );
					printf("Result of midi modprobe: %d \n", result );
			}*/

#ifndef GBLI_DEVEL
		int iRes = system("modprobe -qs sound");
		if (iRes<0) Log::logMain(2,"Load midi:modprobe -qs sound: %d",iRes);
		if ( ( cardsArray[icard_type].flags & HAS_OPL3 ) ||
         ( cardsArray[icard_type].flags & HAS_MPU401 ) ||
				 ( cardsArray[icard_type].type == SB32 )  ){
				iRes = system ( "modprobe -qs midi" );
				if (iRes<0) Log::logMain(2,"Load midi:modprobe -qs midi: %d",iRes);
		}
#endif
}
/****************************************************************************************/
void LSound:: MixerOpen(){
	mixer = open("/dev/mixer", O_RDWR);
}

/****************************************************************************************/
int LSound::GetModuleFromProc( QString str1 ){

	str1 = str1 + " ";	
  int num_spaces1 = str1.contains(" ");
	int num_spaces2, count, index1, index2, iequal_words;
	int array_index = -1;
	QString str2;	
	int num_current = 0;
	int num_previous = 0;
	bool b_exit;

	int i_array_size = sizeof( cardsArray )/sizeof( struct SoundCardsTable );			

	int current_length = 0;
	int previous_length = 0;

	for ( int j=0; j<i_array_size; j++ ){
				str2 = cardsArray[j].descr;
				current_length = str2.length();
				str2 = str2 + " ";
				num_spaces2 = str2.contains(" ");					
				if ( num_spaces1 < num_spaces2 ) count = num_spaces1;
				else count = num_spaces2;

        index1=0;
				index2=0;
        num_current=0;					
				iequal_words=0;				
							
            b_exit = false;					
						while ( (iequal_words < count) && ( !b_exit ) ){
 	        		index1=str1.find( " ", index1+1 );
			   			index2=str2.find( " ", index2+1 );		
	 						if ( str1.left( index1 ) == str2.left( index2 ) ){	
              		num_current++;
							}
							else b_exit = true;	
				    	iequal_words++;
						}
						
            if ( num_current > num_previous ){
								array_index = j;
								num_previous = num_current;
								previous_length = current_length;	
						}					
						else if ( (num_current == num_previous) && (num_current != 0) ){														
								if ( (current_length < previous_length) || (previous_length==0) ){
										array_index = j;
										previous_length = current_length;
								}		
						}		
	}

//For testing
/*	if ( array_index != -1 ){
		printf ( "Array_index: %d \n", array_index );
		printf ( "Description: %s \n", cardsArray[array_index].descr );
	}
	else printf("Sorry, I didn't find anythig ;( \n");*/
//For testing
	
return array_index;
	
}

/****************************************************************************************/
bool LSound::EmuLoad(){
#ifndef GBLI_DEVEL
	int result = system( "modprobe -qs emu10k1" );
	//if (result!=0) result = system("lsmod|grep -w emu10k1");
	if ( !result ) system ( "modprobe -qs sound" );
	if ( !result ) return TRUE;
  else return FALSE;
#else
	return false;
#endif
}

/****************************************************************************************/
LSound::LSound(QWidget *parent, const char *name)
  : GBLIPage(parent, name), b_enable( false )
{

   InitializeValues();
//It' s neccessary to change the path of sndconfig	 	

#ifdef SOUND_BY_SNDCONFIG
   QString s_sndconfig = "/usr/sbin/sndconfig";
   #ifdef GBLI_DEVEL
      s_sndconfig = "sndconfig"; 	
   #endif

	 s_sndconfig = s_sndconfig + " --quiet";	
//	 printf("Sndconfig: %s \n", s_sndconfig.ascii() );	
   int snd_result = system( s_sndconfig );
	 if (snd_result<0) Log::logMain(2,"sndconfig: %d",snd_result);
//   printf ( "SND_RESULT: %d\n", snd_result );
#else
#ifdef nooooooooooooooooooo
   #ifndef GBLI_DEVEL
      system( QString("rm -f /etc/sysconfig/hwconf /etc/conf.modules "+ QString( CONF_MODULES ) +" /etc/sysconfig/soundcard").ascii() );
      system( "kudzu -q -s -t10" );
   #endif
#endif
#endif
   LSound :: hasMidi();
   mixer = -1;
	 MixerOpen();
//	 printf( "Mixer(LSound): %d \n", mixer ); 	

   QHBoxLayout *hl = new QHBoxLayout( );
   QHBoxLayout *hbox = new QHBoxLayout( this,6 );

   vbox = new QVBoxLayout( 6 );
	 hbox->addStretch( 1 );
	 hbox->addLayout( vbox, 8 );
	 hbox->addStretch( 1 );

   label = new QLabel(tr("A sound card was detected:"), this);
   label->setAutoMask( true );
   vbox->addWidget(label);
   vbox->addStretch( 1 );

 	 QGridLayout *grid = new QGridLayout(vbox,1,1);
	 lblAudio =  new QLabel ( tr("Audio devices:"), this );	
   lblAudio->setAutoMask( true );
	 lblAudio_text = new QLabel ( "", this );
   lblAudio_text->setAutoMask( true );
	 lblSynth = new QLabel ( tr("Synth devices:"), this );		
   lblSynth->setAutoMask( true );
	 lblSynth_text = new QLabel ( "", this ); 	
   lblSynth_text->setAutoMask( true );
	 lblMidi = new QLabel ( tr("Midi devices:"), this );	
   lblMidi->setAutoMask( true );
	 lblMidi_text = new QLabel ( "", this );	
   lblMidi_text->setAutoMask( true );
	 lblTimers = new QLabel ( tr("Timers:"), this );	
   lblTimers->setAutoMask( true );
	 lblTimers_text = new QLabel ( "", this );	
   lblTimers_text->setAutoMask( true );
	 lblMixers = new QLabel ( tr("Mixers:"), this );	
   lblMixers->setAutoMask( true );
	 lblMixers_text = new QLabel ( "", this );		
   lblMixers_text->setAutoMask( true );
 	 grid->addWidget( lblAudio, 0, 0 );
 	 grid->addWidget( lblAudio_text, 0, 1 );
	 grid->addWidget( lblSynth, 1, 0 );		
	 grid->addWidget( lblSynth_text, 1, 1 );	
	 grid->addWidget( lblMidi, 2, 0 );
	 grid->addWidget( lblMidi_text, 2, 1 );	
	 grid->addWidget( lblTimers, 3, 0 );	
	 grid->addWidget( lblTimers_text, 3, 1 );	
	 grid->addWidget( lblMixers, 4, 0 );	
	 grid->addWidget( lblMixers_text, 4, 1 );	
   grid->setColStretch( 0,1 );
   grid->setColStretch( 1,2 );
		
	label->setText( tr("Sound card is not detected") );			
	lblAudio->hide();
  lblSynth->hide();
  lblMidi->hide();
	lblTimers->hide();
	lblMixers->hide();	

	 int i_type = CardType();  		
	 if ( i_type != -1 ) LoadMidi( i_type );	


	 bool b_proc_exists = false;	
	
	 if ( b_proc_exists = ReadInfo() ) {						
				//	printf("Proc exists...\n");
					label->setText( tr("A sound card was detected:") );			
        	lblAudio->show();
		      b_enable = true;
	 }					
	 	
	 if ( i_type != -1  ) {
			//	printf("Soundcard exists\n");
				iPubType = i_type;
	//			printf("iPubType (soundcard) : %d \n", iPubType );
				sPubModulename = cardsArray[ iPubType ].modulename;
				sPubDescr = cardsArray[ iPubType ].descr;	
				sPubType = CardTypeToString( cardsArray[ iPubType ].type  );
	/*			printf( "iPubType (cardType) : %d \n", iPubType );
				printf( "sPubModulename (cardType) %s \n", sPubModulename.ascii() );
				printf( "sPubDescr (cardType): %s \n", sPubDescr.ascii() );
				printf( "sPubType (cardType): %s \n", sPubType.ascii() );*/
				if ( !b_proc_exists ){
						b_enable = true;	
//		   			LoadMidi( i_type );				 		 	
						label->setText( tr("A sound card was detected:") );
        		lblAudio->show();	
		    		lblAudio_text -> setText ( sPubDescr );
			 }	
	}	

	QPushButton *buttadv = new QPushButton( tr( "Manual setup" ), this);

  b_emu = false;
	if ( i_type == -1 ){
		 b_emu = EmuLoad();
		 if ( b_emu ){
		 			label->setText( tr("A sound card was detected:") );
        	lblAudio->show();	
		    	lblAudio_text -> setText ( "Sound Blaster Live" );
					buttadv -> setEnabled( FALSE ); 		
					b_enable = true;
		 }else{
				// IBM ThinkPad
Log::logMain(33, "probing IBM ThinkPad sound card..." );
					int res = system( "modprobe -qs cs46xx" );
					if( !res ){
Log::logMain(33, " successful\n" );
  		 			label->setText( tr("A sound card was detected:") );
          	lblAudio->show();	
  		    	lblAudio_text -> setText ( "Crystal 4280/461x + AC97 Audio" );
  					buttadv -> setEnabled( FALSE ); 		
  					b_enable = true;
					}else{
Log::logMain(33, " failed\n" );
					}
			}
  }

//	printf ( "iPubType (LSound): %d \n", iPubType );
/*	if ( b_enable )  printf("B_enable is TRUE \n");
	else printf("B_enable is FALSE \n");*/

//Temporarily
/*	printf( "sPubType: %s \n",sPubType.ascii()  );
  printf( "sPubIRQ: %s \n", sPubIRQ.ascii() );
  printf( "sPubIO: %s \n", sPubIO.ascii() );
	printf( "sPubDMA: %s \n", sPubDMA.ascii() );
	printf( "sPubModulename: %s \n", sPubModulename.ascii() );
  printf( "sPubDescr: %s \n", sPubDescr.ascii() );
  printf( "sPubDMA2: %s \n", sPubDMA2.ascii() );
  printf( "sPubMPU_IO: %s \n", sPubMPU_IO.ascii() );
	printf( "sPubMPU_IRQ: %s \n", sPubMPU_IRQ.ascii() );
	printf( "sPubMEM: %s \n", sPubMEM.ascii() );*/

int cardsArray_index = -1;
  if ( ( !sProcDescr.isEmpty() ) && ( i_type == -1 ) ){
//printf("111\n");
			cardsArray_index = GetModuleFromProc( sProcDescr );
			if ( cardsArray_index == -1 )
						cardsArray_index = GetModuleFromProc( sSynthDescr );
      if ( cardsArray_index != -1 ){
//printf("222\n");
				iPubType = cardsArray[ cardsArray_index ].type;
				sPubModulename = cardsArray[ cardsArray_index ].modulename;
				sPubDescr = cardsArray[ cardsArray_index ].descr;	
				sPubType = CardTypeToString( cardsArray[ cardsArray_index ].type  );
			/*	printf( "iPubType (proc) : %d \n", iPubType );
				printf( "sPubModulename (proc) %s \n", sPubModulename.ascii() );
				printf( "sPubDescr (proc): %s \n", sPubDescr.ascii() );
				printf( "sPubType (proc): %s \n", sPubType.ascii() );	*/
			}             		

		 for ( int m=0; m<cardsArrayShow_size; m++ ){
					if ( cardsArrayShow[ m ].type == iPubType )
							iPubCount = m;
		 }		
	}
//Temporarily for testing

/*printf ( "iPubType2 (LSound): %d \n", iPubType );*/

/*if ( iPubType != -1 ){
  WriteConfModules(); // move to finish();
	WriteCardSettings( CardTypeToString( iPubType ) );	
}*/

//Temporarily for testing

   vbox->addStretch( 1 );

   QFrame *line = new QFrame(this);
   line->setFrameStyle(QFrame::HLine | QFrame::Sunken);
   vbox->addWidget(line);

   grid = new QGridLayout(vbox, 1, 1, 6);

   wav = new QPushButton(this);
   wav->setPixmap(LIconLoader::loadIcon( "play.xpm" ));
   wav->setFixedSize(32,32);
   grid->addWidget(wav,0, 0);
   lblDigSnd = new QLabel(tr("Test digital sound"), this);
   lblDigSnd->setAutoMask( true );
   grid->addWidget( lblDigSnd, 0, 1 );
	 connect(wav, SIGNAL(clicked()), this, SLOT(testWav()));	
	 wav->setEnabled( FALSE );
	 lblDigSnd->setEnabled( FALSE );		

   if (LSound::hasSound())
    {
			if ( b_enable ){					
					wav->setEnabled( TRUE );
					lblDigSnd->setEnabled( TRUE );
			}
    }
	
		midi = new QPushButton(this);
 		midi->setPixmap(LIconLoader::loadIcon("play.xpm"));
 		midi->setFixedSize(32,32);
 		grid->addWidget(midi,0, 2);
 		lblMidiSnd = new QLabel(tr("Test MIDI playback"), this);
		lblMidiSnd->setAutoMask( true );
 		grid->addWidget(lblMidiSnd,0 , 3);
		midi->hide();
		lblMidiSnd->hide();
		
//		printf("sPubModulename: %s \n", sPubModulename.ascii() );

		if (LSound::hasMidi())
 		{
        if ( b_enable ){
						if ( sPubModulename != "gus" ){
								midi->show();
								lblMidiSnd->show();				
								midi->setEnabled( TRUE );
			  				lblMidiSnd->setEnabled( TRUE );
						}
				}
 		}

    QAccel *a = new QAccel(this);
    a->connectItem(a->insertItem(Qt::CTRL+Qt::ALT+Qt::Key_S), this, SLOT(testMpg()));
  
    grid->setColStretch(1,1);
    vbox->addSpacing(12);

		//mixer = -1;
		//MixerOpen();
  //  mixer = open("/dev/mixer", O_RDWR);
//		printf( "Mixer: %d \n", mixer );
//	mixer = 0;

      QGridLayout *mgrid = new QGridLayout(vbox,1,1,6);
      lbl_volume = new QLabel(tr("Volume:"),this);
			lbl_volume->setAutoMask( true );
      mgrid->addWidget( lbl_volume, 0, 0 );
      slider1 = new QSlider(0,100,25,75,QSlider::Horizontal,this);
      slider1->setTickmarks(QSlider::Below);
      slider1->setTickInterval(25);			
      mgrid->addWidget(slider1, 0, 1);
      mgrid->setColStretch(1,1);

      connect(slider1, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));

      lbl_balance = new QLabel(tr("Balance:"),this);
			lbl_balance->setAutoMask( true );
      mgrid->addWidget( lbl_balance, 1, 0 );			

      slider2 = new QSlider(0,100,25,50,QSlider::Horizontal,this);
      slider2->setTickmarks(QSlider::Below);
      slider2->setTickInterval(10);
      mgrid->addWidget(slider2,1,1);

      connect(slider2, SIGNAL(valueChanged(int)), this, SLOT(balanceChanged(int)));

      volume = 75;
      balance = 50;
      setMixer(volume, balance);

			lbl_volume->setEnabled( FALSE );
			slider1->setEnabled( FALSE );
			lbl_balance->setEnabled( FALSE );
			slider2->setEnabled( FALSE );			

    if ( mixer >= 0 )
    {
				if ( b_enable ){					
					lbl_volume->setEnabled( TRUE );
					slider1->setEnabled( TRUE);
					lbl_balance->setEnabled( TRUE );
					slider2->setEnabled( TRUE );	
				}
     }

//--------------------------------------------------------------------------
    //button "Advanced" adding

     hl->addStretch( 1 );
     hl->addWidget( buttadv );

     vbox->addStretch( 1 );
     vbox->addLayout( hl );

     connect( buttadv, SIGNAL(clicked()), this, SLOT(showDialog()) );
     connect(midi, SIGNAL(clicked()), this, SLOT(testMidi()));
     vbox->addStretch();
//     printf( "finished sound\n" );
}


/*****************************************************************************************************/

QString LSound :: CardTypeToString( int itype ){

    switch ( itype ){
 #
		case ACERNB: return "ACERNB";
    case SBORIG: return "SBORIG";
    case ALIM15XX: return "ALIM15XX";
    case SBPRO: return "SBPRO";
    case SB16: return "SB16";
    case SB32: return "SB32";
		case SBLIVE: return "SBLIVE"; //added by AA
    case TRIDENT: return "TRIDENT";
    case AD1816: return "AD1816";
    case ALS007: 	return "ALS007";
    case ALS100:	return "ALS100";
    case SMGAMES: return "SMGAMES";
    case JAZZ16:	return "JAZZ16";
    case CMI8330: return "CMI8330";
    case CS4232: return "CS4232";
    case CS46XX: return "CS46XX";
    case CS461X: return "CS461X";
		case CS4281: return "CS4281";//Added by AA
    case ESS1868: return "ESS1868";
    case ESSMAESTRO: return "ESSMAESTRO";
    case ESS1688:	return "ESS1688";
    case ESS688: return "ESS688";
    case MOZART: return "MOZART";
    case MAD16PRO: return "MAD16PRO";
    case MSND_CLASSIC: return "MSND_CLASSIC";
    case MSND_PINNACLE: return "MSND_PINNACLE";
    case OPL3_SA: return "OPL3_SA";
    case OPL3_SAX: return "OPL3_SAX";
		case WAVEFRONT: return "WAVEFRONT";//Added by AA
    case ES1370: return "ES1370";
    case ES1371: return "ES1371";
    case ESSSOLO1:	return "ESSSOLO1";
    case SONICVIBES: return "SONICVIBES";
    case GUS: return "GUS";
    case GUSMAX:	return "GUSMAX";
    case GUSPNP:	return "GUSPNP";
    case I810:	return "I810";
    case PAS16: 	return "PAS16";
    case WSS: return "WSS";
    case YMFPCI: return "YMFPCI";
    case CMI8X38: 	return "CMI8X38";
    case VIA82CXXX: return "VI82CXXX";
    case SGALAXY: return "SGALAXY";
    case DESKPROXL: 	return "DESKPROXL";
    case AUDIOTRIX: 	return "AUDIOTRIX";
    case ADLIB: return "ADLIB";
    case SOUNDSCAPE: return "SOUNDSCAPE";
    case SSVIVO: return "SSVIVO";
    case PSS: 	return "PSS";
    case MIROSOUND: return "MIROSOUND";
    case NEOMAGIC: return "NEOMAGIC";
    case AMD7930: 	return "AMD7930";
    case CS4231: return "CS4231";
    case DBRI:	return "DBRI";
    case UNSPEC: default:	return "UNSPEC";
    }



}

/*****************************************************************************************************/

int LSound::CardType(){

  QString sPath;
  QString sCardType = "";

	sPath = "/etc/sysconfig/soundcard";

  QString sline = "CARDTYPE=";
//  printf( "Length of CARDTYPE:%d\n", sline.length() );

  QFile fcard( sPath );
  if ( fcard.open(IO_ReadOnly) ){
    QTextStream ts( &fcard );
    QString str;
    while ( !ts.eof() ){
     str = ts.readLine();
     if ( str.left( 1 ) == "#" )   continue;
     if ( str.find( sline ) >= 0 ){
//       printf("Dlina stroki: %d\n", str.length());
//       bval = true;
       sCardType = str.mid( sline.length(), str.length()-sline.length() );
     }
    }
   }

/*  printf( "CARDTYPE:%s\n", sCardType.ascii() );
  printf( "Length:%d\n", sCardType.length() );*/
  if ( !sCardType.isEmpty() ){
		 int i_array_size = sizeof( cardsArray )/sizeof( struct SoundCardsTable );	
		 for ( int i=0; i<i_array_size; i++ ){
				if ( CardTypeToString( cardsArray[i].type ) == sCardType )
				return cardsArray[i].type;
		 }			
	}
	return -1;

}

/*****************************************************************************************************/

bool LSound::ReadInfo(){

  bool bval = false;		
  QString sPathSound = "/proc/sound";

	if ( !QFile( sPathSound ).exists() ){
				sPathSound = NEWROOT + sPathSound;
//				printf ("sPathSound: %s \n", sPathSound.ascii() );
  }


  QFile sound( sPathSound );
  if (sound.open(IO_ReadOnly)){	
//      QGridLayout *grid = new QGridLayout(vbox,1,1);
      QTextStream ts(&sound);
      QString label_txt;
			QString lab_text = "";
      QStrList contents;
      QString line = ""; 						
 	
      while ( !line.isEmpty() ){
	      line = ts.readLine();
	      if (line.isEmpty())
	        line = ts.readLine();

	      if (line == "Audio devices:"){
	        contents.clear();
	      }

    	  else if (line == "Synth devices:"){
	        if (contents.count() > 0){
							 sProcDescr = contents.first();
//		   	  		 printf("sProcDescr:%s \n", sProcDescr.ascii());								
		        for (const char *it = contents.first(); it != 0; it = contents.next()){																					
								if ( !lab_text.isEmpty() ) lab_text = lab_text + "\n";
						  	lab_text = lab_text + it;
            }
						lblAudio_text -> setText(  lab_text );
            bval = true;
	    	  }	
  	      contents.clear();
	      }

	        else if (line == "Midi devices:"){
	        if (contents.count() > 0 && bval ){
							lblSynth -> show();
						 lab_text = "";	
  						 sSynthDescr = contents.first();
  			//		 sProcDescr = contents.first();
		   	//		 printf("sProcDescr:%s \n", sProcDescr.ascii());								
		         for (const char *it = contents.first(); it != 0; it = contents.next()){
							 if ( !lab_text.isEmpty() )	lab_text = lab_text + "\n";
							 lab_text = lab_text + it;
						 }	
             lblSynth_text -> setText( lab_text );		
	      	}	
	        contents.clear();
	      }

  	    else if (line == "Timers:"){
         if (contents.count() > 0 && bval){
						lblMidi->show();
						lab_text = "";
		        for (const char *it = contents.first(); it != 0; it = contents.next()){
						if ( !lab_text.isEmpty() ) lab_text = lab_text + "\n";
						lab_text = lab_text + it;
						}
						lblMidi_text -> setText( lab_text );	
	    	 }	

	       contents.clear();
	      }
	
        else if (line == "Mixers:"){
          if (contents.count() > 0 && bval){
						lblTimers->show();
						lab_text = "";
		        for (const char *it = contents.first(); it != 0; it = contents.next()){
								if ( !lab_text.isEmpty() ) lab_text = lab_text + "\n";
								lab_text = lab_text + it;
						}			
						lblTimers_text -> setText( lab_text );		
  	    	}	
	        contents.clear();
	      }
	
        else if (!line.isEmpty()){
	         if (int pos = line.find(": ") > 0)
		          line.remove(0,pos+2);
	         contents.append(line);
	     }
   	  }


      if (contents.count() > 0 && bval){
				lblMixers->show();	
				lab_text = "";
	      for (const char *it = contents.first(); it != 0; it = contents.next()){
					if ( !lab_text.isEmpty() ) lab_text = lab_text + "\n";
					lab_text = lab_text + it;
				}
				lblMixers_text->setText( lab_text );
	    }	
				
      sound.close();
  }

  return bval;

}

/*****************************************************************************************************/

void LSound::showDialog(){
  //create a "Advanced" dialog
	int res;
	res = system( "killall -9 -q sox >/dev/null 2>&1" );
	Log::logMain( 34, "Kill sox: %d \n", res );	
	res=0;
	res = system( "killall -9 -q playmidi >/dev/null 2>&1" );
  Log::logMain( 34, "Kill playmidi result: %d \n", res );
  AdvSoundDialog *adialog = new AdvSoundDialog( this, this,  "Sound manual configuration" );
  adialog->exec();
}

/*****************************************************************************************************/
bool LSound::CheckKernFile(){
bool err = false;					

system("cp /var/log/messages.kern /var/log/messages.kern2");
QString str_mesages_path = "/var/log/messages.kern2";	
    	QFile fkern( str_mesages_path );
		if ( fkern.open( IO_ReadOnly ) ){
				QString line;	
				QTextStream ts( &fkern );
				while ( !ts.eof() ){
					 line = ts.readLine();				
					 if ( line.find("Sound:") != -1 ){
//							 printf("Line: %s \n", line.ascii() );
							 err=true;
					 }	
					 else  if ( line.find( "install last message repeated" ) != -1 && err )
     						err = true;
					 else err = false;							
				}
				fkern.close();	
		}else
fkern.remove();	
		
return err;

}
/*****************************************************************************************************/


LSound::~LSound()
{
  if (mixer >= 0)
    ::close(mixer);
}


/*****************************************************************************************************/
void LSound::testWav()
{
  QFile data( NEWROOT +"/tmp/test.wav");
  if (!data.exists()) {
    ::mkdir( (NEWROOT +"/tmp/").latin1(),01777);
    system( ("cp -af "+ QString( GBLI_DATA_PATH ) +"/sounds/test.wav "+ NEWROOT +"/tmp/test.wav").ascii() );
  }
	int res;
  res = system( ("sox "+ NEWROOT +"/tmp/test.wav -t .ossdsp /dev/dsp &").latin1() );
	sleep ( 1 );	
  if ( CheckKernFile() ){	
			QString messg = tr("Soundcard configuration is incorrect...\nPlease, change settings\n");
					QMessageBox::critical (
       	    		this, tr("Warning"),
     	    			messg,
  	      			tr("OK") , 0, "", 1 );
  }
/*	int pid;
	pid = fork();
	if ( !pid ){
    int pid2 = fork();
		if ( !pid2 ){		
		::execlp ( "sox", "sox", (NEWROOT +"/tmp/test.wav").latin1(), "-t", ".ossdsp", "/dev/dsp", 0);
    ::exit(0);
		}
		::waitpid( pid2, NULL, 0 );
		printf("PID2 finished...\n");
		::exit(0);
				 if ( CheckKernFile() ){
						printf("CheckKernFile() is TRUE \n");
						QString messg = tr("Soundcard configuration is failed...\nPlease, change settings\n");
						QMessageBox::critical (
       	    		this, tr("Warning"),
     	    			tr( messg ),
  	      			tr("OK") , 0, "", 1 );
  				}
				 else 	printf("CheckKernFile() is FALSE\n");
	}*/
}

/*************************************************************************/
void LSound::testMidi()
{

//  ClearKernFile();
//printf("Test MIDI \n");
/*  QFile data( NEWROOT +"/tmp/test.mid" );
  if (!data.exists()) {
    ::mkdir( (NEWROOT +"/tmp/").latin1(),01777);
    system( ("cp ./sounds/test.mid "+ NEWROOT +"/tmp/test.mid").latin1() );
  }
  system( ("playmidi "+ NEWROOT +"/tmp/test.mid &").latin1() );*/

  int i_type=-1;
  if ( iPubType != -1 ) i_type = iPubType;
	else ( i_type = CardType() );
	Log::logMain(38,"testMidi: i_type: %d",i_type);
//  printf("INDEX CARD TYPE: %d\n", i_type );	
  if ( i_type != -1 ){	

/*					LSound::hasMidi();
					LoadMidi( i_type );	*/


/*			if ( (cardsArray[i_type].type == SOUNDSCAPE) ||
				 (cardsArray[i_type]).type == ADLIB )
  		system ( "modprobe -qs sound" );
  		else
			system ( "modprobe -qs midi" );*/

			QFile data( NEWROOT +"/tmp/test.mid");
 		 	if (!data.exists()) {
			 Log::logMain(38,(NEWROOT +"/tmp/test.mid  does not exist").latin1());
   		 ::mkdir( (NEWROOT +"/tmp/").latin1(),01777);
   		 int iRes = system( ("cp -af "+ QString( GBLI_DATA_PATH ) +"/sounds/test.mid "+ NEWROOT +"/tmp/test.mid").ascii() );
			 if (iRes<0) Log::logMain(2,"Result of copying test.mid: %d",iRes);
  		}
	
			QString tmp_midi = "playmidi ";

  		QString str_play_midi = tmp_midi + NEWROOT +"/tmp/test.mid";
//			printf( "%s\n", str_play_midi.ascii() );
  		if ( cardsArray[i_type].flags & HAS_AWE ){
					str_play_midi = str_play_midi + " -a &";
//					printf(" -a \n");
			}	
			else
				if ( (cardsArray[i_type].flags & HAS_MPU401 ) ||
						 (cardsArray[i_type].type == SOUNDSCAPE) ){
				str_play_midi = str_play_midi + " -e &";
//				printf(" -e \n");
				}
  		else
				if ( (cardsArray[i_type].flags & HAS_OPL3 ) ||
						 (cardsArray[i_type].type == ADLIB ) ){
				str_play_midi = str_play_midi + " -f &";	
//  			printf(" -f \n");
				}
			else {
				str_play_midi = str_play_midi + " -g &";
//   			printf(" -g \n");
			}
		
		int res;
 	 	res = system( str_play_midi );
		if (res<0) Log::logMain(2,"system(%s) %d",str_play_midi,res);
		sleep ( 1 );
	   if ( 	CheckKernFile()  ){
			QString messg = tr("Soundcard configuration is incorrect...\nPlease, change settings\n");
								QMessageBox::critical (
        	  		this, tr("Warning"),
  	     				messg,
  	      			tr("OK") , 0, "", 1   );
					
  	}		
	}

}

/*************************************************************************/

void LSound::testMpg()
{
  QFile data( NEWROOT +"/tmp/test.mp3" );
  if (!data.exists()) {
    ::mkdir( (NEWROOT +"/tmp/").latin1(), 01777 );
    system( ("cp -af "+ QString( GBLI_DATA_PATH ) +"/sounds/test.mp3 "+ NEWROOT +"/tmp/test.mp3").ascii() );
  }
  system( ("mpg123 "+ NEWROOT +"/tmp/test.mp3 &").latin1() );
}

/*************************************************************************/

void LSound::balanceChanged(int value)
{
  balance = value;
  setMixer(volume, balance);
}

/*************************************************************************/

void LSound::volumeChanged(int value)
{
  volume = value;
  setMixer(volume, balance);
}

/*************************************************************************/

void LSound::setMixer(int vol, int bal)
{
  if ( mixer<0 ) MixerOpen();

  int volume, vr, vl;

  if (mixer < 0)
    return;

  // calculate the volumes
  vr = vl = 100;
  if (bal<50)
    vr = 2*bal;
  else
    vl = 200 - 2*bal;

  vl = vl*vol/100;
  vr = vr*vol/100;
  volume = vl + (vr << 8);

  int mask;
  ioctl(mixer, SOUND_MIXER_READ_DEVMASK, &mask);
  for (int dev=0; dev<SOUND_MIXER_NRDEVICES; dev++)
    if ((mask & (1 << dev)) != 0)
      if (dev != SOUND_MIXER_MIC)
	ioctl(mixer, MIXER_WRITE(dev), &volume);
      else
	{
	  volume = 0;
	  ioctl(mixer, MIXER_WRITE(dev), &volume);
	}
}

/*************************************************************************/
void LSound::WriteConfModules(){

   strMod = NEWROOT + CONF_MODULES;
	
   struct stat memstat;
	 FILE *fconf; 	

/*   printf("Write configuration to %s...\n", CONF_MODULES );	
   printf("Type: %d \n", iPubType );*/

/*!!!!! If file exists, append data to file!!!!*/

   if ( (fconf = fopen( strMod, "a" )) != NULL	 ){
 			fprintf( fconf, "alias sound-slot-0 %s\n", sPubModulename.ascii() );

// always use dmabuf to force loading modules on startup. Maddy.
//yp			fprintf( fconf, "options sound-slot-0 dmabuf=1\n" );

  /*If the user has a ISA card and > 16MB of memory, we want to lock ISA
     dma buffers */
   if ( !stat( "/proc/kcore", &memstat ) )
       if ( (cardsArray[iPubType].bus == isa) && (memstat.st_size > 16777216) )
	       	fprintf( fconf, "pre-install sound-slot-0 /sbin/insmod sound dmabuf=1\n" );

/*printf("TypePub: %d \n", iPubType );
printf("TypeString: %s \n", CardTypeToString( iPubType ).ascii()  );
printf("Descr: %s \n", cardsArray[iPubType].descr );
printf("Modulename: %s\n", cardsArray[iPubType].modulename );*/

      if ( cardsArray[iPubType].flags & HAS_OPL3 ){
//printf("HAS_OPL3 \n");
					if ( !(cardsArray[iPubType].flags & HAS_AWE) )
				  fprintf( fconf, "alias midi opl3\n" );
      fprintf( fconf, "options opl3 io=0x388\n" );
			}

			if ( cardsArray[iPubType].flags & HAS_MPU401 ){
					fprintf( fconf, "alias midi mpu401\n" );					
					fprintf( fconf, "options mpu401 io=%s irq=%s \n",
                   sPubMPU_IO.ascii(), sPubMPU_IRQ.ascii() );
					if ( cardsArray[iPubType].type != MSND_PINNACLE ){
						sPubMPU_IO="";
   					sPubMPU_IRQ="";
					}
			}
			
      if ( cardsArray[iPubType].flags & HAS_AWE ){
					fprintf( fconf,
          "alias midi awe_wave\npost-install awe_wave /bin/sfxload /etc/midi/GU11-ROM.SF2\n" );
			}

			if ( cardsArray[iPubType].flags & SB_AS_LIB )
					fprintf( fconf, "options sb %s=1\n", sPubModulename.ascii() );


      if ( cardsArray[iPubType].flags & ACI )
      		fprintf( fconf,
          "post-install %s /sbin/modprobe \"aci\"\n", sPubModulename.ascii() );

			if ( cardsArray[iPubType].flags & NO_INPUT_REQ ){
					if ( cardsArray[iPubType].type == ADLIB ){
							fprintf( fconf, "options adlib_card io=0x388\n" );		
					}
      } else if ( ( !sPubIRQ.isEmpty() ) && ( !sPubIO.isEmpty() ) )
        {
      		fprintf( fconf, "options %s", sPubModulename.ascii() );
					if ( !sPubIO.isEmpty() ){
							if ( cardsArray[iPubType].type == OPL3_SAX )
									fprintf( fconf, " miss_io=%s", sPubIO.ascii() );						
              else {
									if ( sPubModulename == "mad16" ){
                        QString str_cut = sPubIO.right( sPubIO.length() - ( sPubIO.find("x",0) + 1 ) );
                        bool ok;
                        int i_num = str_cut.toInt( &ok, 16 );
												if ( i_num % 16 ) i_num -= 4;
                        sPubIO = "";
                        sPubIO = "0x" + sPubIO.setNum ( i_num, 16 );
                        fprintf( fconf, " io=%s", sPubIO.ascii() );	
									}//if ( sPubModulename == "mad16" ){		
                  else
                  		fprintf( fconf, " io=%s", sPubIO.ascii() );
							} //else
							
					} // ( !sPubIO.isEmpty() )
					if ( !sPubIRQ.isEmpty() ) fprintf ( fconf, " irq=%s", sPubIRQ.ascii() );	
					if ( !sPubDMA.isEmpty() ) fprintf ( fconf, " dma=%s", sPubDMA.ascii() );
					if ( !sPubDMA2.isEmpty() ){
							int idma2 = sPubDMA2.toInt();
							if(  ( sPubModulename == "sb" ) ||
                   ( sPubModulename == "pas2" ) ||
                   ( sPubModulename == "gus" ) ||
                   ( sPubModulename == "mad16" ) ){ 									
/*printf("DMA2=DMA16 \n");
printf("DMA2=%d \n", idma2 );
printf("DMA2 string: %s\n", sPubDMA2.ascii() );*/
									if (( iPubType != SB16 ) || ( idma2 >= 5 )){
											fprintf( fconf, " dma16=%s", sPubDMA2.ascii() );

									}	
              }else
									if ( iPubType == MSND_PINNACLE )
							    		fprintf( fconf, " cfg=%s", sPubDMA2.ascii() );
									else fprintf ( fconf, " dma2=%s", sPubDMA2.ascii() );
          }// ( !sPubDMA2.isEmpty() )
					
					if ( ( !sPubMPU_IO.isEmpty() ) && ( iPubType != SGALAXY ) ){
							if ( cardsArray[iPubType].flags & LOAD_CS4232  ){
									fprintf ( fconf, " synthio=%s", sPubMPU_IO.ascii() );
							}
              else {
									if ( sPubModulename == "cs4232" )
											fprintf( fconf, " mpuio=%s", sPubMPU_IO.ascii() );
									else fprintf( fconf, " mpu_io=%s", sPubMPU_IO.ascii() );
							}	
					}//if ( ( !sPubMPU_IO.isEmpty() ) && ( iPubType != SGALAXY ) )
					
					if ( (!sPubMPU_IRQ.isEmpty()) && iPubType != SGALAXY ){
							if ( cardsArray[iPubType].flags & LOAD_CS4232 ){
									fprintf( fconf, " synthirq=%s", sPubMPU_IRQ.ascii() );
							}
              else {
									if ( sPubModulename == "cs4232" )
											fprintf( fconf, " mpuirq=%s", sPubMPU_IRQ.ascii() );
                  else fprintf( fconf, " mpu_irq=%s", sPubMPU_IRQ.ascii() );
							}		
					}// if ( (!sPubMPU_IRQ.isEmpty()) && iPubType != SGALAXY )

					if ( !sPubMEM.isEmpty() ) {
							if ( cardsArray[iPubType].type == OPL3_SAX ) {
									fprintf( fconf, " io=%s", sPubMEM.ascii() );
							}
							else if ( cardsArray[iPubType].type == SGALAXY ){
									fprintf( fconf, " sgbase=%s", sPubMEM.ascii() );
							}		
              else {
									fprintf( fconf, " mem=%s", sPubMEM.ascii() );
							}                                             							
					} //if ( !sPubMEM.isEmpty() )

				  if ( cardsArray[iPubType].flags & JOYSTICK )
							fprintf( fconf, " joystick=1");
					if ( cardsArray[iPubType].flags & SM_GAMES )
							fprintf( fconf, " sm_games=1" );
          if ( cardsArray[iPubType].flags & COMPAQ )
							fprintf( fconf, " deskpro_xl=1" );
					if ( cardsArray[iPubType].flags & ACER )
							fprintf( fconf, " acer=1");
					if ( cardsArray[iPubType].flags & SOUNDPRO )
							fprintf( fconf, " soundpro=1" );
					fprintf( fconf, "\n" );		
					
      } //else
	  fclose( fconf );	
	  chmod ( strMod, 0644 );
   }// if ( fopen	 )
	
	
}

/*************************************************************************/
void LSound :: WriteCardSettings( QString sCardType ){
  strMod = NEWROOT+CONF_FILE;
#ifdef GBLI_DEVEL
	strMod = "/home/nd/temp/soundcard";
#endif

	FILE *f_soundcard;

	if ( (f_soundcard = fopen( strMod, "w" )) != NULL	 ){
			fprintf( f_soundcard, "#THIS FILE IS WRITTEN BY BEST LINUX\n" );			
			fprintf( f_soundcard, "#PLEASE USE SNDCONFIG TO MODIFY\n" );
			fprintf( f_soundcard, "#TO CHANGE THIS FILE!\n" );
			fprintf( f_soundcard, "#There should be no spaces at the start of a line\n");
			fprintf( f_soundcard, "#or around the '=' sign\n");

			fprintf( f_soundcard, "CARDTYPE=%s\n", sCardType.ascii() );
			fclose( f_soundcard );
	}
	
}

/*************************************************************************/
void LSound::finish()
{
//  printf("sound finish() is called \n");
  system( ("rm -f "+ NEWROOT +"/tmp/*.mid* >/dev/null 2>&1").ascii() );
  system( ("rm -f "+ NEWROOT +"/tmp/test.wav >/dev/null 2>&1").ascii() );
  system( ("rm -f "+ NEWROOT +"/tmp/test.mp3 >/dev/null 2>&1").ascii() );

  if ( b_emu ){
		// hack ;(( Change later	
		 QFile modconf( NEWROOT + CONF_MODULES );
     if( modconf.open( IO_WriteOnly | IO_Append ) ){
     		QTextStream mc( &modconf );
				//modified sound to sound-slot-0
     		mc << "alias sound-slot-0 emu10k1" << endl;
//yp     		mc << "options sound-slot-0 dmabuf=1" << endl;
//yp     		mc << "pre-install sound-slot-0 /sbin/insmod sound dmabuf=1" << endl;
     		modconf.close();
		 }	
     WriteCardSettings( "emu10k1" );
  }else if ( iPubType != -1 ){
  	 		WriteConfModules(); // move to finish();
				WriteCardSettings( CardTypeToString( iPubType ) );	
	}else{
		if( !system( "grep cs46xx /proc/modules >/dev/null 2>&1" ) ){
  		 QFile modconf( NEWROOT + CONF_MODULES );
       if( modconf.open( IO_WriteOnly | IO_Append ) ){
       		QTextStream mc( &modconf );
       		mc << "alias sound-slot-0 cs46xx" << endl;
       		mc << "options sound-slot-0 dmabuf=1" << endl;
       		modconf.close();
  		 }
	     WriteCardSettings( "cs46xx" );
		}
	}


	//Testing
/*  printf("iPubType: %d \n", iPubType );
	printf("iPubCount: %d \n", iPubCount );
  printf("sPubType: %s \n", sPubType.ascii() );
	printf("sPubIRQ: %s \n", sPubIRQ.ascii() );
	printf("sPubIO: %s \n", sPubIO.ascii() );
	printf("sPubDMA: %s \n", sPubDMA.ascii() );
	printf("sPubModulename: %s \n", sPubModulename.ascii() );
	printf("sPubDescr: %s \n", sPubDescr.ascii() );
	printf("sPubDMA2: %s \n", sPubDMA2.ascii() );
	printf("sPubMPU_IO: %s \n", sPubMPU_IO.ascii() );
	printf("sPubMPU_IRQ: %s \n", sPubMPU_IRQ.ascii() );
  printf("sPubMEM: %s \n", sPubMEM.ascii() );*/
	//Testing
}

/*************************************************************************/
bool LSound::hasSound(){

#ifndef GBLI_DEVEL
  int fd;
  fd = ::open( "/dev/dsp", O_WRONLY );
  if( fd != -1 ) {
    ::close( fd );
    return TRUE;
  }
  if( errno == EBUSY ) // we are playing?
#endif
    return TRUE;
  return FALSE;
}

/*************************************************************************/

bool LSound::hasMidi()
{
  int fd;

  fd = ::open("/dev/sequencer",O_WRONLY);
  if (fd!=-1) {
    ::close(fd);
    return TRUE;
  }
  if (errno==EBUSY) // we are playing?
    return TRUE;
  return FALSE;
}

/*************************************************************************/
bool LSound::hasMixer()
{
  int fd;

  fd = ::open("/dev/mixer",O_WRONLY);
  if (fd!=-1) {
    ::close(fd);
    return TRUE;
  }
  if (errno==EBUSY) // we are mixing?
    return TRUE;
  return FALSE;
}
//*************************************************************************/
