/***************************************************************************
  																mouse.h
                             -------------------
    begin                : Fri Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MOUSE_H
#define MOUSE_H

#include <qwidget.h>
#include <qlabel.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include "config.h"

#include "gblipage.h"

#include "log.h"
class QLabel;
class QSocketNotifier;
class QListBox;

// that mouse figure, where you can press the buttons and see the reaction
class MouseView : public QWidget{
    Q_OBJECT
public:
    MouseView( QWidget *parent=0, const char *name=0);
    ~MouseView() { };
    void setWheel( bool );
    void resetState();

protected:
    void paintEvent( QPaintEvent * );
    bool eventFilter( QObject *, QEvent * );

    void enterEvent( QEvent * );
    void mouseMoveEvent( QMouseEvent * );
    void focusInEvent( QFocusEvent * ){};
    void focusOutEvent( QFocusEvent * ){};

private:
    void paintState( bool = FALSE);
    ButtonState state;
    int wheelOffset;
    bool hasWheel;
};

// data holder for the mouse interface (Mouse class)
// also setups X server
class MouseObserver : public QObject{
    Q_OBJECT
public:
    MouseObserver( QObject * parent, const char * name = 0 );
    ~MouseObserver();

    enum Type { PS2=0, SERIAL, USB };
    enum Port { PSAux=0, S0, S1, S2, S3, HIDBP };

    void setType( Type m );
		Type getType() { return mouse; };
    void setProtocol( int p ) { protocol = p; };
		int  getProtocol() { return protocol; };
    void setPort( Port p ) { port = (Port)p; };
		Port getPort() { return port; };
    void setEmulation( bool emulate ) {	emulate3rd = emulate; };
		const char * getPortName();
		Port getPortNum( char *a_szPortName );

public slots:
    void tellXServer();
		void runProbing();
		void checkValue(){
			allowCheck = true;
			tellXServer();
			allowCheck = false; };

signals:
    void gotMouse();

private slots:
		void setAllowCheck( bool state ) {
			allowCheck = state;
			tellXServer(); };
    void readMouseInfo(int);

private:
    Type mouse;
    Port port;
    int protocol;

    bool hasDetected;
    QString mdetect;
    bool emulate3rd;
    QSocketNotifier * rn;
    int pid;
		bool blProbe;
		bool allowCheck;
		bool blProbing;
};

// interface of the mouse page
// also contains translation tables to save mouse configs in
// correct format
class Mouse: public GBLIPage
{
    Q_OBJECT
public:
    Mouse( QWidget * parent, const char * name = 0 );
    ~Mouse();

		// mouse type for X server
    const char * getMouseName();

		// port name, e.g. /dev/ttyS0
    const char * getPortName() { return observer->getPortName(); };

		// mouse type for GPM
		const char * getGPMName();

		// PS2, SERIAL, USB
    MouseObserver::Type getMouseType() { return observer->getType(); }

    bool emulate3buttons() { return m_pchkEmulate->isChecked(); };
    bool hasWheel();
		void showEvent( QShowEvent *);
		void hideEvent( QHideEvent *);
		
		// autoprobe for mouse in the stage1
		static void initMouse();

		// flag that the mouse was found on USB port
		static bool runningUSB;
		void finish();

public slots:
		// set to X server current settings
    void tellXServer(){ observer->checkValue(); }

private slots:
		void setPort( int a_i );
    void gotMouse();
    void setType( int );
    void setProtocol( int i );
    void emulateClicked( bool );

protected:
	int getProtocol();
	int getBackProtocol( int i );
	MouseObserver::Port getPort();
	void setUpProtocolList();
  MouseObserver* observer;

	QPushButton *m_pbttnProbe;
	QLabel     *m_plblType;
	QListBox   *m_plboxType;
	QLabel     *m_plblProtocol;
	QListBox   *m_plboxProtocol;
	QLabel     *m_plblPort;
	QComboBox   *m_plboxPort;
	QCheckBox  *m_pchkEmulate;
//	QCheckBox  *m_pchkActivate;
	QPushButton  *m_pchkActivate;
	MouseView *m_testMouse;

	bool usbOHCI;

};


#endif


















