/***************************************************************************
                          			network.h
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef NETWORK_H
#define NETWORK_H

#define MAXPPPX 32
//that's should be enougth for ifcfg-pppX

#include "config.h"

#include <qwidget.h>
#include <qframe.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qdict.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qlist.h>
#include <qdialog.h>
#include <qtabwidget.h>

#include <netinet/in.h>

#include "configfile.h"
#include "sysconfigfile.h"

#define ADAPTERS_MAX 8

class QButtonGroup;
class QCheckBox;
class QRadioButton;
class LLabel;
class Network;
class QSocket;

// input field for entering network addresses, both IPv4 and v6
class AddressEdit: public QFrame
{
    Q_OBJECT
public:
		// mode affects address validation
    enum Mode { Address = 0, Netmask, Router, Other };
		// specifies the length and form of the field + validation
		enum Protocol { IPv4 = 0, IPv6, IPv46 };

    AddressEdit( Mode, QWidget * parent, Protocol a_pr = IPv4, const char * name = 0 );
    ~AddressEdit();

		// gets normal string representation of the address from the field
    QString text();
    struct in_addr address();

    bool eventFilter( QObject *, QEvent * );

    QSize sizeHint() const;
    QSizePolicy sizePolicy() const;

    bool edited() const;

    void setEnabled( bool );

public slots:
    void setAddress( const QString & );
    void setAddress( struct in_addr );
		// parameter is of type Protocol
    void setProtocol( int a_pr );

signals:
		// runs validation on leaving the field
    void returnPressed();

protected:
    void drawContents( QPainter * );

private:
#ifndef GBLI_IPv6
    QLineEdit *b1, *b2, *b3, *b4;
#else
    QLineEdit *ab[9];
		QLabel *labAB[8];
#endif
		// it is easier to access mode and protocol directly
    friend class Network;
    Mode m;
		Protocol pr;
    bool e;
};

class SitInfo {
public:
	SitInfo();
	~SitInfo();

	void setName( QString newName ) { Name = newName; }
	QString getName() { return Name; }
	void setLocalIP( QString srcIP ) { LocalIP = srcIP; }
	QString getLocalIP() { return LocalIP; }
	void setDestination( QString dstIP ) { DestIP = dstIP; }
	QString getDestination() { return DestIP; }
	void setOnBoot( bool flag ) { OnBoot = flag; }
	bool getOnBoot() { return OnBoot; }
	QStringList getRouteList() { return RouteNetworks; }
	void setRouteList( QStringList newList ) { RouteNetworks.clear(); RouteNetworks = newList; }

protected:
	QString	Name;
	QString	LocalIP, DestIP;
	QStringList RouteNetworks;
	bool	OnBoot;
};

// stores the data which is showen using different pages and tabs,
// e.g. NetworkPage, ISDN, ADSL, SIT, common addresses
// so, that GBLI needs only to call this class save functions at the end
// of installation
class Netinfo : public QObject {
    Q_OBJECT;
public:
    Netinfo();
    ~Netinfo();

		// how we got that data
    enum Source {
    	Default = 0,
    	Wildguess = 1,
    	Windows = 2,
    	Probed = 3,
    	User = 4
    };

		// what is the type of network configuration
    enum Setup {
    	NoConfig = 0, // no network
    	UseDHCPConfig, // auto
    	UseStaticConfig // static
    };

		// what is the progress of the probe?
    enum Probe {
    	Idle = 0,
    	Netmask,
    	GBLI,
    	MaxProbe
    };

		// what is the function of given interface
		enum Interface{
			NoNet = -1,
			Intranet = 0,
			Extranet,
			Bothnet
		};

		bool ISDNboot;
		static QString insertZeroSeq( QString a_str );
		// remove the end of IPv6 address
    static QString removePrefix( QString a_str );
		// replace zero inside the address, e.g. 12:0:0:0:13 -> 12::13
    static QString replaceZeroSeq( QString a_str );

protected:
		// storage unit, e.g. IP-address
    class Item {
    public:
    	Item(const char *, const char *);
    	~Item();

    	bool set(const char *, Source);

    	Source source;
    	QString name;
    	QString value;
    };

public:
    QString		prvHostName;  // for standalone
    // Set various items
    // (either from previous bootp/dhcp probe, or by user input).
#ifndef GBLI_IPv6
    bool			setMode(Setup);
    bool			setAddress(const char *, Source);
    bool			setNetmask(const char *, Source);
#else
		// we support several adapters, so adapter is a name, e.g. "eth0"
    bool			setMode( QString adapter, Setup );
    bool			setAddress( QString adapter,const char *, Source);
    bool			setNetmask( QString adapter,const char *, Source);
    bool			setAddress6( QString adapter,const char *, Source);
    bool			setNs1Protocol( AddressEdit::Protocol a_pr ){
			Ns1Protocol = a_pr;
			return true;
		}
    bool			setNs2Protocol( AddressEdit::Protocol a_pr ){
			Ns2Protocol = a_pr;
			return true;
		}
    bool			setRouterProtocol( AddressEdit::Protocol a_pr ){
			RouterProtocol = a_pr;
			return true;
		}
		bool			setInterface( QString adapter, Netinfo::Interface a_int );
#endif

    bool			setProtocol( QString adapter, AddressEdit::Protocol );

    bool			setGateway(const char *, Source);
    bool			setGatewayDevice(const char *, Source);
    bool			setNameserver(const char *, Source);
    bool			setNameserver2(const char *, Source);
    bool			setHostname(const char *, Source);
    bool			setNISDomain(const char *, Source);

    bool			setProxyName(const char *, Source);
    bool			setProxyPort(const char *, Source);
    bool			setNoProxyFor(const char *, Source);

    bool			setADSLMode(Setup);
		bool			setADSLAddress(const char *, Source);
		bool			setADSLNetmask(const char *, Source);
		bool			setADSLCard(const char *, Source);
		bool			setADSLUser(const char *, Source);
		bool			setADSLPass(const char *, Source);

		bool			setISDNMode(Setup);
		bool			setISDNPass(const char *, Source);
		bool			setISDNUser(const char *, Source);
		bool			setISDNInPhone(const char *, Source);
		bool			setISDNOutPhone(const char *, Source);
		bool			setISDNType(const char *, Source);

    // Read configuration (for update and standalone)
		bool			readEther(const char *root);
    bool			readADSL(const char *root);
		bool			readISDN(const char *root);
    bool			readSysNetwork(const char *root);
		bool			readTunnels(const char *root);

    // Get these items
#ifndef GBLI_IPv6
    inline const char *		getAddress()     { return address.value; }
    inline const char *		getNetmask()     { return netmask.value; }
		inline const Netinfo::Setup getMode()	{	return mode; }
#else
		inline const Netinfo::Setup getMode( QString adapter ){
			Netinfo::Setup *item = mode.find( adapter );
			if( item ) return (Netinfo::Setup)*item;
			else return (Netinfo::Setup)-1;
		}
    inline const AddressEdit::Protocol getProtocol( QString adapter ){
			AddressEdit::Protocol *item = protocol.find( adapter );
			if( item ) return *item;
			else return AddressEdit::IPv4;
		}

    inline const char *		getAddress( QString adapter ){
//printf( "getAddress %s\n", adapter.ascii() );
			Item *item = address.find( adapter );
			if( item ){
//printf( "Name: %s, value: %s\n", item->name.ascii(), item->value.ascii() );
				return item->value;
			}else return "";
		}
    inline const char *		getNetmask( QString adapter ){
			Item *item = netmask.find( adapter );
			if( item ) return item->value;
			else return "";
		}
    inline const char *		getAddress6( QString adapter ){
			Item *item = address6.find( adapter );
			if( item ){
//printf( "Name: %s, value: %s\n", item->name.ascii(), item->value.ascii() );
				return item->value;
			}else return "";
		}
    inline const Netinfo::Interface	getInterface( QString adapter ){
			Netinfo::Interface *item = interface.find( adapter );
			if( item ) return *item;
			else return Netinfo::NoNet;
		}
    inline const AddressEdit::Protocol getNs1Protocol() { return Ns1Protocol; }
    inline const AddressEdit::Protocol getNs2Protocol() { return Ns2Protocol; }
    inline const AddressEdit::Protocol getRouterProtocol() { return RouterProtocol; }
#endif
    inline const char *		getGateway()     { return gateway.value; }
    inline const char *		getGatewayDevice()     { return gatewayDevice.value; }
    inline const char *		getNameserver()  { return nameserver.value; }
    inline const char *		getNameserver2() { return nameserver2.value; }
    inline const char *		getHostname()    { return hostname.value; }
    inline const char *		getNISDomain()   { return nisdomain.value; }

    inline const char *		getProxyName()   { return proxy.value; }
    inline const char *		getProxyPort()   { return proxyport.value; }
    inline const char *		getNoProxy()     { return noproxyfor.value; }

		inline const char *		getADSLAddress() { return adsladdress.value; }
    inline const char *		getADSLNetmask() { return adslnetmask.value; }
    inline const char *		getADSLCard()    { return adslcard.value; }
    inline const char *		getADSLUser()    { return adsluser.value; }
    inline const char *		getADSLPass()		 { return adslpass.value; }
    inline const Setup		getADSLMode()    { return ADSLmode; }

		inline const char * 	getISDNUser()    { return isdnuser.value; }
		inline const char * 	getISDNPass()    { return isdnpass.value; }
		inline const char * 	getISDNOutPhone(){ return isdnoutphone.value; }
		inline const char * 	getISDNInPhone() { return isdninphone.value; }
		inline const char * 	getISDNType()    { return isdntype.value; }
    inline const Setup		getISDNMode()    { return ISDNmode; }
	  inline const bool 		getISDNboot() 	 { return ISDNboot; } //Added by dk

		// Tunnel sit devices functions
		SitInfo*	getSitInfo( uint i );
		uint addSitInfo( SitInfo* newsit );
		void removeSitInfo( uint i );
		uint countSitInfo();

    // Start the network probe
    void probe(bool);

    // Save configuration, calls all other save functions
    void save(const char *root, QString userName);

    bool hadDHCP() { return dhcp; };

  /**  */
  void setISDNboot(bool isdnBoot);
  /**  */
    bool saveHosts(const char *root);

signals:
    void			newConfig();

private slots:
    void			inputNetprobe();

protected:
    void			inputBOOTP();
    void			startProbe(Probe, bool);
    void			stopProbe();

    bool			saveEther(const char *);
    bool			saveADSL(const char *);
    bool			saveADSLppp(const char *, QString);
		bool			saveISDN(const char *, QString);
    bool			saveResolv(const char *);
#ifdef NOT_USED
    bool			saveNIS(const char *);
#endif
    bool			saveSysNetwork(const char *);
    bool			saveProxy(const char *);
#ifdef NO_USED
    bool			saveTmpNetwork();
#endif
    bool			saveXDMCP(const char *);
		bool			saveTunnels(const char *);

protected:
    const char *netprobePath;

    Probe			currentProbe;
    QSocket *	probeSock;
    int				probePid;

    bool      dhcp;

#ifndef GBLI_IPv6
    Setup			mode;
    Item			address;
    Item			netmask;
#else
		// different modes for different adapters, so the key is adapter name
    QDict<Setup> mode;
    QDict<AddressEdit::Protocol> protocol;
		QDict<Item> address;
		QDict<Item> netmask;
		QDict<Item> address6;
		QDict<Netinfo::Interface> interface;

		AddressEdit::Protocol Ns1Protocol;
		AddressEdit::Protocol Ns2Protocol;
		AddressEdit::Protocol RouterProtocol;
#endif
    Item			gateway;
		Item			gatewayDevice;
    Item			nameserver;
    Item			nameserver2;
    Item			hostname;
    Item			nisdomain;

    Item			proxy;
    Item			proxyport;
    Item			noproxyfor;

    Setup			ADSLmode;
		Item			adsladdress;
    Item			adslnetmask;
		Item			adslcard;
		Item			adsluser;
		Item			adslpass;
		QString		oldadsluser; //initialized by readADSL
		QString		adslpppfile; //... used to save read ifcfg-pppx file (name)

		Setup			ISDNmode;
		Item			isdnuser;
		Item			isdnpass;
		Item			isdnoutphone;
		Item			isdninphone;
		Item			isdntype;
		QString		oldisdnuser;

		QList<SitInfo>	SITdevices;
};

// a tab with network address and netmask, also interface and protocol
class Network: public QWidget
{
    Q_OBJECT
public:
    Network( Netinfo *, QWidget * parent, QString a_adapter, const char * name = 0 );
    ~Network();

    QString ip() { return ipEdit->text(); }
    QString netmask() { return netmaskEdit->text(); }

		QString getAdapter(){ return adapter; };
		void setAdapter(QString a_adapter ){ adapter = a_adapter; };
//    Netinfo::Setup mode() const;

		QString module;

public slots:
    void setMode( int );
    void updateInfo();
    void updateWidgets();
		void changeProtocol( int a_pr );
		void changeInterface( int a_int );

private slots:
    void		addInputWidget(QWidget *);
    bool		eventFilter(QObject *, QEvent *);

private:
		QLabel *l1, *l2;
    AddressEdit * ipEdit;
    AddressEdit * netmaskEdit;

		QString adapter;

#ifdef GBLI_IPv6
		QLabel *l3, *l4;
    AddressEdit * ip6Edit;
    AddressEdit * netmask6Edit;
		QComboBox 	* ipCombo;
		QComboBox 	* intCombo;
#endif

public:
		QGridLayout * grid;

private:
    QButtonGroup * bg;
    Netinfo * info;
};

// a tab with namservers and gateways
class NetworkCommon: public QWidget
{
    Q_OBJECT
public:
    NetworkCommon( Netinfo *, QWidget * parent, const char * name = 0 );
    ~NetworkCommon();

    QString nameserver1() { return ns1Edit->text(); }
    QString nameserver2() { return ns2Edit->text(); }
    QString gateway() { return routerEdit->text(); }
//		QString gatewayDevice() { return devCombo->currentText(); }
//    QString nisDomain() { return nisEdit->text(); }

public slots:
    void updateInfo();
    void updateWidgets();

private slots:
    void		addInputWidget(QWidget *);
    bool		eventFilter(QObject *, QEvent *);

		void setNs1Protocol( bool );
		void setNs2Protocol( bool );
		void setRouterProtocol( bool );

private:
    AddressEdit * ns1Edit;
    AddressEdit * ns2Edit;
    AddressEdit * routerEdit;
		QComboBox *	devCombo;
//    QLineEdit * nisEdit;

		QCheckBox * ns1IPv6;
		QCheckBox * ns2IPv6;
		QCheckBox * routerIPv6;

private:

    Netinfo * info;
};

// a tab with SIT entering
class NetworkTunnel: public QWidget
{
    Q_OBJECT
public:
    NetworkTunnel( Netinfo *, QWidget * parent, const char * name = 0 );
    ~NetworkTunnel();

    void updateInfo();
    void updateWidgets();

private slots:
		void AddSitDevice();
		void RemoveSitDevice();
		void ConfigureSitDevice();
		void CurrentDeviceChanged( QListViewItem* );

//    void		addInputWidget(QWidget *);
//    bool		eventFilter(QObject *, QEvent *);

private:
		QListView	*sitListView;
		int	tunnelSelected;
		QPushButton *ConfButton;

    Netinfo *info;
};

// a dialog for configuring SIT devices
class TunnelCfgDialog : public QDialog
{
    Q_OBJECT
public:
    TunnelCfgDialog( QWidget * parent, SitInfo * sitdev, const char * name = 0 );
    ~TunnelCfgDialog()
		{
			if (CfgPage) delete CfgPage;
			if (RoutesPage) delete RoutesPage;
		}

		void updateWidgets();
		void updateInfo();

		QTabWidget	*TabForm;

		QWidget			*CfgPage,
								*RoutesPage;

		QCheckBox*	OnBoot;
		AddressEdit*	EndPointIP;
		AddressEdit*	LocalIP;
		AddressEdit*	RouteIP;
		QListBox*		RouteList;
		QLineEdit*	NameEdit;
		SitInfo*		si;

public slots:
		void AddRouteNetwork();
		void RemoveRouteNetwork();
};

#endif



