/***************************************************************************
                          networkpage.cpp  -  description                              
                             -------------------                                         
    begin                : Fri Dec 8 2000                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		 : Oleg Golovachov (Demiurg)
    author email         : og@bestlinux.net
    url			 : http://www.bestlinux.net                          
    email		 : linux@bestlinux.net                              

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "networkpage.h"
#include "liconloader.h"
#include "log.h"
#include "addmodulespage.h"

#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

#define NET_DEBUG
#ifdef NET_DEBUG
//	# define D(args...)	printf(##args)
	# define D(args...)	Log::logMain( 33, ##args )
#else
	# define D(args...)	do { } while (0)
#endif

NetworkPage::NetworkPage(QWidget *parent, const char *name )
		: GBLIPage(parent,name) {

		pageSeparator = new QTabWidget( this );
//		pageSeparator->setAutoMask( true );
		pageSeparator->setBackgroundPixmap( LIconLoader::loadIcon( TEXTURE ) );

		netdata = new Netinfo;

#ifdef GBLI_IPv6

		int count = 0;
		Network *tmpNetwork = NULL;

		QFile fin( "/etc/sysconfig/hwconf" );
		if( fin.open( IO_ReadOnly ) ){
	  	QTextStream tin( &fin );
			QString temp;
			bool in_sect = false;
			while( (temp = tin.readLine()).length() ){
				if( temp == "-" ){
					in_sect = false;
					tmpNetwork = NULL;
					continue;
				}
				
				if( temp == "class: NETWORK" ){
					in_sect = true;
					QString name = "eth" + QString::number( count++ );
					tmpNetwork = new Network( netdata, pageSeparator, name, "eth" );
//					printf("-->Previous mode: %s %d\n", name.ascii(), netdata->getMode( name ));
					netdata->setMode( name, Netinfo::NoConfig );
					network.insert( name, tmpNetwork );
					pageSeparator -> addTab( tmpNetwork, count==1?(tr("Network card")+" ("+ name +")"):name );
				}else if( in_sect && temp.left( 7 ) == "driver:" ){
					QString module_name = temp.right( temp.length() - 8 );
					if( module_name.length() ){
						QString mprobe = "modprobe -qs "+ module_name +" >/dev/null 2>&1";
						D("%s\n", mprobe.ascii() );
						if( tmpNetwork ) tmpNetwork->module = module_name;
#ifndef GBLI_DEVEL
						system( mprobe.ascii() );
#endif
					}
				}
			}
			fin.close();
		}

		if( !count ){ // kudzu failed
    	QFile inf( "/proc/modules" );
    	if( inf.open( IO_ReadOnly ) ){
  			QDict<int> DMod;
    		int *pi;
    		int i = 0;
    		while( drivers[i].name ){
//printf( "drivers\n" );
					if( drivers[i].type != DRIVER_NET && drivers[i].type != DRIVER_PCMCIA ){
						i++;
						continue;
					}
    			pi = (int *)new int[1];
    			*pi = i;
    			QString str( drivers[i].modules );
    			DMod.insert( str, pi );
    			i++;
    		}

      	QTextStream ints( &inf );
    		QString temp;
    		while( (temp = ints.readLine()).length() ){
//printf( "modules\n" );
    			temp = temp.left( temp.find( " " ) );
    			int *ii = DMod.find( temp );
    //printf( "Modules: \"%s\" - %d\n", temp.ascii(), ii );
					if( ii && temp != "pcmcia_core" && temp != "ds" && temp != "i82365" && temp != "tcic" && temp != "yenta_socket" ){
    //printf( "adding...\n" );
  					QString name = "eth" + QString::number( count++ );
						netdata->setMode( name, Netinfo::NoConfig );
  					tmpNetwork = new Network( netdata, pageSeparator, name, "eth" );
  					network.insert( name, tmpNetwork );
						pageSeparator -> addTab( tmpNetwork, count==1?(tr("Network card")+" ("+ name +")"):name );
    			}
    		}
    		inf.close();
    	}
		}

  	if( !count ){ //
			D("lets add just something\n");
  		QString name = "eth" + QString::number( count++ );
  		tmpNetwork = new Network( netdata, pageSeparator, name, "eth" );
  		network.insert( name, tmpNetwork );
			pageSeparator -> addTab( tmpNetwork, count==1?(tr("Network card")+" ("+ name +")"):name );
			tmpNetwork->setMode( Netinfo::NoConfig );
  	}

//#ifdef GBLI_DEVEL
//    Network *network0 = new Network( netdata, pageSeparator, "eth4", "eth" );
//    pageSeparator -> addTab( network0, "eth4" );
//    network.insert( "eth4", network0 );
//		network0->module = "test_module";
//#endif

		ISDN = new ISDNPage ( netdata, pageSeparator );
		pageSeparator -> addTab( ISDN, "ISDN");

		ADSL = new ADSLPage ( netdata, pageSeparator );
		pageSeparator -> addTab( ADSL, "ADSL");

		ncommon = new NetworkCommon( netdata, pageSeparator, "NetworkCommon" );
    pageSeparator->addTab( ncommon, tr("Common") );

		ntunnel = new NetworkTunnel( netdata, pageSeparator, "NetworkTunnel" );
		pageSeparator->addTab( ntunnel, tr("IPv6-in-IPv4") );

//printf( "after common\n" );

#else
		interfaceName = "eth0";
    network = new Network( netdata, pageSeparator, interfaceName );
		pageSeparator -> addTab( network, "Ethernet" );

		ISDN = new ISDNPage ( netdata, pageSeparator );
		pageSeparator -> addTab( ISDN, "ISDN");

		ADSL = new ADSLPage ( netdata, pageSeparator );
		pageSeparator -> addTab( ADSL, "ADSL");
#endif

		QVBoxLayout *V1 = new QVBoxLayout();
		V1->addStretch(1);
		V1->addWidget(pageSeparator,0);		
		V1->addStretch(1);

		QHBoxLayout *ML = new QHBoxLayout(this);
		ML->addStretch(1);
		ML->addLayout(V1,5);
		ML->addStretch(1);

////////
		testButton = new QPushButton("Save data", this);
    connect( testButton, SIGNAL(clicked()),
	     this, SLOT(SaveDATA()) );
////////

#ifndef GBLI_IPv6
		ns1Edit = new AddressEdit( AddressEdit::Other, this );
    ns2Edit = new AddressEdit( AddressEdit::Other, this );
    routerEdit = new AddressEdit( AddressEdit::Router, this );
    ns3Edit = new AddressEdit( AddressEdit::Other, this );
ns3Edit->hide();
    nisEdit = new QLineEdit( this );
#endif

    proxyEdit = new QLineEdit( this );

    proxyPortEdit = new QLineEdit( this );
    proxyPortEdit->setValidator( new QIntValidator( 0, 32768, proxyPortEdit ) );
    proxyPortEdit->setAlignment( AlignRight );
    proxyPortEdit->setMaxLength( 5 );
    proxyPortEdit->setFixedWidth( fontMetrics().width( "12345" ) + 15 );
    proxyPortEdit->setText( "3128" );

    noproxyforEdit = new QLineEdit( this );

    nameEdit = new QLineEdit( this );

		QGridLayout *grid = new QGridLayout(6,5, 7 /*10*/ );
//		QLabel *l=new QLabel(this);

#ifndef GBLI_IPv6
		QLabel *l1 = new QLabel(tr("Name server:"),this);
    l1->setAutoMask(true);
		grid->addWidget(l1,0,0);
		grid->addWidget(ns1Edit,0,1,AlignLeft);
		QLabel *l2 = new QLabel(tr("Name server2:"),this);
    l2->setAutoMask(true);
		grid->addWidget(l2,0,3);
		grid->addWidget(ns2Edit,0,4,AlignLeft);
		QLabel *l3 = new QLabel(tr("Gateway:"),this);
    l3->setAutoMask(true);
		grid->addWidget(l3,1,0);
		grid->addWidget(routerEdit,1,1,AlignLeft);
		QLabel *l4 = new QLabel(tr("NIS domain:"),this);
    l4->setAutoMask(true);
		grid->addWidget(l4,1,3);
		grid->addWidget(nisEdit,1,4,AlignLeft);
		QLabel *l5 = new QLabel(tr("Proxy server:"),this);
    l5->setAutoMask(true);
		grid->addWidget(l5,2,0,AlignLeft);
		grid->addMultiCellWidget(proxyEdit, 2,2,1,3);
		grid->addWidget(proxyPortEdit,2,4,AlignLeft);
		QLabel *l6 = new QLabel(tr("No proxy for:"),this);
    l6->setAutoMask(true);
		grid->addWidget(l6,3,0);
		grid->addMultiCellWidget(noproxyforEdit, 3,3,1,4);
#else
//		QLabel *l1 = new QLabel(tr("Name server:"),this);
//    l1->setAutoMask(true);
//		grid->addWidget(l1,0,0);
//		grid->addWidget(ns1Edit,0,1,AlignLeft);
//		QLabel *l2 = new QLabel(tr("Name server2:"),this);
//    l2->setAutoMask(true);
//		grid->addWidget(l2,0,3);
//		grid->addWidget(ns2Edit,0,4,AlignLeft);
//		QLabel *l3 = new QLabel(tr("Gateway:"),this);
//    l3->setAutoMask(true);
//		grid->addWidget(l3,1,0);
//		grid->addWidget(routerEdit,1,1,AlignLeft);
//		QLabel *l4 = new QLabel(tr("NIS domain:"),this);
//    l4->setAutoMask(true);
//		grid->addWidget(l4,1,3);
//		grid->addWidget(nisEdit,1,4,AlignLeft);
		proxyEdit->hide();
		proxyPortEdit->hide();
		noproxyforEdit->hide();
#endif

		QFrame * hack = new QFrame( this, 0, 0, TRUE );
    hack->setFrameStyle( QFrame::HLine + QFrame::Sunken );
    grid->addMultiCellWidget( hack, 4, 4, 0, 4 );
		QLabel *lab = new QLabel(tr("Hostname:"),this);
		lab->setAutoMask( true );
		grid->addWidget(lab,5,0);
		grid->addMultiCellWidget(nameEdit,5,5,1,4);
		
//		grid->addWidget)
		V1->addLayout(grid, 1 );

		V1->addStretch(1);
//		V1->addWidget(testButton);
testButton->hide();

#ifndef GBLI_IPv6
    addInputWidget( ns1Edit );
    addInputWidget( ns2Edit );
		addInputWidget( routerEdit );
//    addInputWidget( ns3Edit );
    addInputWidget( nisEdit );
#endif
		addInputWidget( proxyEdit );
		addInputWidget( proxyPortEdit );
    addInputWidget( nameEdit );

//		connect( this, SIGNAL(hide()), GBLIPage, SLOT(updateInfo()) );
//		this
		
#ifdef GBLI_STANDALONE
		readConfig(NEWROOT);
		updateWidgets();
#endif

//printf( "at the end\n" );
}

void NetworkPage::hideEvent( QHideEvent * )
{
#ifndef GBLI_DEVEL
	updateInfo();
#ifndef GBLI_STANDALONE
	saveHosts( "/" );
#endif
#endif
}

void NetworkPage::addInputWidget(QWidget *edit){
    connect( edit, SIGNAL(returnPressed()), this, SLOT(updateInfo()) );
    edit->installEventFilter(this);
}

void NetworkPage::SaveDATA() {
//enabled only for TEST
//#ifdef GBLI_DEVEL
//	QMessageBox::information( this, "GBLI internal", "void NetworkPage::SaveDATA()" );
//	save("/home/nettest","turbo_user");
//#endif
}

void NetworkPage::updateInfo(){
//    bool	changed = false;

    D("NetworkPage::updateInfo()\n");
//    DUMP_ALL_WIDGETS;

//    if (nameEdit->edited())
//			printf("Wid:%s\tNetData:%s\n", (const char*)nameEdit->text(), (const char*)netdata->getHostname());

//sleep( 10 );
//printf( "check1\n" );

	    netdata->setHostname(nameEdit->text(), Netinfo::User);

//sleep( 10 );
//printf( "check2\n" );

#ifndef GBLI_IPv6
//    if (nisEdit->edited())
	    netdata->setNISDomain(nisEdit->text(), Netinfo::User);
//    if (routerEdit->edited())
	    netdata->setGateway(routerEdit->text(), Netinfo::User);
//    if (ns1Edit->edited())
	    netdata->setNameserver(ns1Edit->text(), Netinfo::User);
//    if (ns2Edit->edited())
	    netdata->setNameserver2(ns2Edit->text(), Netinfo::User);
#else
			ncommon->updateInfo();
#endif

//sleep( 10 );
//printf( "check3\n" );

//		if ( proxyEdit->edited() )
	    netdata->setProxyName(proxyEdit->text(), Netinfo::User);
//		if ( proxyPortEdit->edited() )
	    netdata->setProxyPort(proxyPortEdit->text(), Netinfo::User);

//sleep( 10 );
//printf( "check4\n" );

#ifndef GBLI_IPv6
		network->updateInfo();
#else
	  QDictIterator<Network> it(network);
  	while( it.current() ){
			it.current()->updateInfo();
	    ++it;
		}

#endif

//sleep( 10 );
//printf( "check5\n" );

		ADSL->updateInfo();

//sleep( 10 );
//printf( "check6\n" );

		ISDN->updateInfo();

//sleep( 10 );
//printf( "check7\n" );

//		if(changed) netdata->probe(true);
}

void NetworkPage::saveHosts(const char *root) {
		netdata->saveHosts(root);
}

void NetworkPage::show() {
		updateWidgets();
		QWidget::show();
}

void NetworkPage::finish() {
}

void NetworkPage::save(const char *root,QString userName) {
QString qsSysCom;

		netdata->save(root, userName);

#ifdef GBLI_IPv6
		ConfigFile config( root, CONF_MODULES );

 	  QDictIterator<Network> it(network);
   	while( it.current() )
		{
			QString str = "alias "+ it.current()->getAdapter();
			if( it.current()->module.length() && it.current()->getAdapter().length() )
			{
#ifdef GBLI_DEVEL
				printf("Writing alias for %s adapter\n", it.current()->getAdapter().ascii());
#endif
//hha :) We should delete previous info,then we should check only for ethX
//not for alias ethX
						config.replaceLineWithRet( it.current()->getAdapter(), str+" " + it.current()->module, true );
//					qsSysCom.sprintf("sed -e \"/%s/d\" %s/%s >%s/%s.new 2>/dev/null",it.current()->getAdapter().latin1(),root,CONF_MODULES,root,CONF_MODULES);
//					system(qsSysCom.latin1());
//					qsSysCom.sprintf("mv -f %s/%s.new %s/%s >/dev/null 2>/dev/null",root,CONF_MODULES,root,CONF_MODULES);
//					system(qsSysCom.latin1());				
//					config.replaceLine( str, str+" " + it.current()->module, true );
			}
			++it;
		}
#endif
}

void NetworkPage::readConfig(const char *root)
{
	D("Reading network config files\n");
	if (!netdata->readEther(root)) Log::logMain(32, "Error reading Ethernet config\n");
	if (!netdata->readADSL(root)) Log::logMain(32, "Error reading ADSL config\n");
	if (!netdata->readISDN(root)) Log::logMain(32, "Error reading ISDN config\n");
	if (!netdata->readSysNetwork(root)) Log::logMain(32, "Error reading SysNetwork config\n");
	if (!netdata->readTunnels(root)) Log::logMain(32, "Error reading IPV6 tunnels config\n");
}

void NetworkPage::updateWidgets() {
//printf( "network page::updateWidgets\n" );

//printf( "call net::update\n" );
#ifndef GBLI_IPv6
		network->updateWidgets();
#else
	  QDictIterator<Network> it(network);
  	while( it.current() ){
			it.current()->updateWidgets();
/// Added by Serg_
			it.current()->changeProtocol( netdata->getProtocol(it.current()->getAdapter()) );
			it.current()->setMode( netdata->getMode(it.current()->getAdapter()) );

	    ++it;
		}
//		for( int i = 0; i < 8; i++ ){		
//			QString name = "eth" + QString::number( i );
//			Network *pnet = network.find( name );
//			if( pnet )
//				pnet->updateWidgets();
//		}
#endif
//printf( "finished net::update\n" );

		ADSL->updateWidgets();
		
		ntunnel->updateWidgets();

		ISDN->updateWidgets();

#ifndef GBLI_IPv6
    ns1Edit->setAddress(netdata->getNameserver());
    ns2Edit->setAddress(netdata->getNameserver2());
		routerEdit->setAddress(netdata->getGateway());
		nisEdit->setText(netdata->getNISDomain());
#else
		ncommon->updateWidgets();
#endif

		proxyEdit->setText(netdata->getProxyName());
		proxyPortEdit->setText(netdata->getProxyPort());
		noproxyforEdit->setText(netdata->getNoProxy());
		nameEdit->setText(netdata->getHostname());

		netdata->probe(true);
}


NetworkPage::~NetworkPage(){
}

bool NetworkPage::runIfconfig()
{
	int pid, ret = 1;

	QString name, addr;

	updateInfo();
#ifdef GBLI_IPv6
	QDictIterator<Network> it(network);
	while(it.current())
	{
		name = it.current()->getAdapter();
		addr = it.current()->ip();

		if(!(pid = fork()))
		{
			printf("%s\n", ("ifconfig " + name + " " + addr).latin1());
#ifndef GBLI_DEVEL
			if(::system("mount | grep 192.168.1.1 &>/dev/null"))
				ret = ::execlp( "/sbin/ifconfig", "ifconfig", name.latin1(), addr.latin1(), 0);
#endif
			::exit(0);
		}
		::waitpid(pid, NULL, 0);

		if(ret == -1)
			return false;

		++it;
	}
#else
		name = interfaceName;
		addr = netdata->getAddress();

		if(!(pid = fork()))
		{
			printf("%s\n", ("ifconfig " + name + " " + addr).latin1());
#ifndef GBLI_DEVEL
			if(::system("mount | grep 192.168.1.1 &>/dev/null"))
				ret = ::execlp( "/sbin/ifconfig", "ifconfig", name.latin1(), addr.latin1(), 0);
#endif
			::exit(0);
		}
		::waitpid(pid, NULL, 0);

		if(ret == -1)
			return false;
#endif

	return true;
}
