/***************************************************************************
                               partitionsel.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


//#include "partitionsel.h"
#include "partitionview.h"
#include "partitionitem.h"
#include "fdisk.h"
#include "linlinehelp.h"
#include "liconloader.h"
#include "mountpoint.h"
#include "language.h"
#include "installselect.h"
#include "partitioninfo.h"
#include "log.h"

#include <sys/mount.h>
#include <mntent.h>
#include <string.h>

#include <qapplication.h>
#include <qfile.h>

PartitionSelectorItem::PartitionSelectorItem( PartitionSelector* sel, QListView * parent,
			QString s1,     QString s2,
			QString s3)
    : QListViewItem( parent, s1, s2, s3)
{
    isNone = s1.isEmpty();
    selector = sel;
    setPixmap(0, LIconLoader::loadIcon("hds_part.gif"));
}

QString PartitionSelectorItem::text( int c ) const
{
    if ( c == 1 && !isNone ) { // the used_for (Used as) text
			QString used_for = MountPoints::mountPoint( text(0) );

		PartitionSelector::Mode mode = selector->partitionMode();
	
		if ( isSelected() ) {
	    if ( mode == PartitionSelector::RootMode || mode == PartitionSelector::OnlyRootMode ) {
		used_for = PartitionSelector::tr("Root Partition");
	    }
	    else {
		used_for = PartitionSelector::tr("User Partition");
	    }
	    return used_for;
		}
	
	if ( used_for == "/usr") {
	    if ( mode == PartitionSelector::RootMode  || mode == PartitionSelector::OnlyRootMode )
		used_for = PartitionSelector::tr("User Partition");
	    else
		used_for = "";
	}
	
	if ( used_for == "/" ) {
	    if ( mode != PartitionSelector::RootMode  || mode == PartitionSelector::OnlyRootMode )
		used_for = PartitionSelector::tr("Root Partition");
	    else
		used_for = "";
	}
	
	if ( used_for == "")
	    used_for = PartitionSelector::tr("Nothing yet");
	
	return used_for;
    } else if ( c == 0 && isNone)
	return PartitionSelector::tr("None");

    if (c > 1 )
	c--;
	
    return QListViewItem::text(c );
}


void PartitionSelectorItem::paintCell(QPainter * p, const QColorGroup & cg, int column, int width, int align)
{
  QColorGroup mygroup(cg);

  if (!isSelectable())
    mygroup.setColor(QColorGroup::Text, Qt::lightGray);

  QListViewItem::paintCell(p, mygroup, column, width, align);
}

PartitionSelector::PartitionSelector( PartitionView* pview_, Mode m,
					QWidget *parent, const char *name, Target *target_, UpgradePage *update_)
    : GBLIPage( parent, name)
{

		shown=FALSE;
    initializing = FALSE;
    pview = pview_;
    mode = m;
		target = target_;
		update = update_;
		
	  QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		lh->addStretch( 1 );

    view = new QListView( this );
		QFont font( QFont::defaultFont() );
		font.setPointSize( Language::iSmallFontSize );
    view->setFont( font );
    view->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    connect( view, SIGNAL( selectionChanged(QListViewItem*) ),
	     this, SLOT( selectionChanged(QListViewItem*) ) );
    view->setColumnWidthMode( view->addColumn(tr("Device")),
			      QListView::Maximum );
    int usedForCol = view->addColumn(tr("Used as"));
    view->setColumnAlignment( usedForCol, AlignCenter );
    view->setColumnAlignment( usedForCol, QListView::Maximum );
    int sizeCol = view->addColumn(tr("Size"));
    view->setColumnAlignment( sizeCol, AlignRight );
    view->setColumnWidthMode( sizeCol, QListView::Maximum );
		l->addStretch( 1 );
    l->addWidget( view, 10 );
		l->addStretch( 1 );

    view->setFocus();
}

PartitionSelector::~PartitionSelector()
{
}


void PartitionSelector::selectionChanged( QListViewItem *l )
{
    selectedItem = l;
    repaint();
}


void PartitionSelector::showEvent( QShowEvent * )
{
    init();
}

void PartitionSelector::hideEvent( QHideEvent * )
{
    if (!selectedItem)
			return;
    QString partition = selectedItem->text(0);
    if ( mode != RootMode && mode != OnlyRootMode && selectedItem->text(2).isEmpty() ) {
			MountPoints::clearMountPoint( MountPoints::mountPartition("/usr") );
			return;
    }
    if ( mode == RootMode || mode == OnlyRootMode ){
	 		MountPoints::setMountPoint( partition, "/" );
//Log::logMain(32, "hideEvent1: %d\n", pview->idOfPartition( partition ) );
			PartitionItem* pi = pview->partition( partition );
			if( pi )
				pi->bootable = true;
    }else
 			MountPoints::setMountPoint( partition, "/usr" );
    pview->invalidateView();
//Log::logMain(32, "hideEvent2: %d\n", pview->idOfPartition( partition ) );
#ifdef GBLI_SERVER
 if (!shown)
 {
    shown=TRUE;
  	if ( QMessageBox::warning( this, tr( "WARNING" ),
				   tr( "During the installation process you will be required \n to enter your SOT Linux installation key \n \nYou can obtain or check your key by registering \n your software at http://register.sotlinux.com/ \n \n  Do not proceed without a current installation key." ),
				   tr( "&Proceed." ),
				   tr( "  &Cancel" ).simplifyWhiteSpace(),
				   QString::null,
				 1, 1 ) == 1 )
  {
	 qApp->mainWidget()->close( FALSE );
  }
 }
#endif

}


void PartitionSelector::init()
{
    if  (initializing)
			return; // avoid recursion
    selectedItem = 0;
    initializing = TRUE;
    view->clear();
		if( InstallSelect::blUpdate )
		{
			MountPoints::clear();
  		PartitionInfo::unMounting();
//			update->umountFS();
		}
    if( mode == RootMode || mode == OnlyRootMode )
			initRoot();
    else
			initUser();
    initializing = FALSE;
}


PartitionSelector::Mode PartitionSelector::partitionMode() const
{
    return mode;
}

void PartitionSelector::initRoot()
{
//Log::logMain(32, "init Root\n" );

    bool didSelect = FALSE;
    double selectedSize = 0;
    bool selectedIsNew = FALSE;

	  qApp->setOverrideCursor( Qt::waitCursor );

// Let's try it this way - hacky style, couldn't find now where it clears MountPoints ;(
//pview->invalidateView();
//pview->checkState();
		
    QStringList parts = pview->allPartitions();
    QStringList::Iterator s;
    for (s = parts.begin(); s != parts.end(); ++s) {
	
  		if (pview->idOfPartition( *s ) == 0x83 || pview->idOfPartition( *s ) == 0x19) {
  	
        if( Fdisk::isInstallSource(*s) && !InstallSelect::blUpdate )
					continue;
		
    		if(InstallSelect::blUpdate)
				{
					if(target->notAvailForUpdate((*s).ascii()))
						continue;
				}

        QString size = pview->sizeOfPartitionAsString( *s );
        QListViewItem* i = new PartitionSelectorItem( this, view, *s, size );

        /* Rem by aa
           // let the user not select partitions he can't boot from
           if (!pview->partitionBelow1024Cylinders(*s))
             i->setSelectable(FALSE);
        */

/*
        // don't let the user select partitions he is installing from
				if(Fdisk::isInstallSource(*s))
          i->setSelectable(FALSE);
*/

        // was there a root partition defined yet?
        QString mp = MountPoints::mountPartition( "/" );
   	
  	    if (mp.isEmpty()) {
   			// if the partition has no mount point or the preferred mount point, select it
  		
  				if (mode == OnlyRootMode) {
	  		    // we are in only root mode, i.e. we want to have the biggest  partition without
  			    // filesystem
  			    bool newIsNew = Fdisk::isNew( *s );
  			    if (!didSelect
      				|| ( newIsNew && !selectedIsNew )
      				|| ( pview->sizeOfPartition( *s ) > selectedSize && ( newIsNew || !selectedIsNew )))
						{
         			view->setSelected( i, TRUE );
         			didSelect = TRUE;
        			selectedSize = pview->sizeOfPartition( *s );
        			selectedIsNew = newIsNew;
	  		    }
  				} else {
	  		    if (!didSelect) {
  						view->setSelected( i, TRUE );
  						didSelect = TRUE;
  			    }
  				}
	   	  } else {
     				if (*s == mp) {
     		    	view->setSelected( i, TRUE );
	     		    didSelect = TRUE;
  	   		}
      	}
	  	}
  	}

	qApp->restoreOverrideCursor();	
}


void PartitionSelector::initUser()
{
    bool didSelect = FALSE;
    double selectedSize = 0;
    QStringList parts = pview->allPartitions();
    QStringList::Iterator s;
    for (s = parts.begin(); s != parts.end(); ++s) {
	
    	if (pview->idOfPartition( *s ) == 0x83 || pview->idOfPartition( *s ) == 0x19 ) {
    	
    	  if ( MountPoints::mountPoint( *s ).isEmpty() || MountPoints::mountPoint( *s ) == "/usr") {
      		// ignore partitions that are already used otherwise
      	
      		QString size = pview->sizeOfPartitionAsString( *s );
      		
      		QListViewItem* i = new PartitionSelectorItem( this, view, *s, size);
      		
      		// was there a main partition defined yet?
      		QString mp = MountPoints::mountPartition( "/usr" );
      		
       		if (mp.isEmpty()) {
     		    // if the partition has no mount point or the preferred mount point, select it
     		    if (!didSelect || pview->sizeOfPartition( *s ) > selectedSize ) {
         			view->setSelected( i, TRUE );
         			didSelect = TRUE;
        			selectedSize = pview->sizeOfPartition( *s );
     		    }
       		} else {
     		    if (*s == mp) {
         			view->setSelected( i, TRUE );
         			didSelect = TRUE;
     		    }
       		}
   	    }
    	}
    }

    // last not least: the None partition
    new PartitionSelectorItem( this, view, "", "");
}





