/***************************************************************************
                              partitionview.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author	          	 : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "partitionview.h"
#include "partitionitem.h"
#include "partitioninfo.h"
#include "diskitem.h"
#include "changepartitiondialog.h"
#include "fdisk.h"
#include "mountpoint.h"
#include "swapper.h"
#include "language.h"

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/reboot.h>
#include <fcntl.h>

#include <qapplication.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>

PartitionView::PartitionView( QWidget *parent, const char *name )
    : QListView( parent, name)
{
#ifndef GBLI_DEVEL
		system( "touch -t 01010000 "+ QString( CONF_MODULES ) );
    system( "modprobe fat -qs" );
    system( "modprobe vfat -qs" );
    system( "modprobe msdos -qs" );
    system( "modprobe ntfs -qs" );
    system( "modprobe reiserfs -qs" );
    system( "modprobe ext2 -qs" );
		system( "touch -t 01010000 "+ QString( CONF_MODULES ) );
#endif

		/*Added by dk, 25.02.2002*/
		/*dk*/
		partitionOverflow=false;
		/*dk*/
	
		setTreeStepSize( 30 );
		setAllColumnsShowFocus( true );
		setShowSortIndicator ( true );

    setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
		QFont font( QFont::defaultFont() );
#ifdef ADVANCED_SIZE_VIEW		
 		font.setPointSize( 14 );
#else
 		font.setPointSize( Language::iSmallFontSize );
#endif
//		font.setFamily( "times" );
    setFont( font );
    setSorting( -1 );
    view_invalidated = TRUE;

    int col;
    col = addColumn( tr("Device") );
    setColumnWidthMode( col, Maximum );
//    setSorting( col, true );

    col = addColumn( tr("Type") );
    setColumnWidthMode( col, Maximum );

    col = addColumn( tr("Mount Point") );
    setColumnWidthMode( col, Maximum );

#ifdef ADVANCED_SIZE_VIEW
    col = addColumn( tr("Start") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );
    setSorting( col, false );
    col = addColumn( tr("End") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );
#endif
    col = addColumn( tr("Size") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignRight );

    col = addColumn( tr("Bootable") );
    setColumnWidthMode( col, Maximum /*Manual*/ );
//    setColumnWidth( col, 30 );
    setColumnAlignment( col, AlignCenter );
    col = addColumn( tr("Format") );
    setColumnWidthMode( col, Maximum );
    setColumnAlignment( col, AlignCenter );

    somethingWritten = FALSE;
		resizeAvail = TRUE;

    connect( this, SIGNAL (selectionChanged( QListViewItem* ) ),
	     this, SLOT (selectionChanged( QListViewItem* ) ) );

    partitionDialog = new ChangePartitionDialog( this );
}

PartitionView::~PartitionView()
{
}

/*! reread the partition table if the view is invalidated */
void PartitionView::checkState()
{
    if (viewInvalidated()) {
			QApplication::setOverrideCursor( waitCursor );
			rereadPartitionTable();
			QApplication::restoreOverrideCursor();
    }
}


PartitionItem* PartitionView::currentPartition()
{
    if (!current || current->isDisk() )
			return 0;
    return (PartitionItem*) current;
}

DiskItem* PartitionView::currentDisk()
{
    if (!current )
			return 0;
    if( currentPartition() )
			return currentPartition()->diskItem;
    return (DiskItem*) current;
}


/*
  invalidates the view. That means the table will be updated when the
  partition view is shown the next time.
 */
void PartitionView::invalidateView()
{
    view_invalidated = TRUE;
}


/*  updates the view */
void PartitionView::updateView(){
//printf( "update view\n" );

    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    p->updateView();
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }

    setSorting( 0, true );
		viewport()->repaint();
}

void PartitionView::editCurrent()
{
    if (!currentPartition())
			return;
    if (currentDisk()->edit( currentPartition() )) {
			emit changed( TRUE );
			resizeAvail = FALSE;
			emit notOriginal( TRUE );
    }
}

void PartitionView::addLogicalPartition()
{
		/*Added by dk, 25.02.2002, prevent partition overflow*/
		/*dk*/
		allPartitions();
		if (partitionOverflow)
			return;				

		/*dk*/
    if (!currentDisk())
			return;
    if (currentDisk()->addLogicalPartition() ) {
			emit changed( TRUE );
			resizeAvail = FALSE;
  		emit notOriginal( TRUE );
    }
}

void PartitionView::removeOrClearCurrent()
{
    if (!currentPartition())
			return;
    currentDisk()->removeOrClear( currentPartition() );

    emit changed( TRUE );
		resizeAvail = FALSE;
    emit notOriginal( TRUE );
}


/*
  Writes the modified partition tables back onto the hardisk.
*/
void PartitionView::writePartitionTable()
{
    QApplication::setOverrideCursor( waitCursor );

		PartitionInfo::unMounting();// lets try to make it this way (Maddy)

    // stop swapping, we might swap on this disk!
    Swapper::swapOff();

    QListViewItem*  i = firstChild();
    while ( i ) {
			((DiskItem*)i)->writePartitionTable();
			i = i->nextSibling();
    }

    somethingWritten = TRUE;
		resizeAvail = FALSE;

    emit changed( FALSE );
    QApplication::restoreOverrideCursor();
}


/*
  Undos everything to the state when the partition view was created.
 */
void PartitionView::defaultPartitionTable()
{
    emit currentAvailable( FALSE );

		PartitionInfo::unMounting();// lets try to make it this way (Maddy)
    Swapper::swapOff();		

    MountPoints::clear(); // there were no mount points when he first came here

    QListViewItem*  i = firstChild();
    while ( i ) {
			( (DiskItem*)i )->defaultPartitionTable();
			i = i->nextSibling();
    }

    emit changed( somethingWritten );
		resizeAvail = !somethingWritten;
    emit notOriginal( FALSE );

    updateView();
}

void PartitionView::rereadPartitionTable()
{
    int diskCount;
		QStringList::Iterator tmp;
    view_invalidated = FALSE;

Fdisk::startup = true;
		PartitionInfo::unMounting();// lets try to make it this way (Maddy)
    Swapper::swapOff();		

    clear();
    // put all available disks into the partition view
    QStringList availableDisks = Fdisk::availableDisks();

    QStringList::Iterator disk;
    DiskItem *di = NULL;

    for (disk = availableDisks.begin(); disk != availableDisks.end(); ++disk) {
			di = new DiskItem( *disk, this, partitionDialog, di );
//printf( "Disk: %s - %s\n", (*disk).ascii(), di->text( 0 ).ascii() );
			di->setOpen( true );
    }

//Added by dk to workaround QStringList::count bug, 01.03.2002
/*dk*/
 QStringList bzz = allPartitions();
 diskCount=0;
 for (tmp=bzz.begin();tmp!=bzz.end();tmp++)
 {
  diskCount++;
 }

    emit diskCheckMaximum( diskCount );
/*dk*/

    int n = 1;
		char winLetter = (char)'c';
		char buf[16];
    for (DiskItem* i =  (DiskItem*)firstChild(); i; i =  (DiskItem*) i->nextSibling() ) {
    	for (PartitionItem* p =  (PartitionItem*)i->firstChild(); p; p =  (PartitionItem*) p->nextSibling() ) {
    	    p->checkForFilesystem();
    	    emit diskCheckProgress( n++ );

					if( p->isWindows() && /*Fdisk::winDrive.length() &&*/ winLetter < 'g' && !Fdisk::isNew( p->name() ) ){
Log::logMain( 33, "Add windows mountpoint\n" );
						sprintf( buf, "/mnt/%c", winLetter++ );
		    		MountPoints::setMountPoint( p->name(), buf );
 					}else if( p->isSwap() ){
 		    		MountPoints::setMountPoint( p->name(), "swap" );
 					}
    	    for (PartitionItem* lp =  (PartitionItem*)p->firstChild(); lp; lp =  (PartitionItem*) lp->nextSibling() ) {
    				lp->checkForFilesystem();
    				emit diskCheckProgress( n++ );
  					if( lp->isWindows() && /*Fdisk::winDrive.length() &&*/ winLetter < 'g' && !Fdisk::isNew( p->name() ) ){
Log::logMain( 32, "Add windows mountpoint.\n" );
							sprintf( buf, "/mnt/%c", winLetter++ );
			    		MountPoints::setMountPoint( lp->name(), buf );
  					}else if( lp->isSwap() ){
			    		MountPoints::setMountPoint( lp->name(), "swap" );
						}
    	    }
    	}
    }


    somethingWritten = FALSE;
		resizeAvail = TRUE;

    emit changed( FALSE );
    emit notOriginal( FALSE );
    Fdisk::startup = FALSE; // end initialization

		updateView();
}

void PartitionView::selectionChanged( QListViewItem * item)
{
    current = (DiskOrPartitionItem*) item;
		PartitionItem	*tmp_item = currentPartition();

		

    if (currentPartition() ) {
			emit deleteAvailable ( tmp_item->id == 0x00 ? FALSE : TRUE );
			emit editAvailable ( TRUE );
			if ( resizeAvail )
				emit resizeAvailable ( (tmp_item->id == 0x06) || (tmp_item->id == 0x0b) ||(tmp_item->id == 0x0c) ||(tmp_item->id == 0x0e) ||(tmp_item->id == 0x0f) ||(tmp_item->id == 0x05) ||(tmp_item->id == 0x82) ||(tmp_item->id == 0x83) );
			else
				emit resizeAvailable ( resizeAvail );
//    	emit currentAvailable( TRUE );
    } else {
    	emit deleteAvailable( FALSE );
    	emit editAvailable( FALSE );
    	emit resizeAvailable( FALSE );
    }
		
    emit diskAvailable( currentDisk() != 0 );
}




/*
  Returns a stringlist of all currently available partitions in this partition view
 */
QStringList PartitionView::allPartitions()
{
    QStringList result;
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    result.append( p->name() );
    	    n++;
					/*Added by dk, 25.02.2002, to avoid crashing with too many partitions on one drive*/
          /*dk*/
					if (n > 255) //255 is the criteria
					{
						partitionOverflow=true;
						break;
					}
					/*dk*/
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return result;
}


/*
  Returns a stringlist of all currently available swap partitions in
  this partition view
 */
QStringList PartitionView::allSwapPartitions()
{
    QStringList result;
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    if ( p->isSwap() )
		    		result.append( p->name() );
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return result;
}


/*
  Returns whether some of the disks already contain at least one Linux
  partition.
 */
bool PartitionView::hasLinuxPartitions()
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while( p ) {
    	    if ( p->id == 0x83 || p->id == 0x19  )
		    		return TRUE;
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

/*
  Returns a pointer to the specified partition item
 */
PartitionItem* PartitionView::partition( const QString& name )
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	int n = 1;
    	PartitionItem* p = disk->partition( n );
    	while (p ) {
    	    if ( p->name() == name )
		    		return p;
    	    n++;
    	    p = disk->partition( n );
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return 0;
}

/*
  Returns the size of the specified partition in MB
 */
double PartitionView::sizeOfPartition(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->sizeInMB();
    return 0;
}

/*!
  Returns the size of the specified partition in MB as QString
 */
QString PartitionView::sizeOfPartitionAsString(const QString& name)
{
    QString result;
    result.setNum( sizeOfPartition( name) );
    return result + tr("MB");
}

/*
  Returns the id of the specified partition (system type)
 */
int PartitionView::idOfPartition(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->id;
    return 0;
}

/*
  Returns whether the specified partition is below the first 1024
  cylinders ( which makes it possible for current PC bioses to boot a
  kernel from such a partition).

 */
bool PartitionView::partitionBelow1024Cylinders(const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
      return p->cylinderOfSector( p->end() ) <= 1024;
    return FALSE;
}


/*
  Returns whether the specfied partition is bootable
 */
bool PartitionView::bootable( const QString& name)
{
    PartitionItem* p = partition( name );
    if ( p )
			return p->bootable;
    return FALSE;
}


/*!
  Sets the bootable flag of the specfied partition
 */
void PartitionView::setBootable( const QString& name, bool value)
{
    PartitionItem* p = partition( name );
    if ( p  && p-> bootable != value ) {
			p->bootable = value; // set the partition's bootable flag
			p->diskItem->invalidateDisk(); // inform the disk that it is dirty now
    }
}

/*
  Returns a pointer to the specified disk item
 */
DiskItem* PartitionView::disk( const QString& name )
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk && disk->disk != name) {
			disk = (DiskItem*) disk->nextSibling();
    }
    return disk;
}


/*!
  Returns the size of the specified disk
 */
double PartitionView::capacityOfDisk(const QString& name)
{
    DiskItem* d = disk( name );
    if (!d)
			return 0;
    return  d->capacity();

}

/*!
  Returns the size of the specified disk as a string
 */
QString PartitionView::capacityOfDiskAsString(const QString& name)
{
    QString result;
    result.setNum( capacityOfDisk( name) );
    return result + tr("MB");
}

bool PartitionView::hasLinuxPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while ( p ) {
        		if ( p->id == 0x83 || p->id == 0x19 )
       		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

bool PartitionView::hasDosWindowsPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while (p ) {
        		if ( p->id == 0x01 ||
        		     p->id == 0x04 ||		
        		     p->id == 0x11 ||		
        		     p->id == 0x14 ||		
        		     p->id == 0x16 ||		
        		     p->id == 0x24 ||		
        		     p->id == 0xc1 ||		
        		     p->id == 0xc4 ||		
        		     p->id == 0xc6 ||		
        		     p->id == 0xc4 ||		
        		     p->id == 0x06 ||		
        		     p->id == 0x0b ||		
        		     p->id == 0x0e ||		
        		     p->id == 0x0f
        		     )
        		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

bool PartitionView::hasPartitions(const QString& name)
{
    DiskItem* disk = (DiskItem*) firstChild();
    while (disk ) {
    	if ( disk->disk == name) {
    	    int n = 1;
    	    PartitionItem* p = disk->partition( n );
    	    while (p ) {
        		if ( p->id != 0 )
        		    return TRUE;
        		n++;
        		p = disk->partition( n );
    	    }
    	}
    	disk = (DiskItem*) disk->nextSibling();
    }
    return FALSE;
}

void PartitionView::formatPartition( QString device )
{
    QString result;
    QApplication::flushX();
    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if (pid == 0) {
    	::close( fd[0] );	
    	
    	if (fd[1] != STDOUT_FILENO) {
    	    dup2( fd[1], STDOUT_FILENO);
    	    ::close( fd[1] );
    	}
    	
    	int id = idOfPartition( device );
Log::logMain( 33, "Formatting %s type %d...\n", device.ascii(), id );
    	if( id == 0x83 ){ // ext2
  	    if( device.left(8) == "/dev/rd/" )
  	      execlp("mkfs.ext2", "mkfs.ext2", device.ascii(), 0 );
  	    else
  	      execlp("mkfs.ext2", "mkfs.ext2", "-m1", device.ascii(), 0 );
   	  }else if( id == 0x19 ){ // ReiserFS
					system( "mkreiserfs -h tea "+ device +" <<EOT\ny\nEOT\n" );
    	}else if( Fdisk::isSwapType( id ) ){
    	    execlp("mkswap", "mkswap", /*"-c", "-v1",*/ device.ascii(), 0);
   	  }else if( id == 0x6 ){ // MSDOS FAT16
					char buf[128];
					sprintf( buf, "dd if=/dev/zero of=%s bs=512 count=1 >/dev/null 2>&1", device.ascii() );
					system( buf );
    	    execlp("mkfs.msdos", "mkfs.msdos", /*"-c",*/ device.ascii(), 0 );
    	    sleep( 5 ); // ;-)
   	  }else if( id == 0x0b ){ // Win9X FAT32
					char buf[128];
					sprintf( buf, "dd if=/dev/zero of=%s bs=512 count=1 >/dev/null 2>&1", device.ascii() );
					system( buf );
    	    execlp("mkfs.vfat", "mkfs.vfat", "-f", "2", "-F", "32", device.ascii(), 0 );
    	    sleep( 5 ); // ;-)
    	}
    	exit(1);
    }
//printf( "hasPartitions pid: %d\n", pid );
    ::close( fd[1] );

    QFile f;
    if ( f.open(IO_ReadOnly, fd[0]) ) {    // file opened successfully
    	QTextStream t( &f );	    // use a text stream
    	result = t.read();
    	::close( fd[0] );
    }
	
    ::waitpid(pid, NULL, 0);

    if ( Fdisk::isSwapType( idOfPartition( device ) ) )
			Swapper::swapOn( device );
}

void PartitionView::Resize()
{
	int fd[2], CPid;
	::pipe(fd);
	QApplication::flushX();
	
	if(!(CPid = ::fork()))
	{
		::close( fd[0] );
  	::close( STDERR_FILENO );
    ::close( STDOUT_FILENO );

    if( fd[1] != STDOUT_FILENO )
    {
    	int res = -1;
      res = dup2( fd[1], STDOUT_FILENO );
      if( res >= 0 && fd[1] != STDERR_FILENO )
      	res = dup2( fd[1], STDERR_FILENO );
			::close( fd[1] );
		}

#ifdef GBLI_DEVEL	
		execlp("/root/parted", "parted", "-s", (currentDisk()->disk).ascii(), "print_with_at", 0);
#else
		execlp("/usr/sbin/parted", "parted", "-s", (currentDisk()->disk).ascii(), "print_with_at", 0);
#endif
		exit(0);
	}
	
	::close(fd[1]);

  int s;
	if ( (s = fcntl( fd[0], F_GETFL, 0 ) ) >= 0 )
	{
		s |= FNDELAY;
		fcntl( fd[0], F_SETFL, s );
	}

	rn = new QSocketNotifier( fd[0], QSocketNotifier::Read, this);
	connect(rn, SIGNAL(activated(int)), this, SLOT(ReadPartitionInfo(int)));
}

void PartitionView::ReadPartitionInfo(int fd)
{
	int available = 0, col = 0, pos = -1;
		
	if(::ioctl(fd, FIONREAD, (char*)&available ) >= 0) // FIONREAD asks the number of bytes ready to be transmitted
	{
		if(available)
		{
	    // text came in
	    QByteArray rba(available);
	    int nread = ::read(fd, rba.data(), available);
	    for(int i = 0; i < nread; i++)
				mdetect += rba[i];

			while((pos = mdetect.find("@")) >= 0)
			{
				int i = 0;
				mdetect.replace(pos, 1, "");
				
				while(mdetect[pos] != ' ')
						(ResFromParted[col])[i++] = mdetect[pos++];
				
//				(ResFromParted[col])[i] = '\0';
				col++;
			}
			
			PartCount = col;			
			ResizeDialog();
		}
		else
		{
			// we got it all
			if( rn )
			{
				delete rn;
				rn = 0;
				::close( fd );
			}
		}
	}
}

int PartitionView::ResizeDialog()
{
	ResizeD *ResizeObj = new ResizeD(this, "Resize", true, this, currentDisk()->disk);
	int chk = ResizeObj->exec();

	if(chk == QDialog::Accepted) ResizeObj->ResultFromSecWin(this);
	return 0;
}

void ResizeD::ChangedSize ( int PRTSize )
{
	QString	tmp, tmp1;
	int a = PRTSize - idStart;
	tmp1 = tmp.setNum ( a );
	if ( tmp1.length() > 3 )
	{
		tmp1.truncate ( tmp1.length() - 3 );
		tmp1 = tmp1 + "." + tmp[tmp.length() - 3] + " GB";
		ScrollSize->setLineStep ( 100 );
		ScrollSize->setPageStep ( 1000 );

	}
	else
	{
		tmp1 = tmp1 + " MB";
		ScrollSize->setLineStep ( 10 );
		ScrollSize->setPageStep ( 100 );
	}

	LabelSize->setText ( tmp1 );
	return;
}

ResizeD::ResizeD(QWidget* parent, const char *name, bool modal, PartitionView *pv, QString CurDisk_) : QDialog(parent, name, modal)
{
	CurDisk = CurDisk_;
	PartCount = pv->PartCount;
	ResFromParted = pv->ReturnResFromParted();
//	char *label_names[] = { "Minor", "Start", "End", "Type", "Filesystem", "Name", "Flags" };
	
	PartitionItem	*pPartItem = pv->currentPartition();
	QString	sCurPart;

  QDialog::setCaption(QDialog::tr("Resize Partition"));

  QPushButton *OKBut = new QPushButton (QDialog::tr("Ok"), this);
  QObject::connect(OKBut, SIGNAL(clicked()), this, SLOT(accept()));

  QPushButton *CancelBut = new QPushButton (QDialog::tr("Cancel"), this);
  QObject::connect(CancelBut, SIGNAL(clicked()), this, SLOT(reject()));
	
	QBoxLayout *Vmain = new QVBoxLayout(this, 10);
	QBoxLayout *Hsubmain = new QHBoxLayout(3);

//	LineEdit = (QLineEdit**)::malloc(sizeof(QLineEdit*) * (PartCount / Columns) * 2);
//	QBoxLayout *Vlabels[Columns];

	if ( pPartItem )
		sCurPart = pPartItem->name();

	int iExtStart = -1, iExtEnd = -1, itmpExt;
	QString	sExtStart, sExtEnd, tmpExt;

	bool insideExtended = FALSE;
	int	ss,se;

	for(int j = 0; j < PartCount; j += Columns)
	{
		if ( insideExtended )
		{
			tmpExt = ResFromParted[j+1];tmpExt.truncate( tmpExt.find ( "." ));itmpExt = tmpExt.toInt();
			if ( itmpExt >= iExtEnd ) insideExtended = FALSE;
		}

    if ( ResFromParted[3+j] == "extended" )
		{
			insideExtended = TRUE;
			tmpExt = sExtStart = ResFromParted[j+1];tmpExt.truncate( tmpExt.find( "." ));iExtStart = tmpExt.toInt();
			tmpExt = sExtEnd = ResFromParted[j+2];tmpExt.truncate( tmpExt.find( "." ));iExtEnd = tmpExt.toInt();
		}

		if ( sCurPart == CurDisk + ResFromParted[0+j] )
		{
			sCurPartition = ResFromParted[0+j];
			iCurPartition = j;

					dStart = ResFromParted[1+j];dStart.truncate ( dStart.find (".") );ipStart = idStart = dStart.toInt();
					dEnd = ResFromParted[2+j];dEnd.truncate ( dEnd.find (".") );idEnd = dEnd.toInt();

					if ( (j < ( PartCount - Columns )) && (ResFromParted[3+j] != "extended")  )
					{
						pEnd= ResFromParted[1+j+Columns];
						pEnd.truncate ( pEnd.find ( "." ));
						ipEnd = pEnd.toInt();
						if ( insideExtended && (ipEnd >= iExtEnd)) ipEnd = iExtEnd;
					}
					else
						ipEnd = idEnd;					

					ss = idStart;
					se = ipEnd;
					if (ResFromParted[3+j] == "extended")
					{
						ipStart = iExtStart;
						ipEnd = iExtEnd;
            idEnd = iExtEnd;
						int	a = j;
          	while ( a < PartCount )
            {
            	tmpExt = ResFromParted[a+2];tmpExt.truncate( tmpExt.find ( "." ));itmpExt = tmpExt.toInt();
              if ( iExtEnd >= itmpExt ) ss = itmpExt;
              a += Columns;
            }
					}
//					printf ( "pStart:%i dStart:%i dEnd:%i pEnd:%i\n", ipStart, idStart, idEnd, ipEnd );

					pSize = ipEnd - ipStart;
					dSize = idEnd - idStart;

					ScrollSize = new QScrollBar(ss, se, 1, 10, idStart+dSize, QScrollBar::Horizontal, this);
					QObject::connect(ScrollSize, SIGNAL(valueChanged( int )), this, SLOT(ChangedSize( int )));	
					
					ScrollSize->setMinimumWidth ( 200 );
					Hsubmain->addWidget(ScrollSize, 0, AlignCenter);

					QString	sLabel = tmpp.setNum ( dSize );

					if ( tmpp.length() > 3 )
					{
          	sLabel.truncate ( sLabel.length() - 3 );
						sLabel = sLabel + "." + tmpp[tmpp.length() - 3] + " GB";
						ScrollSize->setLineStep ( 100 );
						ScrollSize->setPageStep ( 1000 );
					}					
          else
					{
						sLabel = sLabel + " MB";					
						ScrollSize->setLineStep ( 10 );
						ScrollSize->setPageStep ( 100 );
					}

					LabelSize = new QLabel(this);
					LabelSize->setText ( sLabel );
					Hsubmain->addWidget(LabelSize, 0, AlignCenter);
		}	
	}
		
	Vmain->addLayout(Hsubmain);
		
	QBoxLayout *Hbut = new QHBoxLayout();
	Hbut->addWidget(OKBut);
	Hbut->addStretch();
	Hbut->addWidget(CancelBut);

	Vmain->addLayout(Hbut);

	Vmain->activate();
}

void ResizeD::ResultFromSecWin(PartitionView* pv)
{
	QString Minor[3];
	if ( ScrollSize->value() != (idStart + dSize) )
	{
		Minor[0] = ResFromParted[iCurPartition+0];
		Minor[1] = ResFromParted[iCurPartition+1];
		Minor[2].setNum ( ScrollSize->value() );
		Minor[2] = Minor[2] + ".000";

#ifdef GBLI_DEVEL	
		printf("%s\n", ("parted " + CurDisk + " resize " + Minor[0] + " " + Minor[1] + " " + Minor[2]).ascii());
#else
//		if(!::fork())
		qApp->setOverrideCursor( Qt::waitCursor );
		int pid = ::fork();
		if ( !pid )
		{
//			printf("%s\n", ("parted " + CurDisk + " resize " + Minor[0] + " " + Minor[1] + " " + Minor[2]).ascii());
			execlp("/usr/sbin/parted", "parted", "-s", CurDisk.ascii(), "resize", Minor[0].ascii(), Minor[1].ascii(), Minor[2].ascii(), 0);
			exit(0);
		}
		::waitpid(pid, NULL, 0);
		qApp->restoreOverrideCursor();	
#endif
	}	
	updatePartitions(pv);
}

void ResizeD::updatePartitions(PartitionView* pv)
{
  QString s,ts;
	QString newPartitions = Fdisk::getPartitionTable( CurDisk );

 	signed long start = 0;
  signed long size = 0;

  int len;
  int pos;

  QTextIStream t( &newPartitions );	    // use a text stream

  while( !t.eof() )
	{
  	s = t.readLine();	    // line of text excluding '\n'
  	if( !s.isEmpty() && s[0] == '/' )
		{
			if ( s.find ( CurDisk+sCurPartition ) >= 0 )
			{
//				printf ("%s\n",s.ascii());
	
		    int colon = s.find(':');

		    QRegExp startR( "start= *[0-9]+" );
  		  pos = startR.match(s, 0, &len);
    		if ( len > 6 && pos > colon )
					start = s.mid(pos+6, len-6).toInt();
		    QRegExp sizeR( "size= *[0-9]+" );
  		  pos = sizeR.match(s, 0, &len);
    		if ( len > 5 && pos > colon )
					size = s.mid(pos+5, len-5).toInt();

//				printf ( " %ld : %ld \n", start, size);

			  DiskItem* disk = (DiskItem*) pv->firstChild();
  			while (disk )
				{
  				int n = 1;
		  		PartitionItem* p = disk->partition( n );
			   	while (p )
					{
//   		    	result.append( p->name() );
						if ( (CurDisk+sCurPartition)==(p->diskItem->disk+ts.setNum(p->number)))
            {
//							printf ( "BLABLA : %s, %d\n",p->diskItem->disk.ascii(), p->number);
							p->size = size;
	            p->start = start;
							p->updateView();
						}							
	   	  	  n++;
	   	    	p = disk->partition( n );
			   	}
   				disk = (DiskItem*) disk->nextSibling();
			  }
			}

		}
//    	insertPartition( s );
  }
}
