/***************************************************************************
                               timezone.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "timezone.h"
#include "linlinehelp.h"
#include "liconloader.h"
#include "language.h"
#include "log.h"
#include "sysconfigfile.h"

#include <qlistview.h>
#include <qlayout.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qlist.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qtl.h>
#include <qvaluelist.h>
#include <qdatetime.h>
#include <qpushbutton.h>

#include <unistd.h>


ZoneInfo::ZoneInfo( const QString& line, bool coords )
{
    
    coordinates = coords;
    
    if ( coordinates ) { //zone.tab format
	country = line.left(2);
	int i = 3;
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	QString tmpnum = line.mid(i, 3);
	tmpnum += '.';
	i += 3;
	while (i < int(line.length() ) && char(line[i]) >= '0' && char(line[i]) <= '9' ){
	    tmpnum += line[i];
	    i++;
	}
	y = tmpnum.toDouble();

	tmpnum = line.mid(i, 4);
	tmpnum += '.';
	i += 4;
	while (i < int(line.length() ) && char(line[i]) >= '0' && char(line[i]) <= '9' ){
	    tmpnum += line[i];
	    i++;
	}
	x = tmpnum.toDouble();

	while (i < int(line.length() ) && line[i].isSpace())
	    i++;

	while (i < int(line.length() ) && !line[i].isSpace()) {
	    city += line[i];
	    i++;
	}
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	while (i < int(line.length() ) ){
	    comment += line[i];
	    i++;
	}
    }
    else { // extrazone.tab format
	int i = 0;
	while (i < int(line.length() ) && !line[i].isSpace()) {
	    city += line[i];
	    i++;
	}
	while (i < int(line.length() ) && line[i].isSpace())
	    i++;
	while (i < int(line.length() ) ){
	    comment += line[i];
	    i++;
	}
    }
}

#include "xserver.h"

World::World( QWidget * parent, const char * name )
    : QWidget( parent, name)
{
    setWFlags( WResizeNoErase );

    zones.setAutoDelete( TRUE );
    w = h = relw = relh = ox = oy = 0;
    setMouseTracking( TRUE );

    high = 0;

		if( XServer::runSVGA )
	    worldpm.load( QString( GBLI_DATA_PATH ) +"/graphics/world.gif" );
		else
	    if( !worldpm.load( QString( GBLI_DATA_PATH ) +"graphics/world16.gif" ) )
		    worldpm.load( QString( GBLI_DATA_PATH ) +"/graphics/world.gif" );

    w = worldpm.width();
    h = worldpm.height();
    setFixedSize( worldpm.size() );

    relw = w - 0;
    relh = h + 12; //24 for the big one

    ox = w/2 + 6; //12 for the big one
    oy = h/2 + 6; //12 for the big one

    cityIndicator
			= new QLabel(0, 0, WStyle_Customize | WStyle_NoBorder | WStyle_Tool );
    cityIndicator->setAutoResize( TRUE );
    cityIndicator->setFrameStyle( QFrame::Box | QFrame::Plain);
    cityIndicator->setBackgroundMode( PaletteBase );

}


World::~World()
{
    delete cityIndicator;
    cityIndicator = 0;
}



void World::readInZoneInfo( QComboBox * combo )
{
  if ( !combo || combo->count() )
		return;

  QStringList l;

  // extra timezones
	QFile f( QString( GBLI_DATA_PATH ) +"/data/extrazone.tab" );
	if ( f.open( IO_ReadOnly) ) {
		QTextStream t(&f);
		while ( !t.atEnd() ) {
			QString line = t.readLine();
			if (line[0] != '#') {
		    zones.append( new ZoneInfo( line, FALSE ) );
		    zoneDict.insert( zones.last()->city, zones.last() );
//		    combo->insertItem( zones.last()->city );
  	    l.append( zones.last()->city );
			}
		}
		f.close();
	}

	f.setName( QString( GBLI_DATA_PATH ) +"/data/zone.tab"	);
	if ( f.open( IO_ReadOnly) ) {
		QTextStream t(&f);
		while ( !t.atEnd() ) {
  		QString line = t.readLine();
  		if (line[0] != '#') {
  	    zones.append( new ZoneInfo( line ) );
  	    l.append( zones.last()->city );
  	    zoneDict.insert(zones.last()->city, zones.last() );
  		}
  	}
		f.close();
	}

  qBubbleSort( l.begin(), l.end() );
  for ( QStringList::Iterator i = l.begin(); i != l.end(); ++i ){
		combo->insertItem( *i );
		if( *i == "Europe/London" )
			combo->setCurrentItem( combo->count() - 1 );
	}
}


ZoneInfo* World::nearestZone( int x, int y)
{
    double d = -1;
    ZoneInfo* result = 0;
    for (ZoneInfo* i = zones.first(); i; i = zones.next() ) {
	if (!i->coordinates)
	    continue;
	int cy = (int) (oy - relh * i->y/180);
	int cx = (int) (ox + relw * i->x/360);
	if (cx > w )
	    cx = cx-w;
	double nd = (cx-x)* (cx-x)+ (cy-y)* (cy-y);
	if ( (d < 0 || nd < d) && !(cx >= x && cy >= y) ) {
	    d = nd;
	    result = i;
	}
    }
    return result;
}

QString World::getComment( const QString& zone)
{
    ZoneInfo* i = zoneDict[zone];
    if (i)
	return  i->comment;
    return QString::null;
}


void World::highlight( const QString& arg)
{
    ZoneInfo* oldHigh = high;
    if (!high || high->city != arg)
	high = zoneDict[arg];

    if (!high)
	return;

    if (high != oldHigh && oldHigh) {
	int y = (int) (oy - relh * oldHigh->y/180);
	int x = (int) (ox + relw * oldHigh->x/360);
	if (x > w )
	    x = x-w;
	repaint( x-6, y-6, 13, 13, FALSE );
    }

    if (!high->coordinates) {
	high = 0;
	cityIndicator->setText("");
	cityIndicator->hide();
	return;
    }

    int y = (int) (oy - relh * high->y/180);
    int x = (int) (ox + relw * high->x/360);
    if (x > w )
	x = x-w;
    repaint( x-6, y-6, 13, 13, FALSE );
    QString s = high->city;
    if ( s.findRev('/') != -1 )
	s = s.right( s.length() - s.findRev('/') - 1 );
    if ( s != cityIndicator->text() )
	{
		s.replace(QRegExp("_")," ");
		cityIndicator->setText( s );
	}

    if ( !cityIndicator->isVisible() )
	cityIndicator->hide();
    emit highlighted(high->city);
}


void World::mouseMoveEvent( QMouseEvent* e)
{
    cityIndicator->move( e->globalPos() + QPoint( 10, 10) );
    ZoneInfo* i = nearestZone( e->pos().x(), e->pos().y() );
    if (i && i != high ) {
	if (high) {
	    int y = (int) (oy - relh * high->y/180);
	    int x = (int) (ox + relw * high->x/360);
	    if (x > w )
		x = x-w;
	    high = 0;
	    repaint( x-6, y-6, 13, 13, FALSE );
	}
	high = i;
	highlight( high->city );
	QApplication::syncX(); // for VGA16
    }
}


void World::enterEvent( QEvent * )
{
    if ( !cityIndicator->text().isEmpty() )
	cityIndicator->show();
    cityIndicator->raise();
}


void World::leaveEvent( QEvent * )
{
    cityIndicator->hide();
}


void World::resizeEvent( QResizeEvent* )
{
}


void World::paintEvent( QPaintEvent*  e)
{
    QPainter p( this );
    p.setClipRegion( e->region() );
    p.setClipping( TRUE );

    p.drawPixmap( 0,0, worldpm );

    p.setPen( black );

    for (ZoneInfo* i = zones.first(); i; i = zones.next() ) {
	int y = (int) (oy - relh * i->y/180);
	int x = (int) (ox + relw * i->x/360);
	if (x > w )
	    x = x-w;
	p.drawEllipse( x-1, y-1, 3, 3);
    }

    if ( high ) {
	int y = (int) (oy - relh * high->y/180);
	int x = (int) (ox + relw * high->x/360);
	if (x > w )
	    x = x-w;
	p.setPen( white );
	p.setBrush( white );
	p.drawEllipse( x-1, y-1, 3, 3);
	p.setBrush( NoBrush );
	p.drawEllipse( x-6, y-6, 13, 13);
	p.setPen( black );
    }
	

}


void World::mouseReleaseEvent( QMouseEvent * )
{
    if (high)
	emit selected(high->city);
}


TimeZone::~TimeZone()
{
}


TimeZone::TimeZone( QWidget * parent,
		      const char * name )
    : GBLIPage( parent, name), iHourDiff( 0 )
{
		QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 0 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		lh->addStretch( 1 );

    world = new World( this );

		l->addStretch( 1 );

    l->addWidget( world );

		l->addSpacing( 2 );
    comboZone = new QComboBox( false, this );
    l->addWidget( comboZone );

		l->addSpacing( 5 );
    comment = new QLabel( "|", this );
    comment->setAlignment( AlignTop );
    QFont commentFont = comment->font();
    commentFont.setItalic( TRUE );
    comment->setFont( commentFont );
    l->addWidget( comment, 2 );
		comment->setAutoMask( true );

    connect( world, SIGNAL( selected(const QString&) ),
	     this, SLOT( setTimezone(const QString&) ) );

    connect( comboZone, SIGNAL( activated(const QString&) ),
	     world, SLOT( highlight(const QString&) ) );
    
    connect( comboZone, SIGNAL( activated(const QString&) ),
	     this, SLOT( setTimezone(const QString&) ) );


    radioLocaltime
			= new QRadioButton( tr("Hardware clock runs in &local time"), this );
		radioLocaltime->setAutoMask( true );
radioLocaltime->hide();
    radioLocaltime->setChecked( TRUE );
    radioGMT
			= new QRadioButton( tr("Hardware clock runs in &GMT time"), this );
		radioGMT->setAutoMask( true );
radioGMT->hide();

		QLabel *lab = new QLabel( tr( "Enter your current time:" ), this );
		lab->setAutoMask( true );
		curTime = new QLineEdit( "XX:XX", this );
		curTime->setMinimumWidth( 50 );
		curTime->setMaximumWidth( 50 );
		curTime->setMaxLength( 5 );

    QButtonGroup* g = new QButtonGroup( this );
    g->hide();
    g->insert( radioGMT );
    g->insert( radioLocaltime );

//QPushButton *but = new QPushButton( tr( "Update" ), this );

		l->addStretch( 1 );
		QHBoxLayout * lhr = new QHBoxLayout( 6 );
		l->addLayout( lhr );
    lhr->addWidget( radioLocaltime );
    lhr->addWidget( radioGMT );
		lhr->addWidget( lab, 1 );
		lhr->addWidget( curTime, 1 );
//		lhr->addWidget( but );
		lhr->addStretch( 10 );
		l->addStretch( 3 );

#ifndef GBLI_DEVEL
/*		struct timeval time;
		gettimeofday( &time, NULL );
		time.tv_sec -= 86400;// full day back
    // in case someone invents time machine and travels
    // back to 0 time to install Best Linux... hence we have to check
    // the condition to be sure
		if( time.tv_sec < 0 )
			time.tv_sec = 0;
    settimeofday( &time, NULL );*/
#endif

		updateTimer = new QTimer( this );
		connect( updateTimer, SIGNAL( timeout() ), this, SLOT( updateTimeField() ) );

    prepared = FALSE;
}


void TimeZone::showEvent( QShowEvent* )
{
    prepare();
		updateTimeField();
		updateTimer->start( 30000 );
}

void TimeZone::hideEvent( QShowEvent* )
{
		updateTimer->stop();	
}

void TimeZone::prepare()
{
    if ( prepared )
			return;
    prepared = TRUE;

    world->readInZoneInfo( comboZone );

    QString lang = QString( *Language::lang->lang ).lower();

    // set default timezones for each language
    if ( lang == "german" )
			setTimezone("Europe/Berlin");
    else if ( lang == "italian" )
			setTimezone("Europe/Rome");
    else if ( lang == "french" )
			setTimezone("Europe/Paris");
    else if ( lang == "estonian" )
			setTimezone("Europe/Tallinn");
    else if ( lang == "finnish" )
			setTimezone("Europe/Helsinki");
    else if ( lang == "swedish" )
			setTimezone("Europe/Stockholm");
    else if ( lang == "norwegian" )
			setTimezone("Europe/Oslo");
    else if ( lang == "french" )
			setTimezone("Europe/Paris");
    else if ( lang == "ukrainian" )
			setTimezone("Europe/Kiev");
    else if ( lang == "spanish" )
			setTimezone("Europe/Madrid");
    else if ( lang == "icelandic" )
			setTimezone("Atlantic/Reykjavik");
    else if ( lang == "russian" )
			setTimezone("Europe/Moscow");
    else
			setTimezone("Europe/London");

    world->highlight( timezone() );
		emit statusChanged( GBLIPage::Good );
}

void TimeZone::updateTimeField(){
	QTime tim = QTime::currentTime();
	int min = tim.minute();
	int hour = tim.hour();
	QString curT = curTime->text();
	if( curT != "XX:XX" ){
  	int curH = curT.left( 2 ).toInt();
  	int curM = curT.mid( curT.find( ":" ), 2 ).toInt();
  	if( curH != hour ){
  		if( curM != 59 ){
  			if( abs( curH - hour ) > 0 )
  				iHourDiff = curH - hour;
  		}
  	}
	}
	curTime->setText( QString::number( hour + iHourDiff ) +":"+
		(min<10?"0":"")+ QString::number( min ) );
}

void TimeZone::setTimezone( const QString& z)
{
    if (comboZone->currentText() != z ) {
    	for (int i = 0; i < comboZone->count(); i++) {
        if (z == comboZone->text(i) ) {
		   		comboZone->setCurrentItem( i );
        }
    	}
    }
    
    comment->setText( world->getComment( z ) );
}

QString TimeZone::timezone()
{
    return comboZone->currentText();
}

void TimeZone::finish( QString dir ){
	if( dir.length() ){
		// get time back to normal
#ifndef GBLI_DEVEL
/*		struct timeval time;
		gettimeofday( &time, NULL );
		time.tv_sec += 86400; // one day +
		if( time.tv_sec < 0 )
			time.tv_sec = 0;
    settimeofday( &time, NULL );*/
#endif
	}

//			QString opts( "--hctosys " );
//				opts += "-u";
//			QString cmd( "/sbin/hwclock "+ opts );
//			Log::logMain( 32, ("Trying to set clock by: "+ cmd).ascii() );
//#ifdef GBLI_DEVEL_TODODODODODO
//// TODO: check & fix packages installation - it will break its time!!!!
//			system( cmd.ascii() );
//#endif

  // write the entry in /etc/sysconfig/clock
/*	system( ("touch "+ dir +"/etc/sysconfig/clock").ascii() );
  SysConfigFile config( dir.ascii(), "/etc/sysconfig/clock" );
  config.load();
  config.set( "UTC", radioGMT->isChecked() ? "true" : "false", false );
  config.set( "ARC", "false", false );
  config.set( "ZONE", timezone().ascii() );
  config.save();

  // create the /etc/localtime symlink
	QString localTime = dir + "/etc/localtime";
	if( QFileInfo( localTime ).exists() )
	  ::rename( localTime.ascii(), (localTime +".old").ascii() );
  ::symlink( ("/usr/share/zoneinfo/"+ timezone()).ascii(), localTime.ascii() );
*/
}


