/***************************************************************************
                          windowsconfig.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Jan 11 2001                                           
    copyright            : (C) 2001 SOT Finnish Software Engineering Ltd. 
    author		 : Anton Safonov (Maddy)                                         
    author email         : as@bestlinux.net                                          
    url			 : http://www.bestlinux.net                          
    email		 : linux@bestlinux.net                              

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "windowsconfig.h"
#include "config.h"

WindowsConfig*		pWindowsConfig = new WindowsConfig("linux.cfg");

WindowsConfig::WindowsConfig()
{
	sConfigFileName = "";
}

WindowsConfig::WindowsConfig(char* szFileName)
{
	sConfigFileName = szFileName;
}

WindowsConfig::~WindowsConfig()
{
}

void WindowsConfig::setConfigFileName(char* szFileName)
{
	sConfigFileName = szFileName;
}

bool WindowsConfig::getNetworkSettings(Netinfo* pNetInfo)
{
	char			szBuf[256];
	QString			sTmp,
					sName;
	SplitString		sValues;	

	if (Ini.ProfileStr(sConfigFileName, "TCP/IP", "HostName", "", szBuf) != 0)
	{
		sTmp = szBuf;
		if (Ini.ProfileStr(sConfigFileName, "TCP/IP", "Domain", "", szBuf) != 0)
		{
			sTmp += ".";
			sTmp += szBuf;
		}
		pNetInfo->setHostname(sTmp, Netinfo::Windows);
	} else return FALSE;

	if (Ini.ProfileStr(sConfigFileName, "TCP/IP", "DNS", "", szBuf) != 0)
	{
		sValues.SetSectionsString(szBuf);
		if (sValues.Next(sTmp)) pNetInfo->setNameserver(sTmp, Netinfo::Windows);
		if (sValues.Next(sTmp)) pNetInfo->setNameserver2(sTmp, Netinfo::Windows);
	} else return FALSE;
	
	if (Ini.ProfileStr(sConfigFileName, "Net", "Names", "", szBuf) != 0)
	{
		sValues.SetSectionsString(szBuf);
		while (sValues.Next(sName))
		{
#ifdef GBLI_DEVEL
			printf(">Network device: %s\n", sName.ascii());
#endif		
			if (sName=="Dial-Up Adapter") continue;
			if (Ini.ProfileStr(sConfigFileName, sName, "Gateway", "", szBuf) != 0)
				pNetInfo->setGateway(szBuf, Netinfo::Windows);
#ifndef GBLI_IPv6
			if (Ini.ProfileStr(sConfigFileName, sName, "IPAddress", "", szBuf) != 0)
			{
				if (!strcmp(szBuf, "0.0.0.0"))
					pNetInfo->setMode(Netinfo::UseDHCPConfig);
				else
				{
					pNetInfo->setAddress(szBuf, Netinfo::Windows);
					pNetInfo->setMode(Netinfo::UseStaticConfig);
				}
			}
			if (Ini.ProfileStr(sConfigFileName, sName, "IPMask", "", szBuf) != 0)
				pNetInfo->setNetmask(szBuf, Netinfo::Windows);
#else
			if (Ini.ProfileStr(sConfigFileName, sName, "IPAddress", "", szBuf) != 0)
			{
				if (!strcmp(szBuf, "0.0.0.0"))
					pNetInfo->setMode("eth0", Netinfo::UseDHCPConfig);
				else
				{
					pNetInfo->setAddress("eth0",szBuf, Netinfo::Windows);
					pNetInfo->setMode("eth0", Netinfo::UseStaticConfig);
				}
			}
			if (Ini.ProfileStr(sConfigFileName, sName, "IPMask", "", szBuf) != 0)
				pNetInfo->setNetmask("eth0",szBuf, Netinfo::Windows);
#endif
		}
	} else return FALSE;
	
	return TRUE;
}

QString WindowsConfig::getUserName()
{
	char			szBuf[256];
	
	Ini.ProfileStr(sConfigFileName, "User Settings", "UserName", "", szBuf);
	return QString(szBuf);
}

bool WindowsConfig::getMonitorSettings(LMonitor* pMonitor)
{
	QString		sMonitorName,
				sTmp,
				sVendorName;
	int			iHorzMin, iHorzMax,
				iVertMin, iVertMax;
	SplitString		sValues;	
	char			szBuf[256];
	
	if (Ini.ProfileStr(sConfigFileName, "Monitor", "Names", "", szBuf) != 0)
	{
		sValues.SetSectionsString(szBuf);
		if (!sValues.Next(sMonitorName)) return FALSE;
		
		if (Ini.ProfileStr(sConfigFileName, sMonitorName, "Manufacturer", "", szBuf) != 0)
		{
			sVendorName=szBuf;	
		}
		else return FALSE;
		if (Ini.ProfileStr(sConfigFileName, sMonitorName, "HorzRefresh", "", szBuf) != 0)
		{
			sValues.SetSectionsString(szBuf);
			sValues.SetSeparatorChar('-');
			sValues.Next(sTmp);
			iHorzMin = sTmp.toInt();
			sValues.Next(sTmp);
			iHorzMax = sTmp.toInt();
		}
		else return FALSE;
		if (Ini.ProfileStr(sConfigFileName, sMonitorName, "VertRefresh", "", szBuf) != 0)
		{
			sValues.SetSectionsString(szBuf);
			sValues.SetSeparatorChar('-');
			sValues.Next(sTmp);
			iVertMin = sTmp.toInt();
			sValues.Next(sTmp);
			iVertMax = sTmp.toInt();
		}		
		else return FALSE;
	}
	else return FALSE;
	
	sMonitorName = sMonitorName.right(sMonitorName.length() - sVendorName.length() - 1 );
	pMonitor->setMonitor( sVendorName, sMonitorName, iHorzMin, iHorzMax, iVertMin, iVertMax );
	
	return TRUE;
}

bool WindowsConfig::getModemSettings(Modem* pModem)
{
	QString		sModemName,
						sTmp;
	char			szBuf[256];
	SplitString		sValues;
	LModemData*		pModemData = pModem->getModemData();
	
	if (Ini.ProfileStr(sConfigFileName, "Modem", "Names", "", szBuf) != 0)
	{
		sValues.SetSectionsString(szBuf);
		if (!sValues.Next(sModemName)) return FALSE;
		
		if (Ini.ProfileStr(sConfigFileName, sModemName, "ConnectedTo", "", szBuf) != 0)
		{
			sTmp = szBuf;
			if (sTmp=="COM1") pModemData->setDevice("/dev/ttyS0");
			else if (sTmp=="COM2") pModemData->setDevice("/dev/ttyS1");
			else if (sTmp=="COM3") pModemData->setDevice("/dev/ttyS2");
			else if (sTmp=="COM4") pModemData->setDevice("/dev/ttyS3");
		} else return FALSE;
		
		if (Ini.ProfileStr(sConfigFileName, sModemName, "ResetString", "", szBuf) != 0)
		{
			sTmp = szBuf;
			sTmp.truncate(sTmp.length()-4);  // remove '<cr>'
			pModemData->setInitString(sTmp.ascii());		
		}

		if (Ini.ProfileStr(sConfigFileName, sModemName, "Speed", "", szBuf) != 0)
		{
			sTmp = szBuf;
			int iSpeed = sTmp.toInt();
			if (iSpeed <= 14400) pModemData->setSpeed(14400);
			else if (iSpeed <= 28800) pModemData->setSpeed(28800);
			else if (iSpeed <= 33600) pModemData->setSpeed(33600);
			else if (iSpeed <= 56000) pModemData->setSpeed(56000);
			else pModemData->setSpeed(64000);
		}
		
		if (Ini.ProfileStr(sConfigFileName, sModemName, "FlowControl", "", szBuf) != 0)
		{
			sTmp = szBuf;
			if (sTmp=="None") pModemData->setFlow(LModemData::None);
			else if (sTmp=="Software") pModemData->setFlow(LModemData::Software);
			else if (sTmp=="Hardware") pModemData->setFlow(LModemData::Hardware);
		}
		
		if (Ini.ProfileStr(sConfigFileName, sModemName, "FlowControl_Software", "", szBuf) != 0)
		{
			sTmp.sprintf("AT%s", szBuf);
			pModemData->setSoftCommand(sTmp.ascii());
		}		
		
		if (Ini.ProfileStr(sConfigFileName, sModemName, "FlowControl_Hardware", "", szBuf) != 0)
		{
			sTmp.sprintf("AT%s", szBuf);
			pModemData->setHardCommand(sTmp.ascii());
		}
		pModemData->setModel(sModemName);
		
		pModemData->setExists(TRUE);
	}
	else return FALSE;

	pModem->updateWidgets();

	return TRUE;
}

void WindowsConfig::getISPEntrys(ISPPage* pISPPage)
{
	QString		sEntryName,
						sTmp;
	char			szBuf[256];
	SplitString		sValues;
	
	if (Ini.ProfileStr(sConfigFileName, "Dial-up connections", "Names", "", szBuf) != 0)
	{
		sValues.SetSectionsString(szBuf);
		while (sValues.Next(sEntryName))
		{		
	    LISP*							pISP = new LISP();
			
      pISP->setName(sEntryName);
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "PhoneNumber", "", szBuf) != 0)
	  	{
				pISP->setPhone(QString(szBuf));
			} else continue;
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "DNS", "", szBuf) != 0)
	  	{
				pISP->setDNS1(QString(szBuf));
			}
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "DNS2", "", szBuf) != 0)
	  	{
				pISP->setDNS2(QString(szBuf));
			}
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "UserName", "", szBuf) != 0)
	  	{
				pISP->setUser(QString(szBuf));
			}
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "Password", "", szBuf) != 0)
	  	{
				pISP->setPassword(QString(szBuf));
			}
			if (Ini.ProfileStr(sConfigFileName, sEntryName, "ServerType", "", szBuf) != 0)
	  	{
				pISP->setPPP(QString(szBuf) == "PPP");				
			}

			pISPPage->addWindowsEntry(pISP);
		}
	}
}

/*
int advCmpStr( QString s1, QString s2 )
{
	SplitString		ss1, ss2;
	int						WordsInString = 0;
	QString				sSearchWord1,
								sSearchWord2;
	
	if (s1 == s2) return 100;
	
	ss1.SetSectionsString(s1);
	ss1.SetSeparatorChar(' ');
	ss2.SetSectionsString(s2);
	ss2.SetSeparatorChar(' ');
	while (ss1.Next(sSearchWord1) && ss2.Next(sSearchWord2))
	{
		if (sSearchWord1 == sSearchWord2) WordsInString++;
		else break;
	}

	return WordsInString;	
}
*/

bool WindowsConfig::getPrinterSettings(PrinterCfg* printer)
{
	char buf[256];
	int printerType;
	bool found = false;
	QString printerName, defaultPrn, paperSize, DPI, device = "", host, shareName, tmp;
	SplitString printerNames, netValuesTmp, netValues, advValues;
	QListViewItem *model;

	//Log::logMain(32, "config: %s\n", sConfigFileName.latin1());
	
	if(Ini.ProfileStr(sConfigFileName, "Printers", "Default", "", buf) != 0)
		defaultPrn = buf;
	else
		return false;
	
	if(Ini.ProfileStr(sConfigFileName, "Printers", "Names", "", buf) != 0)
		printerNames.SetSectionsString(buf);
	else
		return false;
	
	while(printerNames.Next(printerName))
	{
		if(printerName == defaultPrn)
		{
			found = printer->lookForPrinter(printerName, &model);
			
			if(found == false)
			{
				printf("%s is not found in database\n", printerName.latin1());
				return false;
			}

			printerType = PrinterCfg::Local;
			if(Ini.ProfileStr(sConfigFileName, printerName.latin1(), "Type", "", buf) != 0)
			{
				tmp = buf;
				if(tmp == "Network")
					printerType = PrinterCfg::SMB;
			}

      Ini.ProfileStr(sConfigFileName, printerName.latin1(), "Port", "LPT1:", buf);
      if(printerType == PrinterCfg::Local)
      {
      	tmp = buf;
      	tmp.truncate(3);
      	if(tmp == "LPT")
      		device.sprintf("/dev/lp%c (LPT%c)", buf[3] - 1, buf[3]);
      	else
      		device.sprintf("/dev/ttyS%c (COM%c)", buf[3] - 1, buf[3]);
      }
      else
      {
/*
      	netValuesTmp.SetSectionsString(&buf[2]);
      	netValuesTmp.SetSeparatorChar('\\');
      	netValuesTmp.Next(host);
      	netValuesTmp.Next(shareName);

				netValues += host;
				netValues += shareName;
*/
				netValues.SetSectionsString(buf);
      }

			if (Ini.ProfileStr(sConfigFileName, printerName.latin1(), "PaperSize", "", buf) != 0)
				paperSize = buf;
			if (Ini.ProfileStr(sConfigFileName, printerName.latin1(), "DPI", "", buf) != 0)
				DPI = buf;

			if(paperSize.length())
			{
				paperSize = paperSize[0] + (paperSize.mid(1, 20)).lower();
				advValues += paperSize;
			}
			else
				advValues += "Default";

			if(DPI.length())
			{
				SplitString tmpDPI(buf, 'x');
				QString vals[2];

				tmpDPI.Next(vals[0]);
				tmpDPI.Next(vals[1]);

				if(vals[0].length())
				{
					if(vals[1].length())
					{
						if(vals[0] != vals[1])
							advValues += DPI;
						else
							advValues += vals[0];
					}
					else
						advValues += vals[0];
				}
			}
			else
				advValues += "Default";

			advValues += "Default";

			printer->setInfoFromWindows(printerType, device, netValues, advValues, model);
			return true;
		}
	}
	return false;

/*
	QString				sPrinterName,
								sTmp,
								sDefault;
	char					szBuf[256];
	SplitString		ssNames,
								sValues;
	PrnEntry*			pEntry;
	QString				sPaperSize,	
								sDPI,
								sHost,
								sShareName,
								sDevice;
	int						PrinterType;
								

	printf("config: %s\n", sConfigFileName.ascii());
	
	if (Ini.ProfileStr(sConfigFileName, "Printers", "Default", "", szBuf) != 0) sDefault = szBuf;
	else return FALSE;
	
	if (Ini.ProfileStr(sConfigFileName, "Printers", "Names", "", szBuf) != 0) ssNames.SetSectionsString(szBuf);
	else return FALSE;
	
	while (ssNames.Next(sPrinterName))
	{
		if (sPrinterName == sDefault)
		{
			int i = 0;
			int max = 0;
			int maxIndex = 0;

			getAllPrinters();
			while ((pEntry = pPrinter->getPrnEntry(i++)) != 0)
			{
				int res = advCmpStr(*pEntry->GetDescription(), sPrinterName);
				if (res >= max)
				{
					max = res;
					maxIndex = i - 1;
				}
			}
			
			if (max == 0) return FALSE;

			printf("Printer: Assuming that '%s' is '%s' (%d)\n",
						pPrinter->getPrnEntry(maxIndex)->GetDescription()->ascii(),
						sPrinterName.ascii(),
						max);
				
			PrinterType = PrinterCfg::Local;
			if (Ini.ProfileStr(sConfigFileName, sPrinterName.ascii(), "Type", "", szBuf) != 0)
			{
				sTmp = szBuf;
				if (sTmp == "Network") PrinterType = PrinterCfg::SMB;
			}
			
//			pPrinter->SetPrinter(maxIndex, PrinterType);

      Ini.ProfileStr(sConfigFileName, sPrinterName.ascii(), "Port", "LPT1:", szBuf);
      if (PrinterType == PrinterCfg::Local)
      {
      	sTmp = szBuf;
      	sTmp.truncate(3);
      	if (sTmp == "LPT")
      		sDevice.sprintf("/dev/lp%c (LPT%c)", szBuf[3] - 1, szBuf[3]);
      	else
      		sDevice.sprintf("/dev/ttyS%c (COM%c)", szBuf[3] - 1, szBuf[3]);
      	
      	pPrinter->SetLocalProperties(sDevice);
      }
      else
      {
      	sValues.SetSectionsString(&szBuf[2]);
      	sValues.SetSeparatorChar('\\');
      	sValues.Next(sHost);
      	sValues.Next(sShareName);
      	
      	pPrinter->SetSMBProperties(sHost, sShareName);
      }

			if (Ini.ProfileStr(sConfigFileName, sPrinterName.ascii(), "PaperSize", "", szBuf) != 0)
			{
				sPaperSize = szBuf;
			}
			if (Ini.ProfileStr(sConfigFileName, sPrinterName.ascii(), "DPI", "", szBuf) != 0)
			{
				sDPI = szBuf;
			}
			pPrinter->SetAdvancedOptions(sPaperSize, sDPI);

		}
	}
	return TRUE;
*/	

}
